@@ -2,7 +2,7 @@
 	Copyright (C) 2006 yopyop
 	Copyright (C) 2006-2007 Theo Berkau
 	Copyright (C) 2007 shash
-	Copyright (C) 2008-2013 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,21 +18,26 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
-#include <algorithm>
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <iostream>
-#include "MMU.h"
 #include "GPU.h"
+
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+#include <algorithm>
+#include <iostream>
+
+#include "MMU.h"
+#include "FIFO.h"
 #include "debug.h"
 #include "render3D.h"
+#include "registers.h"
 #include "gfx3d.h"
 #include "debug.h"
 #include "GPU_osd.h"
 #include "NDSSystem.h"
 #include "readwrite.h"
+#include "matrix.h"
+#include "emufile.h"
 
 #ifdef FASTBUILD
 	#undef FORCEINLINE
@@ -2200,10 +2205,20 @@ template<bool SKIP> static void GPU_RenderLine_DispCapture(u16 l)
 							case 0:			// Capture screen (BG + OBJ + 3D)
 								{
 									//INFO("Capture screen (BG + OBJ + 3D)\n");
-
-									u8 *src;
-									src = (u8*)(gpu->tempScanline);
+									u8 *src = (u8*)(gpu->tempScanline);
+#ifdef LOCAL_BE
+									static u16 swapSrc[256];
+									const size_t swapSrcSize = (gpu->dispCapCnt.capx == DISPCAPCNT::_128) ? 128 : 256;
+									
+									for(size_t i = 0; i < swapSrcSize; i++)
+									{
+										swapSrc[i] = LE_TO_LOCAL_16(((u16 *)src)[i]);
+									}
+									
+									CAPCOPY((u8 *)swapSrc,cap_dst,true);
+#else
 									CAPCOPY(src,cap_dst,true);
+#endif
 								}
 							break;
 							case 1:			// Capture 3D
@@ -2571,7 +2586,14 @@ void GPU_RenderLine(NDS_Screen * screen, u16 l, bool skip)
 			{
 				u8 * dst = GPU_screen + (screen->offset + l) * 512;
 				u8 * src = gpu->VRAMaddr + (l*512);
+#ifdef LOCAL_BE
+				for(size_t i = 0; i < 256; i++)
+				{
+					((u16 *)dst)[i] = LE_TO_LOCAL_16(((u16 *)src)[i]);
+				}
+#else
 				memcpy (dst, src, 512);
+#endif
 			}
 			break;
 		case 3: // Display memory FIFO
@@ -2722,6 +2744,16 @@ template<bool MOSAIC> void GPU::modeRender(int layer)
 	}
 }
 
+u32 GPU::getHOFS(int bg)
+{
+	return T1ReadWord(&dispx_st->dispx_BGxOFS[bg].BGxHOFS,0) & 0x1FF;
+}
+
+u32 GPU::getVOFS(int bg)
+{
+	return T1ReadWord(&dispx_st->dispx_BGxOFS[bg].BGxVOFS,0) & 0x1FF;
+}
+
 void gpu_SetRotateScreen(u16 angle)
 {
 	gpu_angle = angle;
