@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2006 yopyop
-	Copyright (C) 2006-2011 DeSmuME team
+	Copyright (C) 2006-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 */
 
 #include "GPU_osd.h"
+#include "driver.h"
 #include "GPU.h"
 #include "mem.h"
 #include <string.h> //mem funcs
@@ -552,7 +553,7 @@ void DrawHUD()
 		drawPad(Hud.GraphicalInputDisplay.x, Hud.GraphicalInputDisplay.y, 2.5);
 	}
 
-	#if defined(WIN32)
+	#if defined(WIN32) && !defined(DESMUME_QT)
 	if (CommonSettings.hud.ShowMicrophone) 
 	{
 		osd->addFixed(Hud.Microphone.x, Hud.Microphone.y, "%03d [%07d]",MicDisplay, Hud.cpuloopIterationCount);
