@@ -1,23 +1,20 @@
-/*	Copyright (C) 2006 yopyop
-
+/*
+	Copyright (C) 2006 yopyop
 	Copyright (C) 2007 shash
-	Copyright (C) 2007-2010 DeSmuME team
+	Copyright (C) 2007-2011 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 // this file is split from MMU.h for the purpose of avoiding ridiculous recompile times
@@ -29,6 +26,7 @@
 #include "MMU.h"
 #include "cp15.h"
 #include "readwrite.h"
+#include "debug.h"
 #include "NDSSystem.h"
 
 ////////////////////////////////////////////////////////////////
@@ -295,7 +293,7 @@ FORCEINLINE u32 _MMU_accesstime(u32 addr, bool sequential)
 #endif
 
 	// for now, assume the cache is always enabled for all of main memory
-	if(TIMING && PROCNUM==ARMCPU_ARM9 && (addr & 0x0F000000) == 0x02000000)
+	if(AT != MMU_AT_DMA && TIMING && PROCNUM==ARMCPU_ARM9 && (addr & 0x0F000000) == 0x02000000)
 	{
 #ifdef ENABLE_CACHE_CONTROLLER_EMULATION
 		bool cached = false;
@@ -318,6 +316,14 @@ FORCEINLINE u32 _MMU_accesstime(u32 addr, bool sequential)
 			// by reading 32 bytes...
 			c += 8 * M32*2;
 		}
+
+		if(CheckDebugEvent(DEBUG_EVENT_CACHE_MISS))
+		{
+			DebugEventData.addr = addr;
+			DebugEventData.size = READSIZE;
+			HandleDebugEvent(DEBUG_EVENT_CACHE_MISS);
+		}
+
 		return c;
 #elif defined(ACCOUNT_FOR_NON_SEQUENTIAL_ACCESS)
 		// this is the closest approximation I could find
