@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2013 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -16,34 +16,48 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "NDSSystem.h"
+
 #include <string.h>
 #include <stdlib.h>
 #include <algorithm>
 #include <math.h>
 #include <zlib.h>
 
+#include "utils/decrypt/decrypt.h"
+#include "utils/decrypt/crc.h"
+#include "utils/advanscene.h"
+#include "utils/task.h"
+
 #include "common.h"
-#include "NDSSystem.h"
+#include "armcpu.h"
 #include "render3D.h"
 #include "MMU.h"
 #include "ROMReader.h"
 #include "gfx3d.h"
-#include "utils/decrypt/decrypt.h"
-#include "utils/decrypt/crc.h"
-#include "utils/advanscene.h"
+#include "GPU.h"
 #include "cp15.h"
 #include "bios.h"
 #include "debug.h"
 #include "cheatSystem.h"
 #include "movie.h"
 #include "Disassembler.h"
+#include "FIFO.h"
 #include "readwrite.h"
+#include "registers.h"
 #include "debug.h"
+#include "driver.h"
 #include "firmware.h"
 #include "version.h"
 #include "path.h"
 #include "slot1.h"
 #include "slot2.h"
+#include "SPU.h"
+#include "wifi.h"
+
+#ifdef GDB_STUB
+#include "gdbstub.h"
+#endif
 
 //int xxctr=0;
 //#define LOG_ARM9
@@ -100,14 +114,31 @@ void Desmume_InitOnce()
 #endif
 }
 
-#ifdef GDB_STUB
-int NDS_Init( struct armcpu_memory_iface *arm9_mem_if,
-struct armcpu_ctrl_iface **arm9_ctrl_iface,
-struct armcpu_memory_iface *arm7_mem_if,
-struct armcpu_ctrl_iface **arm7_ctrl_iface)
-#else
-int NDS_Init( void)
-#endif
+int NDS_GetCPUCoreCount()
+{
+	return getOnlineCores();
+}
+
+void NDS_SetupDefaultFirmware()
+{
+	NDS_FillDefaultFirmwareConfigData(&CommonSettings.fw_config);
+}
+
+void NDS_RunAdvansceneAutoImport()
+{
+	if(CommonSettings.run_advanscene_import != "")
+	{
+		std::string fname = CommonSettings.run_advanscene_import;
+		std::string fname_out = fname + ".ddb";
+		EMUFILE_FILE outf(fname_out,"wb");
+		u32 ret = advsc.convertDB(fname.c_str(),&outf);
+		if(ret == 0)
+			exit(0);
+		else exit(1);
+	}
+}
+
+int NDS_Init()
 {
 	nds.idleFrameCounter = 0;
 	memset(nds.runCycleCollector,0,sizeof(nds.runCycleCollector));
@@ -119,16 +150,29 @@ int NDS_Init( void)
 	if (Screen_Init() != 0)
 		return -1;
 
+	{
+		char	buf[MAX_PATH];
+		memset(buf, 0, MAX_PATH);
+		strcpy(buf, path.pathToModule);
+		strcat(buf, "desmume.ddb");							// DeSmuME database	:)
+		advsc.setDatabase(buf);
+
+		//why is this done here? shitty engineering. not intended.
+		NDS_RunAdvansceneAutoImport();
+	}
+
 	gfx3d_init();
 
-#ifdef GDB_STUB
-	armcpu_new(&NDS_ARM7,1, arm7_mem_if, arm7_ctrl_iface);
-	armcpu_new(&NDS_ARM9,0, arm9_mem_if, arm9_ctrl_iface);
-#else
-	armcpu_new(&NDS_ARM7,1);
 	armcpu_new(&NDS_ARM9,0);
-#endif
-
+	NDS_ARM9.SetBaseMemoryInterface(&arm9_base_memory_iface);
+	NDS_ARM9.SetBaseMemoryInterfaceData(NULL);
+	NDS_ARM9.ResetMemoryInterfaceToBase();
+	
+	armcpu_new(&NDS_ARM7,1);
+	NDS_ARM7.SetBaseMemoryInterface(&arm7_base_memory_iface);
+	NDS_ARM7.SetBaseMemoryInterfaceData(NULL);
+	NDS_ARM7.ResetMemoryInterfaceToBase();
+	
 	if (SPU_Init(SNDCORE_DUMMY, 740) != 0)
 		return -1;
 
@@ -177,63 +221,12 @@ void NDS_DeInit(void)
 #endif
 }
 
-NDS_header * NDS_getROMHeader(void)
+NDS_header* NDS_getROMHeader(void)
 {
-	NDS_header * header = new NDS_header;
-
-	memcpy(header, &gameInfo.header, sizeof(gameInfo.header));
-
-	//endian swap necessary fields. It would be better if we made accessors for these. I wonder if you could make a macro for a field accessor that would take the bitsize and do the swap on the fly
-	struct FieldSwap {
-		size_t offset;
-		int bytes;
-	};
-
-	static const FieldSwap fieldSwaps[] = {
-		{ offsetof(NDS_header,makerCode), 2},
-
-		{ offsetof(NDS_header,ARM9src), 4},
-		{ offsetof(NDS_header,ARM9exe), 4},
-		{ offsetof(NDS_header,ARM9cpy), 4},
-		{ offsetof(NDS_header,ARM7src), 4},
-		{ offsetof(NDS_header,ARM7exe), 4},
-		{ offsetof(NDS_header,ARM7cpy), 4},
-		{ offsetof(NDS_header,ARM7binSize), 4},
-		{ offsetof(NDS_header,FNameTblOff), 4},
-		{ offsetof(NDS_header,FNameTblSize), 4},
-		{ offsetof(NDS_header,FATOff), 4},
-		{ offsetof(NDS_header,FATSize), 4},
-		{ offsetof(NDS_header,ARM9OverlayOff), 4},
-		{ offsetof(NDS_header,ARM9OverlaySize), 4},
-		{ offsetof(NDS_header,ARM7OverlayOff), 4},
-		{ offsetof(NDS_header,ARM7OverlaySize), 4},
-		{ offsetof(NDS_header,normalCmd), 4},
-		{ offsetof(NDS_header,Key1Cmd), 4},
-		{ offsetof(NDS_header,IconOff), 4},
-
-		{ offsetof(NDS_header,CRC16), 2},
-		{ offsetof(NDS_header,ROMtimeout), 2},
-		
-		{ offsetof(NDS_header,ARM9autoload), 4},
-		{ offsetof(NDS_header,ARM7autoload), 4},
-		{ offsetof(NDS_header,endROMoffset), 4},
-		{ offsetof(NDS_header,HeaderSize), 4},
-
-		{ offsetof(NDS_header, ARM9module), 4},
-		{ offsetof(NDS_header, ARM7module), 4},
-
-		{ offsetof(NDS_header,logoCRC16), 2},
-		{ offsetof(NDS_header,headerCRC16), 2},
-	};
-
-	for(int i=0;i<ARRAY_SIZE(fieldSwaps);i++)
-		switch(fieldSwaps[i].bytes)
-		{
-		case 2: HostWriteWord((u8*)header,fieldSwaps[i].offset,T1ReadWord(header,fieldSwaps[i].offset));
-		case 4: HostWriteLong((u8*)header,fieldSwaps[i].offset,T1ReadLong((u8*)header,fieldSwaps[i].offset));
-		}
-
-	return header;
+	NDS_header *newHeader = new NDS_header;
+	memcpy(newHeader, &gameInfo.header, sizeof(NDS_header));
+	
+	return newHeader;
 } 
 
 
@@ -314,29 +307,28 @@ const RomBanner& GameInfo::getRomBanner()
 
 void GameInfo::populate()
 {
-	const char *regions[] = {	"JPFSEODIRKHXWVU",
-								"JPN",		// J
-								"EUR",		// P
-								"FRA",		// F
-								"ESP",		// S
-								"USA",		// E
-								"INT",		// O
-								"NOE",		// D
-								"ITA",		// I
-								"RUS",		// R
-								"KOR",		// K
-								"HOL",		// H
-								"EUU",		// X
-								"EUU",		// V
-								"EUU",		// W
-								"AUS",		// U
+	const char regions_index[] = "JPFSEODIRKHXVWUC";
+	const char *regions[] = {
+					"???",
+					"JPN",		// J
+					"EUR",		// P
+					"FRA",		// F
+					"ESP",		// S
+					"USA",		// E
+					"INT",		// O
+					"NOE",		// D
+					"ITA",		// I
+					"RUS",		// R
+					"KOR",		// K
+					"HOL",		// H
+					"EUU",		// X
+					"EUU",		// V
+					"EUU",		// W
+					"AUS",		// U
+					"CHN",		// C
 
 	};
-
-	NDS_header * _header = NDS_getROMHeader();
-	header = *_header;
-	delete _header;
-
+	
 	memset(ROMserial, 0, sizeof(ROMserial));
 	memset(ROMname, 0, sizeof(ROMname));
 
@@ -354,12 +346,12 @@ void GameInfo::populate()
 		memcpy(ROMserial+4, header.gameCode, 4);
 
 		u32 regions_num = ARRAY_SIZE(regions);
-		u32 region = (u32)(std::max<s32>(strchr(regions[0],header.gameCode[3]) - regions[0] + 1, 0));
+		u32 region = (u32)(std::max<s32>(strchr(regions_index,header.gameCode[3]) - regions_index + 1, 0));
 
 		if (region < regions_num)
-			strcat(ROMserial, regions[region]);
+			strncat(ROMserial, regions[region], sizeof(ROMserial));
 		else
-			strcat(ROMserial, "Unknown");
+			strncat(ROMserial, "???", sizeof(ROMserial));
 	}
 
 	//rom name is probably set even in homebrew, so do it regardless
@@ -376,8 +368,6 @@ void GameInfo::populate()
 				trim(ROMfullName[i]);
 			}
 		}*/
-
-
 }
 
 bool GameInfo::loadROM(std::string fname, u32 type)
@@ -398,7 +388,84 @@ bool GameInfo::loadROM(std::string fname, u32 type)
 
 	if (res)
 	{
+#ifndef LOCAL_LE
+		//endian swap necessary fields. It would be better if we made accessors for these. I wonder if you could make a macro for a field accessor that would take the bitsize and do the swap on the fly
+		struct FieldSwap {
+			const size_t offset;
+			const size_t bytes;
+		};
+		
+		static const FieldSwap fieldSwaps[] = {
+			{ offsetof(NDS_header,makerCode), 2},
+			
+			{ offsetof(NDS_header,ARM9src), 4},
+			{ offsetof(NDS_header,ARM9exe), 4},
+			{ offsetof(NDS_header,ARM9cpy), 4},
+			{ offsetof(NDS_header,ARM9binSize), 4},
+			{ offsetof(NDS_header,ARM7src), 4},
+			{ offsetof(NDS_header,ARM7exe), 4},
+			{ offsetof(NDS_header,ARM7cpy), 4},
+			{ offsetof(NDS_header,ARM7binSize), 4},
+			{ offsetof(NDS_header,FNameTblOff), 4},
+			{ offsetof(NDS_header,FNameTblSize), 4},
+			{ offsetof(NDS_header,FATOff), 4},
+			{ offsetof(NDS_header,FATSize), 4},
+			{ offsetof(NDS_header,ARM9OverlayOff), 4},
+			{ offsetof(NDS_header,ARM9OverlaySize), 4},
+			{ offsetof(NDS_header,ARM7OverlayOff), 4},
+			{ offsetof(NDS_header,ARM7OverlaySize), 4},
+			{ offsetof(NDS_header,normalCmd), 4},
+			{ offsetof(NDS_header,Key1Cmd), 4},
+			{ offsetof(NDS_header,IconOff), 4},
+			
+			{ offsetof(NDS_header,CRC16), 2},
+			{ offsetof(NDS_header,ROMtimeout), 2},
+			
+			{ offsetof(NDS_header,ARM9autoload), 4},
+			{ offsetof(NDS_header,ARM7autoload), 4},
+			{ offsetof(NDS_header,endROMoffset), 4},
+			{ offsetof(NDS_header,HeaderSize), 4},
+			
+			{ offsetof(NDS_header, ARM9module), 4},
+			{ offsetof(NDS_header, ARM7module), 4},
+			
+			{ offsetof(NDS_header,logoCRC16), 2},
+			{ offsetof(NDS_header,headerCRC16), 2},
+		};
+		
+		for(size_t i = 0; i < ARRAY_SIZE(fieldSwaps); i++)
+		{
+			const u8 *fieldAddr = (u8 *)&header + fieldSwaps[i].offset;
+			
+			switch(fieldSwaps[i].bytes)
+			{
+				case 2:
+					*(u16 *)fieldAddr = LE_TO_LOCAL_16(*(u16 *)fieldAddr);
+					break;
+					
+				case 4:
+					*(u32 *)fieldAddr = LE_TO_LOCAL_32(*(u32 *)fieldAddr);
+					break;
+			}
+		}
+#endif
 		cardSize = (128 * 1024) << header.cardSize;
+
+		if (cardSize < romsize)
+		{
+			msgbox->warn("The ROM header is invalid.\nThe device size has been increased to allow for the provided file size.\n");
+			
+			for (u32 i = header.cardSize; i < 0xF; i++)
+			{
+				if (((128 * 1024) << i) >= romsize) 
+				{
+					header.cardSize = i;
+					cardSize = (128 * 1024) << i;
+					break;
+				}
+			}
+		}
+		
 		mask = (cardSize - 1);
 		mask |= (mask >>1);
 		mask |= (mask >>2);
@@ -426,9 +493,19 @@ bool GameInfo::loadROM(std::string fname, u32 type)
 			}
 
 			if(hasRomBanner())
+			{
 				memcpy(&banner, romdata + header.IconOff, sizeof(RomBanner));
+				
+				banner.version = LE_TO_LOCAL_16(banner.version);
+				banner.crc16 = LE_TO_LOCAL_16(banner.crc16);
+				
+				for(size_t i = 0; i < ARRAY_SIZE(banner.palette); i++)
+				{
+					banner.palette[i] = LE_TO_LOCAL_16(banner.palette[i]);
+				}
+			}
 
-			_isDSiEnhanced = ((*(u32*)(romdata + 0x180) == 0x8D898581U) && (*(u32*)(romdata + 0x184) == 0x8C888480U));
+			_isDSiEnhanced = (LE_TO_LOCAL_32(*(u32*)(romdata + 0x180) == 0x8D898581U) && LE_TO_LOCAL_32(*(u32*)(romdata + 0x184) == 0x8C888480U));
 			fclose(fROM); fROM = NULL;
 			return true;
 		}
@@ -437,6 +514,14 @@ bool GameInfo::loadROM(std::string fname, u32 type)
 		{
 			fseek(fROM, header.IconOff + headerOffset, SEEK_SET);
 			fread(&banner, 1, sizeof(RomBanner), fROM);
+			
+			banner.version = LE_TO_LOCAL_16(banner.version);
+			banner.crc16 = LE_TO_LOCAL_16(banner.crc16);
+			
+			for(size_t i = 0; i < ARRAY_SIZE(banner.palette); i++)
+			{
+				banner.palette[i] = LE_TO_LOCAL_16(banner.palette[i]);
+			}
 		}
 		fseek(fROM, headerOffset, SEEK_SET);
 		lastReadPos = 0;
@@ -471,10 +556,27 @@ u32 GameInfo::readROM(u32 pos)
 			fseek(fROM, pos + headerOffset, SEEK_SET);
 		u32 num = fread(&data, 1, 4, fROM);
 		lastReadPos = (pos + num);
-		return data;
+		return LE_TO_LOCAL_32(data);
 	}
 	else
-		return *(u32*)(romdata + pos);
+	{
+		if(pos + 4 > romsize)
+		{
+			printf("Panic! GameInfo reading out of buffer!\n");
+			exit(-1);
+		}
+		return LE_TO_LOCAL_32(*(u32*)(romdata + pos));
+	}
+}
+
+bool GameInfo::isDSiEnhanced()
+{
+	return _isDSiEnhanced;
+}
+
+bool GameInfo::isHomebrew()
+{
+	return ((header.ARM9src < 0x4000) && (T1ReadLong(header.logo, 0) != 0x51AEFF24) && (T1ReadLong(header.logo, 4) != 0x699AA221));
 }
 
 static int rom_init_path(const char *filename, const char *physicalName, const char *logicalFilename)
@@ -493,7 +595,7 @@ static int rom_init_path(const char *filename, const char *physicalName, const c
 		//...but since the data was extracted to gameInfo then it is ok
 	}
 	//ds.gba in archives, it's already been loaded into memory at this point
-	else if (path.isdsgba(std::string(logicalFilename))) {
+	else if (logicalFilename && path.isdsgba(std::string(logicalFilename))) {
 		type = ROM_DSGBA;
 	} else {
 		//well, try to load it as an nds rom anyway
@@ -576,10 +678,6 @@ int NDS_LoadROM(const char *filename, const char *physicalName, const char *logi
 	}
 	INFO("ROM developer: %s\n", ((gameInfo.header.makerCode == 0) && gameInfo.isHomebrew())?"Homebrew":getDeveloperNameByID(gameInfo.header.makerCode).c_str());
 
-	memset(buf, 0, MAX_PATH);
-	strcpy(buf, path.pathToModule);
-	strcat(buf, "desmume.ddb");							// DeSmuME database	:)
-	advsc.setDatabase(buf);
 	buf[0] = gameInfo.header.gameCode[0];
 	buf[1] = gameInfo.header.gameCode[1];
 	buf[2] = gameInfo.header.gameCode[2];
@@ -607,8 +705,10 @@ int NDS_LoadROM(const char *filename, const char *physicalName, const char *logi
 	printf("\n");
 
 	//for homebrew, try auto-patching DLDI. should be benign if there is no DLDI or if it fails
-	if(gameInfo.isHomebrew() && CommonSettings.loadToMemory)
+	if(gameInfo.isHomebrew())
 	{
+		if(!CommonSettings.loadToMemory)
+			msgbox->warn("Sorry.. right now, you can't use the default (stream rom from disk) with homebrew due to a bug with DLDI-autopatching");
 		if (slot1_GetCurrentType() == NDS_SLOT1_R4)
 			DLDI::tryPatch((void*)gameInfo.romdata, gameInfo.romsize, 1);
 		else
@@ -1732,6 +1832,10 @@ void NDS_debug_step()
 template<bool FORCE>
 void NDS_exec(s32 nb)
 {
+	#ifdef GDB_STUB
+	gdbstub_mutex_lock();
+	#endif
+
 	LagFrameFlag=1;
 
 	sequencer.nds_vblankEnded = false;
@@ -1758,10 +1862,23 @@ void NDS_exec(s32 nb)
 			#ifdef DEVELOPER
 				singleStep = false;
 				//(gdb stub doesnt yet know how to trigger these immediately by calling reschedule)
-				while((NDS_ARM9.stalled || NDS_ARM7.stalled) && execute)
+				if ((NDS_ARM9.stalled || NDS_ARM7.stalled) && execute)
 				{
-					driver->EMU_DebugIdleUpdate();
-					nds_debug_continuing[0] = nds_debug_continuing[1] = true;
+					driver->EMU_DebugIdleEnter();
+					
+					while((NDS_ARM9.stalled || NDS_ARM7.stalled) && execute)
+					{
+					        #ifdef GDB_STUB
+					        gdbstub_mutex_unlock();
+					        #endif
+						driver->EMU_DebugIdleUpdate();
+					        #ifdef GDB_STUB
+					        gdbstub_mutex_lock();
+					        #endif
+						nds_debug_continuing[0] = nds_debug_continuing[1] = true;
+					}
+					
+					driver->EMU_DebugIdleWakeUp();
 				}
 			#endif
 
@@ -1858,6 +1975,10 @@ void NDS_exec(s32 nb)
 	DEBUG_Notify.NextFrame();
 	if (cheats)
 		cheats->process();
+
+        #ifdef GDB_STUB
+        gdbstub_mutex_unlock();
+        #endif
 }
 
 template<int PROCNUM> static void execHardware_interrupts_core()
@@ -2175,7 +2296,7 @@ bool NDS_FakeBoot()
 		u32 dst = header->ARM9cpy;
 		for(u32 i = 0; i < header->ARM9binSize; i+=4)
 		{
-			u32 tmp = (hasSecureArea && ((src >= 0x4000) && (src < 0x8000)))?*(u32*)(gameInfo.secureArea + (src - 0x4000)):gameInfo.readROM(src);
+			u32 tmp = (hasSecureArea && ((src >= 0x4000) && (src < 0x8000)))?LE_TO_LOCAL_32(*(u32*)(gameInfo.secureArea + (src - 0x4000))):gameInfo.readROM(src);
 
 			_MMU_write32<ARMCPU_ARM9>(dst, tmp);
 
@@ -2215,12 +2336,12 @@ bool NDS_FakeBoot()
 	{
 		//dsi needs this copied later in memory. there are probably a number of things that  get copied to a later location in memory.. thats where the NDS consoles tend to stash stuff.
 		for (int i = 0; i < (0x170); i+=4)
-			_MMU_write32<ARMCPU_ARM9>(0x027FFE00 + i, LE_TO_LOCAL_32(gameInfo.readROM(i)));
+			_MMU_write32<ARMCPU_ARM9>(0x027FFE00 + i, gameInfo.readROM(i));
 	}
 	else
 	{
 		for (int i = 0; i < (0x170); i+=4)
-			_MMU_write32<ARMCPU_ARM9>(0x027FFE00 + i, LE_TO_LOCAL_32(gameInfo.readROM(i)));
+			_MMU_write32<ARMCPU_ARM9>(0x027FFE00 + i, gameInfo.readROM(i));
 	}
 
 	//the firmware will be booting to these entrypoint addresses via BX (well, the arm9 at least; is unverified for the arm7)
@@ -2288,7 +2409,7 @@ bool NDS_FakeBoot()
 		_MMU_write08<ARMCPU_ARM9>(kCommandline+i, rompath[i]);
 	_MMU_write08<ARMCPU_ARM9>(kCommandline+rompath.size(), 0);
 	//--------------------------------
-    
+
 	//Call the card post_fakeboot hook to perform additional initialization
 	slot1_device->post_fakeboot(ARMCPU_ARM9);
 	slot1_device->post_fakeboot(ARMCPU_ARM7);
@@ -2394,7 +2515,7 @@ void NDS_Reset()
 	memcpy(FW_Mac, (MMU.fw.data + 0x36), 6);
 
 	SPU_DeInit();
-	SPU_ReInit();
+	SPU_ReInit(!canBootFromFirmware && bootResult);
 
 	//this needs to happen last, pretty much, since it establishes the correct scheduling state based on all of the above initialization
 	initSchedule();
@@ -2755,6 +2876,13 @@ void NDS_suspendProcessingInput(bool suspend)
 	}
 }
 
+void NDS_swapScreen()
+{
+	u16 tmp = MainScreen.offset;
+	MainScreen.offset = SubScreen.offset;
+	SubScreen.offset = tmp;
+}
+
 
 void emu_halt() {
 	//printf("halting emu: ARM9 PC=%08X/%08X, ARM7 PC=%08X/%08X\n", NDS_ARM9.R[15], NDS_ARM9.instruct_adr, NDS_ARM7.R[15], NDS_ARM7.instruct_adr);
