@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2010-2013 DeSmuME team
+	Copyright (C) 2010-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -21,13 +21,13 @@
 //									NTR-UORE-0
 //									- 128Mbit
 
-#include "../slot1.h"
-#include "../registers.h"
-#include "../MMU.h"
-#include "../NDSSystem.h"
 #include "slot1comp_rom.h"
 #include "slot1comp_protocol.h"
 
+#include "../slot1.h"
+#include "../NDSSystem.h"
+#include "../emufile.h"
+
 //quick architecture overview:
 //NAND receives GC bus commands from MMU.cpp
 //those are passed on to the protocol component for parsing
@@ -45,8 +45,15 @@ private:
 
 	u32 mode;
 	u32 handle_save;
+
+	//current NAND read/write start position
+	//when this is changed, the read/write cursor will be reset to it
+	//when it is set to the same value, the read/write cursor will NOT be reset
+	//(this is since some value must necessarily come in on the protocol address, so the 'current save_start' is used as a special 'dont change' value
+	u32 save_start;
+
+	//current NAND read/write cursor
 	u32 save_adr;
-	u32 save_start_adr;
 
 public:
 	virtual Slot1Info const* info()
@@ -61,7 +68,7 @@ public:
 		protocol.chipId = gameInfo.chipID;
 		protocol.gameCode = T1ReadLong((u8*)gameInfo.header.gameCode,0);
 
-		save_start_adr = 0;
+		save_adr = 0;
 		handle_save = 0;
 		mode = 0;
 		subAdr = T1ReadWord(gameInfo.header.reserved2, 0x6) << 17;
@@ -83,19 +90,21 @@ public:
 
 	virtual void slot1client_startOperation(eSlot1Operation operation)
 	{
-		protocol.address = (protocol.command.bytes[1] << 24) | (protocol.command.bytes[2] << 16) | (protocol.command.bytes[3] << 8) | protocol.command.bytes[4];
-
 		//INFO("Start command: %02X%02X%02X%02X%02X%02X%02X%02X\t",
 		//	protocol.command.bytes[0], protocol.command.bytes[1], protocol.command.bytes[2], protocol.command.bytes[3],
 		//	protocol.command.bytes[4], protocol.command.bytes[5], protocol.command.bytes[6], protocol.command.bytes[7]);
 		//INFO("FROM: %08X\n", NDS_ARM9.instruct_adr);
 
+		u32 addressFromProtocol = (protocol.command.bytes[1] << 24) | (protocol.command.bytes[2] << 16) | (protocol.command.bytes[3] << 8) | protocol.command.bytes[4];
+
 		//pass the normal rom operations along to the rom component
 		switch(operation)
 		{
 			case eSlot1Operation_00_ReadHeader_Unencrypted:
+				rom.start(operation,addressFromProtocol);
+				return;
+
 			case eSlot1Operation_2x_SecureAreaLoad:
-			//case eSlot1Operation_B7_Read:
 				rom.start(operation,protocol.address);
 				return;
 		}
@@ -116,10 +125,10 @@ public:
 			//Nand Write Page
 			case 0x81:
 				mode = cmd;
-				if (save_start_adr != (protocol.address & gameInfo.mask) - subAdr)
+				if(addressFromProtocol != save_start)
 				{
-					save_start_adr = save_adr = (protocol.address & gameInfo.mask) - subAdr;
-					MMU_new.backupDevice.seek(save_start_adr);
+					save_start = addressFromProtocol;
+					save_adr = (addressFromProtocol & gameInfo.mask) - subAdr;
 				}
 				handle_save = 1;
 				break;
@@ -134,26 +143,29 @@ public:
 				MMU_new.backupDevice.flushBackup();
 				break;
 
+			//Start read mode
 			case 0xB7:
 				if (handle_save)
 				{
 					mode = cmd;
-					if (save_start_adr != (protocol.address & gameInfo.mask) - subAdr)
+					if(addressFromProtocol != save_start)
 					{
-						save_start_adr = save_adr = (protocol.address & gameInfo.mask) - subAdr;
-						MMU_new.backupDevice.seek(save_start_adr);
+						save_start = addressFromProtocol;
+						save_adr = (addressFromProtocol & gameInfo.mask) - subAdr;
 					}
 				}
 				else
 				{
-					rom.start(operation, protocol.address);
+					rom.start(operation, addressFromProtocol);
 				}
 				break;
 
 			case 0xB2: //Set save position
 				mode = cmd;
-				save_start_adr = save_adr = (protocol.address & gameInfo.mask) - subAdr;
-				MMU_new.backupDevice.seek(save_start_adr);
+				save_start = addressFromProtocol;
+				//cursor resets regardless of whether save_start changed, that's what makes this special.
+				//the cursor could be reset to the beginning of the previous save_start region
+				save_adr = (addressFromProtocol & gameInfo.mask) - subAdr;
 				handle_save = 1;
 				break;
 		}
@@ -258,8 +270,8 @@ public:
 
 	virtual void post_fakeboot(int PROCNUM)
 	{
-	    // The BIOS leaves the card in NORMAL mode
-	    protocol.mode = eCardMode_NORMAL;
+		// The BIOS leaves the card in NORMAL mode
+		protocol.mode = eCardMode_NORMAL;
 	}
 
 	virtual void savestate(EMUFILE* os)
@@ -274,7 +286,7 @@ public:
 		os->write32le(mode);
 		os->write32le(handle_save);
 		os->write32le(save_adr);
-		os->write32le(save_start_adr);
+		os->write32le(save_start);
 		os->write32le(subAdr);
 	}
 
@@ -293,7 +305,7 @@ public:
 			is->read32le(&mode);
 			is->read32le(&handle_save);
 			is->read32le(&save_adr);
-			is->read32le(&save_start_adr);
+			is->read32le(&save_start);
 			is->read32le(&subAdr);
 		}
 	}
