@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,18 +18,24 @@
 //this file contains the components used for emulating standard gamecard "MC" devices (eeprom, fram, flash)
 //this is largely done by accessing the BackupDevice resources in the core emulator
 
-#include "types.h"
-#include "../MMU.h"
 #include "slot1comp_mc.h"
 
+#include "../MMU.h"
+#include "../NDSSystem.h"
+
 Slot1Comp_MC g_Slot1Comp_MC;
 
 u8 Slot1Comp_MC::auxspi_transaction(int PROCNUM, u8 value)
 {
 	return MMU_new.backupDevice.data_command(value, PROCNUM);
 }
+
 void Slot1Comp_MC::auxspi_reset(int PROCNUM)
 {
 	MMU_new.backupDevice.reset_command();
 }
 
+void Slot1Comp_MC::connect()
+{
+	if(!memcmp(gameInfo.header.gameCode,"AXBJ", 4)) MMU_new.backupDevice.uninitializedValue = 0x00; // Daigassou! Band Brothers DX (JP)
+}
\ No newline at end of file
