@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2009 CrazyMax
-	Copyright (C) 2009-2013 DeSmuME team
+	Copyright (C) 2009-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -16,12 +16,14 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "../slot2.h"
+
 #include <string.h>
-#include "../mem.h"
-#include "../MMU.h"
+
+#include "../debug.h"
 #include "../NDSSystem.h"
 #include "../path.h"
-#include "../slot2.h"
+#include "../emufile.h"
 
 #define EEPROM		0x52504545
 #define SRAM_		0x4D415253
@@ -40,12 +42,10 @@ static const char *saveTypes[] = {
 class Slot2_GbaCart : public ISlot2Interface
 {
 private:
-	FILE	*fROM;
-	FILE	*fSRAM;
+	EMUFILE* fROM;
+	EMUFILE* fSRAM;
 	u32		romSize;
-	u32		currentROMPos;
 	u32		sramSize;
-	u32		currentSRAMPos;
 	u32		saveType;
 
 	struct 
@@ -60,72 +60,85 @@ private:
 
 	u32	readRom(const u32 pos, const u8 size) 
 	{
-		if (fROM)
-		{
-			if (currentROMPos != pos)
-				fseek(fROM, pos, SEEK_SET);
-
-			u32 data = 0xFFFFFFFF;
-			u32 readed = fread(&data, 1, size, fROM);
-			currentROMPos = (pos + readed);
-			return data;
-		}
-		return 0xFFFFFFFF;
+		if (!fROM) return 0xFFFFFFFF;
+		
+		fROM->fseek(pos, SEEK_SET);
+	
+		u32 data = 0xFFFFFFFF;
+		u32 readed = fROM->fread(&data, size);
+		return data;
 	}
 
 	u32 readSRAM(const u32 pos, const u8 size) 
 	{
-		if (fSRAM)
-		{
-			if (currentSRAMPos != pos)
-				fseek(fSRAM, pos, SEEK_SET);
+		if (!fSRAM)
+			return 0xFFFFFFFF;
+			
+		fSRAM->fseek(pos, SEEK_SET);
 
-			u32 data = 0xFFFFFFFF;
-			u32 readed = fread(&data, 1, size, fSRAM);
-			currentSRAMPos = (pos + readed);
-			return data;
-		}
-		return 0xFFFFFFFF;
+		u32 data = 0xFFFFFFFF;
+		u32 readed = fSRAM->fread(&data, size);
+		return data;
 	}
 
 	void writeSRAM(const u32 pos, const u8 *data, u32 size) 
 	{
-		if (fSRAM)
-		{
-			if (currentSRAMPos != pos)
-				fseek(fSRAM, pos, SEEK_SET);
+		if (!fSRAM)
+			return;
 
-			u32 writed = fwrite(&data, 1, size, fSRAM);
-			currentSRAMPos = (pos + writed);
-		}
+		fSRAM->fseek(pos, SEEK_SET);
+
+		u32 writed = size;
+		fSRAM->fwrite(data, size);
+		fSRAM->fflush();
 	}
 
 
-	u32 getSaveTypeGBA()
+	u32 scanSaveTypeGBA()
 	{
 		if (!fROM) return 0xFF;
 
-		u32 saveROMPos = currentROMPos;
-		u32 tmp = 0;
+		fROM->fseek(0, SEEK_SET);
+		int size = fROM->size();
 
-		fseek(fROM, 0, SEEK_SET);
+		int lastpct=1;
 
-		while (!feof(fROM))
+		int len = fROM->size();
+		for(;;)
 		{
-			u32 readed = fread(&tmp, 1, 4, fROM);
-			if (readed < 4) break;
+			u32 tmp;
+			u32 readed = fROM->fread(&tmp, 4);
+
+			int pos = fROM->ftell();
+			int currPct = pos*100/(size-1);
+			for(int i=lastpct;i<currPct;i++)
+			{
+				if(i%10==0)
+					printf(" %d%%\n",i/10*10);
+				else printf(".");
+				lastpct = currPct;
+			}
+
+			if (readed < 4) 
+				break;
+
+			if(pos >= len)
+				break;
+
 
 			switch (tmp)
 			{
-				case EEPROM:	fseek(fROM, saveROMPos, SEEK_SET); return 1;
-				case SRAM_:		fseek(fROM, saveROMPos, SEEK_SET); return 2;
+				case EEPROM:
+					return 1;
+				case SRAM_:
+					return 2;
 				case FLASH:
 				{
-					fread(&tmp, 1, 4, fROM);
-					fseek(fROM, saveROMPos, SEEK_SET); 
+					u32 tmp = fROM->read32le();
 					return ((tmp == FLASH1M_)?3:5);
 				}
-				case SIIRTC_V:	 fseek(fROM, saveROMPos, SEEK_SET); return 4;
+				case SIIRTC_V:
+					return 4;
 			}
 		}
 
@@ -223,7 +236,7 @@ private:
 			case 0x82:
 				if (val == 0x30)
 				{
-					u32 ofs = (adr & 0x0000F000);
+					u32 ofs = (adr & 0x0000F000) + (0x10000 * gbaFlash.bank);
 					//INFO("GBAgame: Flash: erase from 0x%08X to 0x%08X\n", ofs + 0x0A000000, ofs + 0x0A001000);
 					u8 *tmp = new u8[0x1000];
 					memset(tmp, 0xFF, 0x1000);
@@ -300,54 +313,81 @@ private:
 		return 0xFF;
 	}
 
+	void Close()
+	{
+		delete fROM; fROM = NULL;
+		delete fSRAM; fSRAM = NULL;
+		romSize = 0;
+		sramSize = 0;
+	}
+
 public:
+	Slot2_GbaCart()
+		: fROM(NULL)
+		, fSRAM(NULL)
+	{
+		Close();
+	}
+
 	virtual Slot2Info const* info()
 	{
-		static Slot2InfoSimple info("GBA cartridge", "GBA cartridge in slot", 0x03);
+		static Slot2InfoSimple info("GBA Cartridge", "GBA cartridge in slot", 0x03);
 		return &info;
 	}
 
 	virtual void connect()
 	{
-		if (fROM) fclose(fROM);
-		if (fSRAM) fclose(fSRAM);
-		fROM = NULL; fSRAM = NULL;
+		Close();
 		romSize = 0;
-		currentROMPos = 0;
 		sramSize = 0;
-		currentSRAMPos = 0;
-
-		if (gameInfo.romsize == 0) return;
-
-		if (!strcasecmp(GBAgameName, "self"))
-			strcpy(GBAgameName, path.path.c_str());
-
-		if (!strlen(GBAgameName)) return;
 		
-		printf("GBASlot opening ROM %s", GBAgameName);
-
-		fROM = fopen(GBAgameName, "rb");
-		if (fROM)
+		if (gameInfo.romsize == 0)
 		{
-			fseek(fROM, 0, SEEK_END);
-			romSize = ftell(fROM);
-			fseek(fROM, 0, SEEK_SET);
-			printf(" - Success (%u bytes)\n", romSize);
-
-			//try loading the sram
-			char *dot = strrchr(GBAgameName, '.');
-			if(!dot) return;
-
-			std::string sram_fname = GBAgameName;
-			sram_fname.resize(dot-GBAgameName);
-			sram_fname += ".sav";
-			fSRAM = fopen(sram_fname.c_str(), "rb+");
-			if(!fSRAM) return;
-			fseek(fSRAM, 0, SEEK_END);
-			sramSize = ftell(fSRAM);
-			fseek(fSRAM, 0, SEEK_SET);
-			saveType = getSaveTypeGBA();
-			printf("GBASlot found SRAM %s (%s - %u bytes)\n", sram_fname.c_str(), (saveType == 0xFF)?"Unknown":saveTypes[saveType], sramSize);
+			return;
+		}
+		
+		if (GBACartridge_RomPath.empty())
+		{
+			return;
+		}
+		
+		if (!strcasecmp(GBACartridge_RomPath.c_str(), "self"))
+		{
+			GBACartridge_RomPath = path.path;
+			GBACartridge_SRAMPath = Path::GetFileNameWithoutExt(GBACartridge_RomPath) + "." + GBA_SRAM_FILE_EXT;
+		}
+		
+		printf("GBASlot opening ROM: %s\n", GBACartridge_RomPath.c_str());
+		EMUFILE_FILE *inf = new EMUFILE_FILE(GBACartridge_RomPath, "rb");
+		inf->EnablePositionCache();
+		fROM = inf;
+		if (fROM->fail())
+		{
+			printf(" - Failed\n");
+			Close();
+			
+			return;
+		}
+		
+		romSize = fROM->size();
+		printf(" - Success (%u bytes)\n", romSize);
+		
+		// Load the GBA cartridge SRAM.
+		inf = new EMUFILE_FILE(GBACartridge_SRAMPath, "rb+");
+		fSRAM = inf;
+		if(fSRAM->fail())
+		{
+			delete fSRAM;
+			fSRAM = NULL;
+			printf("GBASlot did not load associated SRAM.\n");
+		}
+		else
+		{
+			inf->EnablePositionCache();
+			sramSize = fSRAM->size();
+			printf("Scanning GBA rom to ID save type\n");
+			saveType = scanSaveTypeGBA();
+			printf("\nGBASlot found SRAM (%s - %u bytes) at:\n%s\n", (saveType == 0xFF)?"Unknown":saveTypes[saveType], sramSize, GBACartridge_SRAMPath.c_str());
 			gbaFlash.size = sramSize;
 			if (gbaFlash.size <= (64 * 1024))
 			{
@@ -359,20 +399,13 @@ public:
 				gbaFlash.idDevice = 0x09;
 				gbaFlash.idManufacturer = 0xC2;
 			}
+			gbaFlash.state = 0;
 		}
-		else
-			printf(" - Failed\n");
 	}
 
 	virtual void disconnect()
 	{
-		if (fROM) fclose(fROM);
-		if (fSRAM) fclose(fSRAM);
-		fROM = NULL; fSRAM = NULL;
-		romSize = 0;
-		currentROMPos = 0;
-		sramSize = 0;
-		currentSRAMPos = 0;
+		Close();
 	}
 
 	virtual void writeByte(u8 PROCNUM, u32 addr, u8 val)
