@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2006 yopyop
-	Copyright (C) 2009-2012 DeSmuME team
+	Copyright (C) 2009-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
 #include <stdio.h>
 #include <assert.h>
 #include <algorithm>
-#include "types.h"
+
+#include "armcpu.h"
+#include "common.h"
 #include "instructions.h"
 #include "cp15.h"
 #include "bios.h"
@@ -53,14 +55,12 @@ armcpu_t NDS_ARM9;
 		      }       \
                       while(0)
 
-#ifdef GDB_STUB
-
 #define STALLED_CYCLE_COUNT 10
 
 static void
 stall_cpu( void *instance) {
   armcpu_t *armcpu = (armcpu_t *)instance;
-  printf("UNSTALL\n");
+  printf("STALL\n");
   armcpu->stalled = 1;
 }
                       
@@ -87,9 +87,7 @@ remove_post_exec_fn( void *instance) {
 
   armcpu->post_ex_fn = NULL;
 }
-#endif
 
-#ifdef GDB_STUB
 static u32 read_cpu_reg( void *instance, u32 reg_num)
 {
 	armcpu_t *armcpu = (armcpu_t *)instance;
@@ -104,6 +102,8 @@ static u32 read_cpu_reg( void *instance, u32 reg_num)
 	  //CPSR
 	  return armcpu->CPSR.val;
 	}
+	
+	return 0;
 }
 
 static void
@@ -120,41 +120,103 @@ set_cpu_reg( void *instance, u32 reg_num, u32 value) {
     /* FIXME: setting the CPSR */
   }
 }
-#endif
 
-#ifdef GDB_STUB
-int armcpu_new( armcpu_t *armcpu, u32 id,
-                struct armcpu_memory_iface *mem_if,
-                struct armcpu_ctrl_iface **ctrl_iface_ret)
-#else
 int armcpu_new( armcpu_t *armcpu, u32 id)
-#endif
 {
 	armcpu->proc_ID = id;
-
-#ifdef GDB_STUB
-	armcpu->mem_if = mem_if;
-
-	/* populate the control interface */
-	armcpu->ctrl_iface.stall = stall_cpu;
-	armcpu->ctrl_iface.unstall = unstall_cpu;
-	armcpu->ctrl_iface.read_reg = read_cpu_reg;
-	armcpu->ctrl_iface.set_reg = set_cpu_reg;
-	armcpu->ctrl_iface.install_post_ex_fn = install_post_exec_fn;
-	armcpu->ctrl_iface.remove_post_ex_fn = remove_post_exec_fn;
-	armcpu->ctrl_iface.data = armcpu;
-
-	*ctrl_iface_ret = &armcpu->ctrl_iface;
-
-	armcpu->post_ex_fn = NULL;
-#endif
-
 	armcpu->stalled = 0;
-
+	
+	armcpu->base_mem_if.prefetch32 = NULL;
+	armcpu->base_mem_if.prefetch16 = NULL;
+	armcpu->base_mem_if.read8 = NULL;
+	armcpu->base_mem_if.read16 = NULL;
+	armcpu->base_mem_if.read32 = NULL;
+	armcpu->base_mem_if.write8 = NULL;
+	armcpu->base_mem_if.write16 = NULL;
+	armcpu->base_mem_if.write32 = NULL;
+	armcpu->base_mem_if.data = NULL;
+	
+	armcpu->SetControlInterface(&arm_default_ctrl_iface);
+	armcpu->SetControlInterfaceData(armcpu);
+	armcpu->SetCurrentMemoryInterface(NULL);
+	armcpu->SetCurrentMemoryInterfaceData(NULL);
+	
+	armcpu->post_ex_fn = NULL;
+	armcpu->post_ex_fn_data = NULL;
+	
 	armcpu_init(armcpu, 0);
 
 	return 0;
-} 
+}
+
+void armcpu_t::SetControlInterface(const armcpu_ctrl_iface *theControlInterface)
+{
+	this->ctrl_iface = *theControlInterface;
+}
+
+armcpu_ctrl_iface* armcpu_t::GetControlInterface()
+{
+	return &this->ctrl_iface;
+}
+
+void armcpu_t::SetControlInterfaceData(void *theData)
+{
+	this->ctrl_iface.data = theData;
+}
+
+void* armcpu_t::GetControlInterfaceData()
+{
+	return this->ctrl_iface.data;
+}
+
+void armcpu_t::SetCurrentMemoryInterface(armcpu_memory_iface *theMemoryInterface)
+{
+	this->mem_if = theMemoryInterface;
+}
+
+armcpu_memory_iface* armcpu_t::GetCurrentMemoryInterface()
+{
+	return this->mem_if;
+}
+
+void armcpu_t::SetCurrentMemoryInterfaceData(void *theData)
+{
+	if (this->mem_if != NULL)
+	{
+		this->mem_if->data = theData;
+	}
+}
+
+void* armcpu_t::GetCurrentMemoryInterfaceData()
+{
+	return (this->mem_if != NULL) ? this->mem_if->data : NULL;
+}
+
+void armcpu_t::SetBaseMemoryInterface(const armcpu_memory_iface *theMemInterface)
+{
+	this->base_mem_if = *theMemInterface;
+}
+
+armcpu_memory_iface* armcpu_t::GetBaseMemoryInterface()
+{
+	return &this->base_mem_if;
+}
+
+void armcpu_t::SetBaseMemoryInterfaceData(void *theData)
+{
+	this->base_mem_if.data = theData;
+}
+
+void* armcpu_t::GetBaseMemoryInterfaceData()
+{
+	return this->base_mem_if.data;
+}
+
+void armcpu_t::ResetMemoryInterfaceToBase()
+{
+	this->SetCurrentMemoryInterface(this->GetBaseMemoryInterface());
+	this->SetCurrentMemoryInterfaceData(this->GetBaseMemoryInterfaceData());
+}
 
 //call this whenever CPSR is changed (other than CNVZQ or T flags); interrupts may need to be unleashed
 void armcpu_t::changeCPSR()
@@ -685,3 +747,39 @@ template u32 armcpu_exec<1,false>();
 template u32 armcpu_exec<1,true>();
 #endif
 
+void setIF(int PROCNUM, u32 flag)
+{
+	//don't set generated bits!!!
+	assert(!(flag&0x00200000));
+	
+	MMU.reg_IF_bits[PROCNUM] |= flag;
+	
+	NDS_Reschedule();
+}
+
+char* decodeIntruction(bool thumb_mode, u32 instr)
+{
+	char txt[20] = {0};
+	u32 tmp = 0;
+	if (thumb_mode == true)
+	{
+		tmp = (instr >> 6);
+		strcpy(txt, intToBin((u16)tmp)+6);
+	}
+	else
+	{
+		tmp = ((instr >> 16) & 0x0FF0) | ((instr >> 4) & 0x0F);
+		strcpy(txt, intToBin((u32)tmp)+20);
+	}
+	return strdup(txt);
+}
+
+const armcpu_ctrl_iface arm_default_ctrl_iface = {
+	stall_cpu,
+	unstall_cpu,
+	read_cpu_reg,
+	set_cpu_reg,
+	install_post_exec_fn,
+	remove_post_exec_fn,
+	NULL
+};
