@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2009-2012 DeSmuME team
+	Copyright (C) 2009-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -15,9 +15,11 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <string.h>
 #include "cheatSystem.h"
+#include "bits.h"
+
 #include "NDSSystem.h"
+#include "common.h"
 #include "mem.h"
 #include "MMU.h"
 #include "debug.h"
@@ -98,6 +100,7 @@ void CHEATS::ARparser(CHEATS_LIST& list)
 
 		if (if_flag > 0) 
 		{
+			if ((type == 0x0E)) i += ((lo + 7) / 8);
 			if ( (type == 0x0D) && (subtype == 0)) if_flag--;	// ENDIF
 			if ( (type == 0x0D) && (subtype == 2))				// NEXT & Flush
 			{
@@ -388,7 +391,7 @@ void CHEATS::ARparser(CHEATS_LIST& list)
 					}
 				}
 				
-				i += (lo / 8);
+				i += ((lo + 7) / 8);
 			}
 			break;
 
