@@ -1,6 +1,6 @@
 /* main.c - this file is part of DeSmuME
  *
- * Copyright (C) 2006,2007 DeSmuME Team
+ * Copyright (C) 2006-2015 DeSmuME Team
  * Copyright (C) 2007 Pascal Giard (evilynux)
  *
  * This file is free software; you can redistribute it and/or modify
@@ -44,21 +44,24 @@
 #define CLI_UI
 #endif
 
-#include "MMU.h"
-#include "NDSSystem.h"
-#include "debug.h"
-#include "sndsdl.h"
-#include "ctrlssdl.h"
-#include "render3D.h"
-#include "rasterize.h"
-#include "saves.h"
-#include "firmware.h"
-#include "GPU_osd.h"
-#include "desmume_config.h"
-#include "commandline.h"
-#include "slot2.h"
+#include "../NDSSystem.h"
+#include "../driver.h"
+#include "../GPU.h"
+#include "../SPU.h"
+#include "../sndsdl.h"
+#include "../ctrlssdl.h"
+#include "../render3D.h"
+#include "../rasterize.h"
+#include "../saves.h"
+#include "../GPU_osd.h"
+#include "../desmume_config.h"
+#include "../commandline.h"
+#include "../slot2.h"
+#include "../utils/xstring.h"
+
 #ifdef GDB_STUB
-#include "gdbstub.h"
+#include "../armcpu.h"
+#include "../gdbstub.h"
 #endif
 
 volatile bool execute = false;
@@ -120,7 +123,8 @@ const u16 cli_kb_cfg[NB_KEYS] =
     SDLK_s,         // X
     SDLK_a,         // Y
     SDLK_p,         // DEBUG
-    SDLK_o          // BOOST
+    SDLK_o,         // BOOST
+    SDLK_BACKSPACE, // Lid
   };
 
 class configured_features : public CommandLine
@@ -483,17 +487,13 @@ static void desmume_cycle(struct ctrls_event_config * cfg)
     SPU_Emulate_user();
 }
 
+#ifdef HAVE_LIBAGG
+T_AGG_RGB555 agg_targetScreen_cli(GPU_screen, 256, 384, 512);
+#endif
+
 int main(int argc, char ** argv) {
   class configured_features my_config;
   struct ctrls_event_config ctrls_cfg;
-#ifdef GDB_STUB
-  gdbstub_handle_t arm9_gdb_stub;
-  gdbstub_handle_t arm7_gdb_stub;
-  struct armcpu_memory_iface *arm9_memio = &arm9_base_memory_iface;
-  struct armcpu_memory_iface *arm7_memio = &arm7_base_memory_iface;
-  struct armcpu_ctrl_iface *arm9_ctrl_iface;
-  struct armcpu_ctrl_iface *arm7_ctrl_iface;
-#endif
 
   int limiter_frame_counter = 0;
   int limiter_tick0 = 0;
@@ -518,12 +518,7 @@ int main(int argc, char ** argv) {
   /* the firmware settings */
   struct NDS_fw_config_data fw_config;
 
-#ifdef GDB_STUB
-  NDS_Init( arm9_memio, &arm9_ctrl_iface,
-            arm7_memio, &arm7_ctrl_iface);
-#else
-        NDS_Init();
-#endif
+  NDS_Init();
 
   /* default the firmware settings, they may get changed later */
   NDS_FillDefaultFirmwareConfigData( &fw_config);
@@ -547,9 +542,17 @@ int main(int argc, char ** argv) {
         slot2_device_type = NDS_SLOT2_CFLASH;
 
     if(my_config.gbaslot_rom != "") {
-        strncpy(GBAgameName, my_config.gbaslot_rom.c_str(), MAX_PATH);
+
+        //set the GBA rom and sav paths
+        GBACartridge_RomPath = my_config.gbaslot_rom.c_str();
+        if(toupper(strright(GBACartridge_RomPath,4)) == ".GBA")
+          GBACartridge_SRAMPath = strright(GBACartridge_RomPath,4) + ".sav";
+        else
+          //what to do? lets just do the same thing for now
+          GBACartridge_SRAMPath = strright(GBACartridge_RomPath,4) + ".sav";
+
         // Check if the file exists and can be opened
-        FILE * test = fopen(GBAgameName, "rb");
+        FILE * test = fopen(GBACartridge_RomPath.c_str(), "rb");
         if (test) {
             slot2_device_type = NDS_SLOT2_GBACART;
             fclose(test);
@@ -590,30 +593,45 @@ int main(int argc, char ** argv) {
     g_thread_init( NULL);
   }
 
-  driver = new UnixDriver();
-
+  driver = new BaseDriver();
+  
 #ifdef GDB_STUB
-  if ( my_config.arm9_gdb_port != 0) {
-    arm9_gdb_stub = createStub_gdb( my_config.arm9_gdb_port,
-                                    &arm9_memio,
-                                    &arm9_direct_memory_iface);
+  gdbstub_mutex_init();
 
+  /*
+   * Activate the GDB stubs
+   * This has to come after NDS_Init() where the CPUs are set up.
+   */
+  gdbstub_handle_t arm9_gdb_stub = NULL;
+  gdbstub_handle_t arm7_gdb_stub = NULL;
+  
+  if ( my_config.arm9_gdb_port > 0) {
+    arm9_gdb_stub = createStub_gdb( my_config.arm9_gdb_port,
+                                   &NDS_ARM9,
+                                   &arm9_direct_memory_iface);
+    
     if ( arm9_gdb_stub == NULL) {
       fprintf( stderr, "Failed to create ARM9 gdbstub on port %d\n",
-               my_config.arm9_gdb_port);
+              my_config.arm9_gdb_port);
       exit( 1);
     }
+    else {
+      activateStub_gdb( arm9_gdb_stub);
+    }
   }
-  if ( my_config.arm7_gdb_port != 0) {
+  if ( my_config.arm7_gdb_port > 0) {
     arm7_gdb_stub = createStub_gdb( my_config.arm7_gdb_port,
-                                    &arm7_memio,
-                                    &arm7_base_memory_iface);
-
+                                   &NDS_ARM7,
+                                   &arm7_base_memory_iface);
+    
     if ( arm7_gdb_stub == NULL) {
       fprintf( stderr, "Failed to create ARM7 gdbstub on port %d\n",
-               my_config.arm7_gdb_port);
+              my_config.arm7_gdb_port);
       exit( 1);
     }
+    else {
+      activateStub_gdb( arm7_gdb_stub);
+    }
   }
 #endif
 
@@ -634,19 +652,6 @@ int main(int argc, char ** argv) {
     exit(-1);
   }
 
-  /*
-   * Activate the GDB stubs
-   * This has to come after the NDS_Init where the cpus are set up.
-   */
-#ifdef GDB_STUB
-  if ( my_config.arm9_gdb_port != 0) {
-    activateStub_gdb( arm9_gdb_stub, arm9_ctrl_iface);
-  }
-  if ( my_config.arm7_gdb_port != 0) {
-    activateStub_gdb( arm7_gdb_stub, arm7_ctrl_iface);
-  }
-#endif
-
   execute = true;
 
   if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) == -1)
@@ -737,7 +742,11 @@ int main(int argc, char ** argv) {
 #ifdef HAVE_LIBAGG
   Desmume_InitOnce();
   Hud.reset();
-  aggDraw.hud->attach(GPU_screen, 256, 384, 512);
+  // Now that gtk port draws to RGBA buffer directly, the other one
+  // has to use ugly ways to make HUD rendering work again.
+  // desmume gtk: Sorry desmume-cli :(
+  aggDraw.hud = &agg_targetScreen_cli;
+  aggDraw.hud->setFont("verdana18_bold");
 #endif
 
   ctrls_cfg.boost = 0;
@@ -813,17 +822,19 @@ int main(int argc, char ** argv) {
   /* Unload joystick */
   uninit_joy();
 
+#ifdef GDB_STUB
+  destroyStub_gdb( arm9_gdb_stub);
+  arm9_gdb_stub = NULL;
+  
+  destroyStub_gdb( arm7_gdb_stub);
+  arm7_gdb_stub = NULL;
+
+  gdbstub_mutex_destroy();
+#endif
+  
   SDL_Quit();
   NDS_DeInit();
 
-#ifdef GDB_STUB
-  if ( my_config.arm9_gdb_port != 0) {
-    destroyStub_gdb( arm9_gdb_stub);
-  }
-  if ( my_config.arm7_gdb_port != 0) {
-    destroyStub_gdb( arm7_gdb_stub);
-  }
-#endif
 
   return 0;
 }
