@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ GPU3DInterface *core3DList[] = {
 @implementation CocoaDSGPU
 
 @dynamic gpuStateFlags;
-@synthesize mutexProducer;
+@synthesize rwlockProducer;
 
 @dynamic layerMainGPU;
 @dynamic layerMainBG0;
@@ -67,6 +67,7 @@ GPU3DInterface *core3DList[] = {
 @dynamic render3DThreads;
 @dynamic render3DLineHack;
 @dynamic render3DMultisample;
+@dynamic render3DFragmentSamplingHack;
 
 
 - (id)init
@@ -78,7 +79,7 @@ GPU3DInterface *core3DList[] = {
 	}
 	
 	spinlockGpuState = OS_SPINLOCK_INIT;
-	mutexProducer = NULL;
+	rwlockProducer = NULL;
 	
 	gpuStateFlags	= GPUSTATE_MAIN_GPU_MASK |
 					  GPUSTATE_MAIN_BG0_MASK |
@@ -99,11 +100,14 @@ GPU3DInterface *core3DList[] = {
 							   &OSXOpenGLRendererBegin,
 							   &OSXOpenGLRendererEnd);
 	
+	GPU_FillScreenWithBGRA5551(0x8000);
+	
 	return self;
 }
 
 - (void)dealloc
 {
+	NDS_3D_ChangeCore(CORE3DLIST_NULL);
 	DestroyOpenGLRenderer();
 	
 	[super dealloc];
@@ -141,96 +145,96 @@ GPU3DInterface *core3DList[] = {
 
 - (void) setRender3DRenderingEngine:(NSInteger)methodID
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	NDS_3D_ChangeCore(methodID);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (NSInteger) render3DRenderingEngine
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const NSInteger methodID = (NSInteger)cur3DCore;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return methodID;
 }
 
 - (void) setRender3DHighPrecisionColorInterpolation:(BOOL)state
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_HighResolutionInterpolateColor = state ? true : false;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) render3DHighPrecisionColorInterpolation
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL state = CommonSettings.GFX3D_HighResolutionInterpolateColor ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return state;
 }
 
 - (void) setRender3DEdgeMarking:(BOOL)state
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_EdgeMark = state ? true : false;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) render3DEdgeMarking
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL state = CommonSettings.GFX3D_EdgeMark ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return state;
 }
 
 - (void) setRender3DFog:(BOOL)state
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_Fog = state ? true : false;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) render3DFog
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL state = CommonSettings.GFX3D_Fog ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return state;
 }
 
 - (void) setRender3DTextures:(BOOL)state
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_Texture = state ? true : false;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) render3DTextures
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL state = CommonSettings.GFX3D_Texture ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return state;
 }
 
 - (void) setRender3DDepthComparisonThreshold:(NSUInteger)threshold
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack = threshold;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (NSUInteger) render3DDepthComparisonThreshold
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const NSUInteger threshold = (NSUInteger)CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return threshold;
 }
@@ -267,7 +271,7 @@ GPU3DInterface *core3DList[] = {
 	
 	const NSInteger renderingEngineID = [self render3DRenderingEngine];
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	
 	CommonSettings.num_cores = numberCores;
 	
@@ -276,55 +280,71 @@ GPU3DInterface *core3DList[] = {
 		NDS_3D_ChangeCore(renderingEngineID);
 	}
 	
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (NSUInteger) render3DThreads
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const NSUInteger numberThreads = isCPUCoreCountAuto ? 0 : (NSUInteger)CommonSettings.num_cores;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return numberThreads;
 }
 
 - (void) setRender3DLineHack:(BOOL)state
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_LineHack = state ? true : false;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) render3DLineHack
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL state = CommonSettings.GFX3D_LineHack ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return state;
 }
 
 - (void) setRender3DMultisample:(BOOL)state
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.GFX3D_Renderer_Multisample = state ? true : false;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) render3DMultisample
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL state = CommonSettings.GFX3D_Renderer_Multisample ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
+	
+	return state;
+}
+
+- (void) setRender3DFragmentSamplingHack:(BOOL)state
+{
+	pthread_rwlock_wrlock(self.rwlockProducer);
+	CommonSettings.GFX3D_TXTHack = state ? true : false;
+	pthread_rwlock_unlock(self.rwlockProducer);
+}
+
+- (BOOL) render3DFragmentSamplingHack
+{
+	pthread_rwlock_rdlock(self.rwlockProducer);
+	const BOOL state = CommonSettings.GFX3D_TXTHack ? YES : NO;
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return state;
 }
 
 - (void) setLayerMainGPU:(BOOL)gpuState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPUDisplayState(DS_GPU_TYPE_MAIN, (gpuState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (gpuState) ? (gpuStateFlags | GPUSTATE_MAIN_GPU_MASK) : (gpuStateFlags & ~GPUSTATE_MAIN_GPU_MASK);
@@ -333,18 +353,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerMainGPU
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL gpuState = GetGPUDisplayState(DS_GPU_TYPE_MAIN) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return gpuState;
 }
 
 - (void) setLayerMainBG0:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_MAIN, 0, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_MAIN_BG0_MASK) : (gpuStateFlags & ~GPUSTATE_MAIN_BG0_MASK);
@@ -353,18 +373,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerMainBG0
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_MAIN, 0) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerMainBG1:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_MAIN, 1, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_MAIN_BG1_MASK) : (gpuStateFlags & ~GPUSTATE_MAIN_BG1_MASK);
@@ -373,18 +393,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerMainBG1
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_MAIN, 1) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerMainBG2:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_MAIN, 2, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_MAIN_BG2_MASK) : (gpuStateFlags & ~GPUSTATE_MAIN_BG2_MASK);
@@ -393,18 +413,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerMainBG2
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_MAIN, 2) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerMainBG3:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_MAIN, 3, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_MAIN_BG3_MASK) : (gpuStateFlags & ~GPUSTATE_MAIN_BG3_MASK);
@@ -413,18 +433,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerMainBG3
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_MAIN, 3) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerMainOBJ:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_MAIN, 4, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_MAIN_OBJ_MASK) : (gpuStateFlags & ~GPUSTATE_MAIN_OBJ_MASK);
@@ -433,18 +453,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerMainOBJ
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_MAIN, 4) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerSubGPU:(BOOL)gpuState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPUDisplayState(DS_GPU_TYPE_SUB, (gpuState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (gpuState) ? (gpuStateFlags | GPUSTATE_SUB_GPU_MASK) : (gpuStateFlags & ~GPUSTATE_SUB_GPU_MASK);
@@ -453,18 +473,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerSubGPU
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL gpuState = GetGPUDisplayState(DS_GPU_TYPE_SUB) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return gpuState;
 }
 
 - (void) setLayerSubBG0:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_SUB, 0, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_SUB_BG0_MASK) : (gpuStateFlags & ~GPUSTATE_SUB_BG0_MASK);
@@ -473,18 +493,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerSubBG0
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_SUB, 0) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerSubBG1:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_SUB, 1, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_SUB_BG1_MASK) : (gpuStateFlags & ~GPUSTATE_SUB_BG1_MASK);
@@ -493,18 +513,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerSubBG1
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_SUB, 1) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerSubBG2:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_SUB, 2, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_SUB_BG2_MASK) : (gpuStateFlags & ~GPUSTATE_SUB_BG2_MASK);
@@ -513,18 +533,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerSubBG2
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_SUB, 2) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerSubBG3:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_SUB, 3, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_SUB_BG3_MASK) : (gpuStateFlags & ~GPUSTATE_SUB_BG3_MASK);
@@ -533,18 +553,18 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerSubBG3
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_SUB, 3) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
 
 - (void) setLayerSubOBJ:(BOOL)layerState
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	SetGPULayerState(DS_GPU_TYPE_SUB, 4, (layerState) ? true : false);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	OSSpinLockLock(&spinlockGpuState);
 	gpuStateFlags = (layerState) ? (gpuStateFlags | GPUSTATE_SUB_OBJ_MASK) : (gpuStateFlags & ~GPUSTATE_SUB_OBJ_MASK);
@@ -553,9 +573,9 @@ GPU3DInterface *core3DList[] = {
 
 - (BOOL) layerSubOBJ
 {
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	const BOOL layerState = GetGPULayerState(DS_GPU_TYPE_SUB, 4) ? YES : NO;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return layerState;
 }
@@ -656,18 +676,18 @@ GPU3DInterface *core3DList[] = {
 {
 	NSString *theString = @"Uninitialized";
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	
 	if(gpu3D == NULL)
 	{
-		pthread_mutex_unlock(self.mutexProducer);
+		pthread_rwlock_unlock(self.rwlockProducer);
 		return theString;
 	}
 	
 	const char *theName = gpu3D->name;
 	theString = [NSString stringWithCString:theName encoding:NSUTF8StringEncoding];
 	
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return theString;
 }
@@ -803,6 +823,26 @@ bool GetGPUDisplayState(const int gpuType)
 	return theState;
 }
 
+void GPU_FillScreenWithBGRA5551(const uint16_t colorValue)
+{
+	const size_t pixCount = sizeof(GPU_screen) / sizeof(uint16_t);
+	
+#ifdef __APPLE__
+	if (pixCount % 16 == 0)
+	{
+		const uint16_t colorValuePattern[] = {colorValue, colorValue, colorValue, colorValue, colorValue, colorValue, colorValue, colorValue};
+		memset_pattern16(GPU_screen, colorValuePattern, sizeof(GPU_screen));
+	}
+	else
+#endif
+	{
+		for (size_t i = 0; i < pixCount; i++)
+		{
+			((uint16_t *)GPU_screen)[i] = colorValue;
+		}
+	}
+}
+
 CGLContextObj OSXOpenGLRendererContext = NULL;
 CGLPBufferObj OSXOpenGLRendererPBuffer = NULL;
 
