@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2012-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,11 +17,12 @@
 */
 
 #import "cocoa_cheat.h"
-#import "cocoa_core.h"
 #import "cocoa_globals.h"
 #import "cocoa_util.h"
 
+#include "../cheatSystem.h"
 #include "../MMU.h"
+#undef BOOL
 
 @implementation CocoaDSCheatItem
 
@@ -87,13 +88,6 @@ static NSImage *iconCodeBreaker = nil;
 		data = cheatData;
 	}
 	
-	if (iconInternalCheat == nil || iconActionReplay == nil || iconCodeBreaker == nil)
-	{
-		iconInternalCheat = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"AppIcon_DeSmuME" ofType:@"icns"]];
-		iconActionReplay = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon_ActionReplay_128x128" ofType:@"png"]];
-		iconCodeBreaker = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon_CodeBreaker_128x128" ofType:@"png"]];
-	}
-	
 	pthread_mutex_init(&mutexData, NULL);
 	
 	willAdd = NO;
@@ -132,9 +126,7 @@ static NSImage *iconCodeBreaker = nil;
 	}
 	
 	pthread_mutex_lock(&mutexData);
-	
 	data = cheatData;
-	
 	pthread_mutex_unlock(&mutexData);
 	
 	[self update];
@@ -633,6 +625,36 @@ static NSImage *iconCodeBreaker = nil;
 			nil];
 }
 
++ (void) setIconInternalCheat:(NSImage *)iconImage
+{
+	iconInternalCheat = iconImage;
+}
+
++ (NSImage *) iconInternalCheat
+{
+	return iconInternalCheat;
+}
+
++ (void) setIconActionReplay:(NSImage *)iconImage
+{
+	iconActionReplay = iconImage;
+}
+
++ (NSImage *) iconActionReplay
+{
+	return iconActionReplay;
+}
+
++ (void) setIconCodeBreaker:(NSImage *)iconImage
+{
+	iconCodeBreaker = iconImage;
+}
+
++ (NSImage *) iconCodeBreaker
+{
+	return iconCodeBreaker;
+}
+
 @end
 
 
@@ -640,32 +662,27 @@ static NSImage *iconCodeBreaker = nil;
 
 @synthesize listData;
 @synthesize list;
-@synthesize cdsCore;
+@dynamic rwlockCoreExecute;
 @synthesize untitledCount;
 @synthesize dbTitle;
 @synthesize dbDate;
 
 - (id)init
 {
-	return [self initWithCore:nil fileURL:nil listData:nil];
+	return [self initWithFileURL:nil listData:nil];
 }
 
-- (id) initWithCore:(CocoaDSCore *)core
+- (id) initWithFileURL:(NSURL *)fileURL
 {
-	return [self initWithCore:core fileURL:nil listData:nil];
+	return [self initWithFileURL:fileURL listData:nil];
 }
 
-- (id) initWithCore:(CocoaDSCore *)core fileURL:(NSURL *)fileURL
+- (id) initWithListData:(CHEATS *)cheatList
 {
-	return [self initWithCore:core fileURL:fileURL listData:nil];
+	return [self initWithFileURL:nil listData:cheatList];
 }
 
-- (id) initWithCore:(CocoaDSCore *)core listData:(CHEATS *)cheatList
-{
-	return [self initWithCore:core fileURL:nil listData:cheatList];
-}
-
-- (id) initWithCore:(CocoaDSCore *)core fileURL:(NSURL *)fileURL listData:(CHEATS *)cheatList
+- (id) initWithFileURL:(NSURL *)fileURL listData:(CHEATS *)cheatList
 {
 	self = [super init];
 	if(self == nil)
@@ -705,7 +722,10 @@ static NSImage *iconCodeBreaker = nil;
 		}
 	}
 	
-	cdsCore = [core retain];
+	rwlockCoreExecute = (pthread_rwlock_t *)malloc(sizeof(pthread_rwlock_t));
+	pthread_rwlock_init(rwlockCoreExecute, NULL);
+	isUsingDummyRWlock = YES;
+	
 	untitledCount = 0;
 	dbTitle = nil;
 	dbDate = nil;
@@ -717,13 +737,50 @@ static NSImage *iconCodeBreaker = nil;
 {
 	self.dbTitle = nil;
 	self.dbDate = nil;
-	self.cdsCore = nil;
 	[list release];
 	delete (CHEATS *)self.listData;
 	
+	if (isUsingDummyRWlock)
+	{
+		pthread_rwlock_destroy(rwlockCoreExecute);
+		free(rwlockCoreExecute);
+		rwlockCoreExecute = NULL;
+	}
+	
 	[super dealloc];
 }
 
+- (void) setRwlockCoreExecute:(pthread_rwlock_t *)theRwlock
+{
+	if (theRwlock == NULL && isUsingDummyRWlock)
+	{
+		return;
+	}
+	else if (theRwlock == NULL && !isUsingDummyRWlock)
+	{
+		rwlockCoreExecute = (pthread_rwlock_t *)malloc(sizeof(pthread_rwlock_t));
+		pthread_rwlock_init(rwlockCoreExecute, NULL);
+		isUsingDummyRWlock = YES;
+		return;
+	}
+	else if (theRwlock != NULL && isUsingDummyRWlock)
+	{
+		pthread_rwlock_destroy(rwlockCoreExecute);
+		free(rwlockCoreExecute);
+		isUsingDummyRWlock = NO;
+		rwlockCoreExecute = theRwlock;
+	}
+	else if (theRwlock != NULL && !isUsingDummyRWlock)
+	{
+		rwlockCoreExecute = theRwlock;
+	}
+}
+
+- (pthread_rwlock_t *) rwlockCoreExecute
+{
+	return rwlockCoreExecute;
+}
+
 - (BOOL) add:(CocoaDSCheatItem *)cheatItem
 {
 	BOOL result = NO;
@@ -737,7 +794,7 @@ static NSImage *iconCodeBreaker = nil;
 	// to check if the list got reallocated.
 	CHEATS_LIST *cheatListData = self.listData->getListPtr();
 	
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	
 	switch (cheatItem.cheatType)
 	{
@@ -769,7 +826,7 @@ static NSImage *iconCodeBreaker = nil;
 			break;
 	}
 	
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	if (![self.list containsObject:cheatItem])
 	{
@@ -809,9 +866,9 @@ static NSImage *iconCodeBreaker = nil;
 		return;
 	}
 	
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	self.listData->remove(selectionIndex);
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	// Removing an item from the raw cheat list data shifts all higher elements
 	// by one, so we need to do the same.
@@ -840,7 +897,7 @@ static NSImage *iconCodeBreaker = nil;
 		return result;
 	}
 	
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	
 	switch (cheatItem.cheatType)
 	{
@@ -872,7 +929,7 @@ static NSImage *iconCodeBreaker = nil;
 			break;
 	}
 		
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	[cheatItem update];
 		
@@ -881,18 +938,18 @@ static NSImage *iconCodeBreaker = nil;
 
 - (BOOL) save
 {
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	BOOL result = self.listData->save();
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	return result;
 }
 
 - (NSUInteger) activeCount
 {
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_rdlock(self.rwlockCoreExecute);
 	NSUInteger activeCheatsCount = self.listData->getActiveCount();
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	return activeCheatsCount;
 }
@@ -944,9 +1001,9 @@ static NSImage *iconCodeBreaker = nil;
 		return;
 	}
 	
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	[CocoaDSCheatManager applyInternalCheatWithItem:cheatItem];
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 }
 
 + (void) setMasterCheatList:(CocoaDSCheatManager *)cheatListManager
@@ -1071,15 +1128,10 @@ static NSImage *iconCodeBreaker = nil;
 
 @synthesize listData;
 @synthesize addressList;
-@synthesize cdsCore;
+@dynamic rwlockCoreExecute;
 @synthesize searchCount;
 
 - (id)init
-{
-	return [self initWithCore:nil];
-}
-
-- (id) initWithCore:(CocoaDSCore *)core
 {
 	self = [super init];
 	if(self == nil)
@@ -1094,9 +1146,12 @@ static NSImage *iconCodeBreaker = nil;
 		return nil;
 	}
 	
+	rwlockCoreExecute = (pthread_rwlock_t *)malloc(sizeof(pthread_rwlock_t));
+	pthread_rwlock_init(rwlockCoreExecute, NULL);
+	isUsingDummyRWlock = YES;
+	
 	listData = newListData;
 	addressList = nil;
-	cdsCore = [core retain];
 	searchCount = 0;
 	
 	return self;
@@ -1104,17 +1159,54 @@ static NSImage *iconCodeBreaker = nil;
 
 - (void)dealloc
 {
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	self.listData->close();
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	[addressList release];
-	self.cdsCore = nil;
 	delete (CHEATSEARCH *)self.listData;
 	
+	if (isUsingDummyRWlock)
+	{
+		pthread_rwlock_destroy(rwlockCoreExecute);
+		free(rwlockCoreExecute);
+		rwlockCoreExecute = NULL;
+	}
+	
 	[super dealloc];
 }
 
+- (void) setRwlockCoreExecute:(pthread_rwlock_t *)theRwlock
+{
+	if (theRwlock == NULL && isUsingDummyRWlock)
+	{
+		return;
+	}
+	else if (theRwlock == NULL && !isUsingDummyRWlock)
+	{
+		rwlockCoreExecute = (pthread_rwlock_t *)malloc(sizeof(pthread_mutex_t));
+		pthread_rwlock_init(rwlockCoreExecute, NULL);
+		isUsingDummyRWlock = YES;
+		return;
+	}
+	else if (theRwlock != NULL && isUsingDummyRWlock)
+	{
+		pthread_rwlock_destroy(rwlockCoreExecute);
+		free(rwlockCoreExecute);
+		isUsingDummyRWlock = NO;
+		rwlockCoreExecute = theRwlock;
+	}
+	else if (theRwlock != NULL && !isUsingDummyRWlock)
+	{
+		rwlockCoreExecute = theRwlock;
+	}
+}
+
+- (pthread_rwlock_t *) rwlockCoreExecute
+{
+	return rwlockCoreExecute;
+}
+
 - (NSUInteger) runExactValueSearch:(NSInteger)value byteSize:(UInt8)byteSize signType:(NSInteger)signType
 {
 	NSUInteger itemCount = 0;
@@ -1124,17 +1216,17 @@ static NSImage *iconCodeBreaker = nil;
 	{
 		byteSize--;
 		
-		pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_rdlock(self.rwlockCoreExecute);
 		listExists = (NSUInteger)self.listData->start((u8)CHEATSEARCH_SEARCHSTYLE_EXACT_VALUE, (u8)byteSize, (u8)signType);
-		pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_unlock(self.rwlockCoreExecute);
 	}
 	
 	if (listExists)
 	{
-		pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_rdlock(self.rwlockCoreExecute);
 		itemCount = (NSUInteger)self.listData->search((u32)value);
 		NSMutableArray *newAddressList = [[CocoaDSCheatSearch addressListWithListObject:self.listData maxItems:100] retain];
-		pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_unlock(self.rwlockCoreExecute);
 		
 		[addressList release];
 		addressList = newAddressList;
@@ -1161,18 +1253,18 @@ static NSImage *iconCodeBreaker = nil;
 	{
 		byteSize--;
 		
-		pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_rdlock(self.rwlockCoreExecute);
 		listExists = (NSUInteger)self.listData->start((u8)CHEATSEARCH_SEARCHSTYLE_COMPARATIVE, (u8)byteSize, (u8)signType);
-		pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_unlock(self.rwlockCoreExecute);
 		
 		addressList = nil;
 	}
 	else
 	{
-		pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_rdlock(self.rwlockCoreExecute);
 		itemCount = (NSUInteger)self.listData->search((u8)typeID);
 		NSMutableArray *newAddressList = [[CocoaDSCheatSearch addressListWithListObject:self.listData maxItems:100] retain];
-		pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+		pthread_rwlock_unlock(self.rwlockCoreExecute);
 		
 		[addressList release];
 		addressList = newAddressList;
@@ -1196,9 +1288,9 @@ static NSImage *iconCodeBreaker = nil;
 
 - (void) reset
 {
-	pthread_mutex_lock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_wrlock(self.rwlockCoreExecute);
 	self.listData->close();
-	pthread_mutex_unlock(self.cdsCore.mutexCoreExecute);
+	pthread_rwlock_unlock(self.rwlockCoreExecute);
 	
 	searchCount = 0;
 	[addressList release];
