@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2011-2013 DeSmuME team
+	Copyright (C) 2011-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -28,11 +28,48 @@
 #include <mach/mach.h>
 #include <mach/mach_time.h>
 
+#include "../movie.h"
 #include "../NDSSystem.h"
+#include "../armcpu.h"
+#include "../driver.h"
+#include "../gdbstub.h"
 #include "../slot1.h"
 #include "../slot2.h"
 #undef BOOL
 
+// Need to include assert.h this way so that GDB stub will work
+// with an optimized build.
+#if defined(GDB_STUB) && defined(NDEBUG)
+	#define TEMP_NDEBUG
+	#undef NDEBUG
+#endif
+
+#include <assert.h>
+
+#if defined(TEMP_NDEBUG)
+	#undef TEMP_NDEBUG
+	#define NDEBUG
+#endif
+
+class OSXDriver : public BaseDriver
+{
+private:
+	pthread_mutex_t *mutexThreadExecute;
+	pthread_rwlock_t *rwlockCoreExecute;
+	CocoaDSCore *cdsCore;
+	
+public:
+	pthread_mutex_t* GetCoreThreadMutexLock();
+	void SetCoreThreadMutexLock(pthread_mutex_t *theMutex);
+	pthread_rwlock_t* GetCoreExecuteRWLock();
+	void SetCoreExecuteRWLock(pthread_rwlock_t *theRwLock);
+	void SetCore(CocoaDSCore *theCore);
+	
+	virtual void EMU_DebugIdleEnter();
+	virtual void EMU_DebugIdleUpdate();
+	virtual void EMU_DebugIdleWakeUp();
+};
+
 
 //accessed from other files
 volatile bool execute = true;
@@ -51,6 +88,13 @@ volatile bool execute = true;
 @dynamic isCheatingEnabled;
 @dynamic speedScalar;
 
+@dynamic isGdbStubStarted;
+@dynamic isInDebugTrap;
+@synthesize enableGdbStubARM9;
+@synthesize enableGdbStubARM7;
+@synthesize gdbStubPortARM9;
+@synthesize gdbStubPortARM7;
+
 @dynamic emulationFlags;
 @synthesize emuFlagAdvancedBusLevelTiming;
 @synthesize emuFlagRigorousTiming;
@@ -65,15 +109,15 @@ volatile bool execute = true;
 @dynamic maxJITBlockSize;
 @synthesize slot1DeviceType;
 @synthesize slot1StatusText;
+@synthesize frameStatus;
+@synthesize executionSpeedStatus;
 
 @dynamic arm9ImageURL;
 @dynamic arm7ImageURL;
 @dynamic firmwareImageURL;
 @synthesize slot1R4URL;
 
-@dynamic mutexCoreExecute;
-
-static BOOL isCoreStarted = NO;
+@dynamic rwlockCoreExecute;
 
 - (id)init
 {
@@ -83,6 +127,23 @@ static BOOL isCoreStarted = NO;
 		return self;
 	}
 	
+	isGdbStubStarted = NO;
+	isInDebugTrap = NO;
+	enableGdbStubARM9 = NO;
+	enableGdbStubARM7 = NO;
+	gdbStubPortARM9 = 0;
+	gdbStubPortARM7 = 0;
+	gdbStubHandleARM9 = NULL;
+	gdbStubHandleARM7 = NULL;
+	
+	int initResult = NDS_Init();
+	if (initResult == -1)
+	{
+		[self release];
+		self = nil;
+		return self;
+	}
+	
 	cdsController = nil;
 	cdsFirmware = nil;
 	cdsGPU = [[[[CocoaDSGPU alloc] init] autorelease] retain];
@@ -117,24 +178,51 @@ static BOOL isCoreStarted = NO;
 	_slot1R4Path = "";
 	
 	threadParam.cdsCore = self;
-	threadParam.cdsController = cdsController;
 	threadParam.state = CORESTATE_PAUSE;
 	threadParam.isFrameSkipEnabled = true;
 	threadParam.frameCount = 0;
 	threadParam.framesToSkip = 0;
+	threadParam.frameJumpTarget = 0;
 	
 	uint64_t timeBudgetNanoseconds = (uint64_t)(DS_SECONDS_PER_FRAME * 1000000000.0 / speedScalar);
 	AbsoluteTime timeBudgetAbsTime = NanosecondsToAbsolute(*(Nanoseconds *)&timeBudgetNanoseconds);
 	threadParam.timeBudgetMachAbsTime = *(uint64_t *)&timeBudgetAbsTime;
 	
-	threadParam.exitThread = false;
-	pthread_mutex_init(&threadParam.mutexCoreExecute, NULL);
 	pthread_mutex_init(&threadParam.mutexOutputList, NULL);
 	pthread_mutex_init(&threadParam.mutexThreadExecute, NULL);
 	pthread_cond_init(&threadParam.condThreadExecute, NULL);
+	pthread_rwlock_init(&threadParam.rwlockCoreExecute, NULL);
 	pthread_create(&coreThread, NULL, &RunCoreThread, &threadParam);
 	
-	[cdsGPU setMutexProducer:self.mutexCoreExecute];
+	// The core emulation thread needs max priority since it is the sole
+	// producer thread for all output threads. Note that this is not being
+	// done for performance -- this is being done for timing accuracy. The
+	// core emulation thread is responsible for determining the emulator's
+	// timing. If one output thread interferes with timing, then it ends up
+	// affecting the whole emulator.
+	//
+	// Though it may be tempting to make this a real-time thread, it's best
+	// to keep this a normal thread. The core emulation thread can use up a
+	// lot of CPU time under certain conditions, which may interfere with
+	// other threads. (Example: Video tearing on display windows, even with
+	// V-sync enabled.)
+	struct sched_param sp;
+	int thePolicy = 0;
+	memset(&sp, 0, sizeof(struct sched_param));
+	pthread_getschedparam(coreThread, &thePolicy, &sp);
+	sp.sched_priority = sched_get_priority_max(thePolicy);
+	pthread_setschedparam(coreThread, thePolicy, &sp);
+	
+	[cdsGPU setRwlockProducer:self.rwlockCoreExecute];
+	
+	OSXDriver *newDriver = new OSXDriver;
+	newDriver->SetCoreThreadMutexLock(&threadParam.mutexThreadExecute);
+	newDriver->SetCoreExecuteRWLock(self.rwlockCoreExecute);
+	newDriver->SetCore(self);
+	driver = newDriver;
+	
+	frameStatus = @"---";
+	executionSpeedStatus = @"1.00x";
 	
 	return self;
 }
@@ -143,78 +231,29 @@ static BOOL isCoreStarted = NO;
 {
 	[self setCoreState:CORESTATE_PAUSE];
 	
-	// Exit the thread.
-	if (self.thread != nil)
-	{
-		self.threadExit = YES;
-		
-		// Wait until the thread has shut down.
-		while (self.thread != nil)
-		{
-			[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.01]];
-		}
-	}
-	
-	pthread_mutex_lock(&threadParam.mutexThreadExecute);
-	threadParam.exitThread = true;
-	pthread_cond_signal(&threadParam.condThreadExecute);
-	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
-	
-	pthread_join(coreThread, NULL);
-	coreThread = nil;
-	pthread_mutex_destroy(&threadParam.mutexThreadExecute);
-	pthread_cond_destroy(&threadParam.condThreadExecute);
-	
-	self.cdsController = nil;
-	self.cdsFirmware = nil;
-	self.cdsGPU = nil;
-	
 	[self removeAllOutputs];
 	[cdsOutputList release];
 	
+	[self setCdsController:nil];
+	[self setCdsFirmware:nil];
+	[self setCdsGPU:nil];
+	
+	pthread_cancel(coreThread);
+	pthread_join(coreThread, NULL);
+	coreThread = NULL;
+	
+	pthread_mutex_destroy(&threadParam.mutexThreadExecute);
+	pthread_cond_destroy(&threadParam.condThreadExecute);
 	pthread_mutex_destroy(&threadParam.mutexOutputList);
-	pthread_mutex_destroy(&threadParam.mutexCoreExecute);
+	pthread_rwlock_destroy(&threadParam.rwlockCoreExecute);
+	
+	[self setIsGdbStubStarted:NO];
+	
+	NDS_DeInit();
 	
 	[super dealloc];
 }
 
-+ (BOOL) startupCore
-{
-	NSInteger result = -1;
-	
-	if (isCoreStarted)
-	{
-		return isCoreStarted;
-	}
-	
-	slot2_Change(NDS_SLOT2_NONE);
-	
-	result = NDS_Init();
-	if (result == -1)
-	{
-		isCoreStarted = NO;
-		return isCoreStarted;
-	}
-	
-	isCoreStarted = YES;
-	
-	return isCoreStarted;
-}
-
-+ (void) shutdownCore
-{
-	if (isCoreStarted)
-	{
-		NDS_DeInit();
-		isCoreStarted = NO;
-	}
-}
-
-+ (BOOL) isCoreStarted
-{
-	return isCoreStarted;
-}
-
 - (void) setMasterExecute:(BOOL)theState
 {
 	OSSpinLockLock(&spinlockMasterExecute);
@@ -249,7 +288,6 @@ static BOOL isCoreStarted = NO;
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
 	[cdsController release];
 	cdsController = theController;
-	threadParam.cdsController = theController;
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
 	
 	OSSpinLockUnlock(&spinlockCdsController);
@@ -276,6 +314,7 @@ static BOOL isCoreStarted = NO;
 	{
 		threadParam.isFrameSkipEnabled = false;
 		threadParam.framesToSkip = 0;
+		NDS_OmitFrameSkip(2);
 	}
 	
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
@@ -340,13 +379,106 @@ static BOOL isCoreStarted = NO;
 	return theState;
 }
 
+- (void) setIsGdbStubStarted:(BOOL)theState
+{
+#ifdef GDB_STUB
+	if (theState)
+	{
+        gdbstub_mutex_init();
+        
+		if ([self enableGdbStubARM9])
+		{
+			const uint16_t arm9Port = (uint16_t)[self gdbStubPortARM9];
+			if(arm9Port > 0)
+			{
+				gdbStubHandleARM9 = createStub_gdb(arm9Port, &NDS_ARM9, &arm9_direct_memory_iface);
+				if (gdbStubHandleARM9 == NULL)
+				{
+					NSLog(@"Failed to create ARM9 gdbstub on port %d\n", arm9Port);
+				}
+				else
+				{
+					activateStub_gdb(gdbStubHandleARM9);
+				}
+			}
+		}
+		else
+		{
+			destroyStub_gdb(gdbStubHandleARM9);
+			gdbStubHandleARM9 = NULL;
+		}
+		
+		if ([self enableGdbStubARM7])
+		{
+			const uint16_t arm7Port = (uint16_t)[self gdbStubPortARM7];
+			if (arm7Port > 0)
+			{
+				gdbStubHandleARM7 = createStub_gdb(arm7Port, &NDS_ARM7, &arm7_base_memory_iface);
+				if (gdbStubHandleARM7 == NULL)
+				{
+					NSLog(@"Failed to create ARM7 gdbstub on port %d\n", arm7Port);
+				}
+				else
+				{
+					activateStub_gdb(gdbStubHandleARM7);
+				}
+			}
+		}
+		else
+		{
+			destroyStub_gdb(gdbStubHandleARM7);
+			gdbStubHandleARM7 = NULL;
+		}
+	}
+	else
+	{
+		destroyStub_gdb(gdbStubHandleARM9);
+		gdbStubHandleARM9 = NULL;
+		
+		destroyStub_gdb(gdbStubHandleARM7);
+		gdbStubHandleARM7 = NULL;
+
+        gdbstub_mutex_destroy();
+	}
+#endif
+	if (gdbStubHandleARM9 == NULL && gdbStubHandleARM7 == NULL)
+	{
+		theState = NO;
+	}
+	
+	isGdbStubStarted = theState;
+}
+
+- (BOOL) isGdbStubStarted
+{
+	return isGdbStubStarted;
+}
+
+- (void) setIsInDebugTrap:(BOOL)theState
+{
+	// If we're transitioning out of the debug trap, then ignore
+	// frame skipping this time.
+	if (isInDebugTrap && !theState)
+	{
+		threadParam.framesToSkip = 0;
+		NDS_OmitFrameSkip(2);
+	}
+	
+	isInDebugTrap = theState;
+}
+
+- (BOOL) isInDebugTrap
+{
+	return isInDebugTrap;
+}
+
 - (void) setEmulationFlags:(NSUInteger)theFlags
 {
 	OSSpinLockLock(&spinlockEmulationFlags);
 	emulationFlags = theFlags;
 	OSSpinLockUnlock(&spinlockEmulationFlags);
 	
-	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_wrlock(&threadParam.rwlockCoreExecute);
 	
 	if (theFlags & EMULATION_ADVANCED_BUS_LEVEL_TIMING_MASK)
 	{
@@ -447,7 +579,7 @@ static BOOL isCoreStarted = NO;
 		CommonSettings.DebugConsole = false;
 	}
 	
-	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_unlock(&threadParam.rwlockCoreExecute);
 }
 
 - (NSUInteger) emulationFlags
@@ -481,16 +613,16 @@ static BOOL isCoreStarted = NO;
 
 - (void) setMaxJITBlockSize:(NSInteger)blockSize
 {
-	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_wrlock(&threadParam.rwlockCoreExecute);
 	CommonSettings.jit_max_block_size = (blockSize > 0) ? blockSize : 1;
-	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_unlock(&threadParam.rwlockCoreExecute);
 }
 
 - (NSInteger) maxJITBlockSize
 {
-	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_rdlock(&threadParam.rwlockCoreExecute);
 	const NSInteger blockSize = CommonSettings.jit_max_block_size;
-	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_unlock(&threadParam.rwlockCoreExecute);
 	
 	return blockSize;
 }
@@ -499,18 +631,72 @@ static BOOL isCoreStarted = NO;
 {
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
 	
-	if (threadParam.state == CORESTATE_PAUSE)
+	if (threadParam.state == CORESTATE_EXECUTE || threadParam.state == CORESTATE_PAUSE)
 	{
-		prevCoreState = CORESTATE_PAUSE;
-	}
-	else
-	{
-		prevCoreState = CORESTATE_EXECUTE;
+		prevCoreState = threadParam.state;
 	}
 	
 	threadParam.state = coreState;
+	threadParam.framesToSkip = 0;
+	NDS_OmitFrameSkip(2);
+	
+	switch (coreState)
+	{
+		case CORESTATE_PAUSE:
+		{
+			for(CocoaDSOutput *cdsOutput in cdsOutputList)
+			{
+				[cdsOutput setIdle:YES];
+			}
+			
+			[self setFrameStatus:[NSString stringWithFormat:@"%lu", (unsigned long)[self frameNumber]]];
+			break;
+		}
+			
+		case CORESTATE_FRAMEADVANCE:
+		{
+			for(CocoaDSOutput *cdsOutput in cdsOutputList)
+			{
+				[cdsOutput setIdle:NO];
+			}
+			
+			[self setFrameStatus:[NSString stringWithFormat:@"%lu", (unsigned long)[self frameNumber]]];
+			break;
+		}
+			
+		case CORESTATE_EXECUTE:
+		{
+			for(CocoaDSOutput *cdsOutput in cdsOutputList)
+			{
+				[cdsOutput setIdle:NO];
+			}
+			
+			[self setFrameStatus:@"Executing..."];
+			break;
+		}
+			
+		case CORESTATE_FRAMEJUMP:
+		{
+			for(CocoaDSOutput *cdsOutput in cdsOutputList)
+			{
+				if (![cdsOutput isKindOfClass:[CocoaDSDisplay class]])
+				{
+					[cdsOutput setIdle:YES];
+				}
+			}
+			
+			[self setFrameStatus:[NSString stringWithFormat:@"Jumping to frame %lu.", (unsigned long)threadParam.frameJumpTarget]];
+			break;
+		}
+			
+		default:
+			break;
+	}
+	
 	pthread_cond_signal(&threadParam.condThreadExecute);
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+	
+	[[self cdsController] setHardwareMicPause:!(coreState == CORESTATE_EXECUTE)];
 }
 
 - (NSInteger) coreState
@@ -573,9 +759,9 @@ static BOOL isCoreStarted = NO;
 	return [NSURL fileURLWithPath:[NSString stringWithCString:CommonSettings.Firmware encoding:NSUTF8StringEncoding]];
 }
 
-- (pthread_mutex_t *) mutexCoreExecute
+- (pthread_rwlock_t *) rwlockCoreExecute
 {
-	return &threadParam.mutexCoreExecute;
+	return &threadParam.rwlockCoreExecute;
 }
 
 - (void) setEjectCardFlag
@@ -602,41 +788,38 @@ static BOOL isCoreStarted = NO;
 
 - (void) slot1Eject
 {
-	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_wrlock(&threadParam.rwlockCoreExecute);
 	NDS_TriggerCardEjectIRQ();
-	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_unlock(&threadParam.rwlockCoreExecute);
 	
 	[self setSlot1StatusText:NSSTRING_STATUS_SLOT1_NO_DEVICE];
 }
 
 - (void) changeRomSaveType:(NSInteger)saveTypeID
 {
-	pthread_mutex_lock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_wrlock(&threadParam.rwlockCoreExecute);
 	[CocoaDSRom changeRomSaveType:saveTypeID];
-	pthread_mutex_unlock(&threadParam.mutexCoreExecute);
+	pthread_rwlock_unlock(&threadParam.rwlockCoreExecute);
 }
 
 - (void) changeExecutionSpeed
 {
 	if (self.isSpeedLimitEnabled)
 	{
-		CGFloat theSpeed = self.speedScalar;
-		if(theSpeed <= SPEED_SCALAR_MIN)
-		{
-			theSpeed = SPEED_SCALAR_MIN;
-		}
-		
+		const CGFloat theSpeed = ([self speedScalar] > SPEED_SCALAR_MIN) ? [self speedScalar] : SPEED_SCALAR_MIN;
 		pthread_mutex_lock(&threadParam.mutexThreadExecute);
-		uint64_t timeBudgetNanoseconds = (uint64_t)(DS_SECONDS_PER_FRAME * 1000000000.0 / theSpeed);
-		AbsoluteTime timeBudgetAbsTime = NanosecondsToAbsolute(*(Nanoseconds *)&timeBudgetNanoseconds);
-		threadParam.timeBudgetMachAbsTime = *(uint64_t *)&timeBudgetAbsTime;
+		threadParam.timeBudgetMachAbsTime = GetFrameAbsoluteTime(1.0/theSpeed);
 		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		
+		[self setExecutionSpeedStatus:[NSString stringWithFormat:@"%1.2fx", theSpeed]];
 	}
 	else
 	{
 		pthread_mutex_lock(&threadParam.mutexThreadExecute);
 		threadParam.timeBudgetMachAbsTime = 0;
 		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		
+		[self setExecutionSpeedStatus:@"Unlimited"];
 	}
 }
 
@@ -724,41 +907,67 @@ static BOOL isCoreStarted = NO;
 	
 	pthread_mutex_lock(&threadParam.mutexThreadExecute);
 	NDS_Reset();
+	GPU_FillScreenWithBGRA5551(0xFFFF);
 	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
 	
 	[self restoreCoreState];
-	self.masterExecute = YES;
+	[self setMasterExecute:YES];
+	[[self cdsController] reset];
+	[[self cdsController] updateMicLevel];
+}
+
+- (NSUInteger) frameNumber
+{
+	pthread_rwlock_rdlock(&threadParam.rwlockCoreExecute);
+	const NSUInteger currFrameNum = currFrameCounter;
+	pthread_rwlock_unlock(&threadParam.rwlockCoreExecute);
+	
+	return currFrameNum;
+}
+
+- (void) frameJumpTo:(NSUInteger)targetFrameNum
+{
+	pthread_mutex_lock(&threadParam.mutexThreadExecute);
+	
+	threadParam.frameJumpTarget = targetFrameNum;
+	if (targetFrameNum <= (NSUInteger)currFrameCounter)
+	{
+		pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+		return;
+	}
+	
+	pthread_mutex_unlock(&threadParam.mutexThreadExecute);
+	
+	[self setCoreState:CORESTATE_FRAMEJUMP];
+}
+
+- (void) frameJump:(NSUInteger)relativeFrameNum
+{
+	[self frameJumpTo:[self frameNumber] + relativeFrameNum];
 }
 
 - (void) addOutput:(CocoaDSOutput *)theOutput
 {
 	pthread_mutex_lock(&threadParam.mutexOutputList);
-	theOutput.mutexProducer = self.mutexCoreExecute;
-	[self.cdsOutputList addObject:theOutput];
+	[theOutput setRwlockProducer:[self rwlockCoreExecute]];
+	[[self cdsOutputList] addObject:theOutput];
 	pthread_mutex_unlock(&threadParam.mutexOutputList);
 }
 
 - (void) removeOutput:(CocoaDSOutput *)theOutput
 {
 	pthread_mutex_lock(&threadParam.mutexOutputList);
-	[self.cdsOutputList removeObject:theOutput];
+	[[self cdsOutputList] removeObject:theOutput];
 	pthread_mutex_unlock(&threadParam.mutexOutputList);
 }
 
 - (void) removeAllOutputs
 {
 	pthread_mutex_lock(&threadParam.mutexOutputList);
-	[self.cdsOutputList removeAllObjects];
+	[[self cdsOutputList] removeAllObjects];
 	pthread_mutex_unlock(&threadParam.mutexOutputList);
 }
 
-- (void) runThread:(id)object
-{
-	[CocoaDSCore startupCore];
-	[super runThread:object];
-	[CocoaDSCore shutdownCore];
-}
-
 - (NSString *) cpuEmulationEngineString
 {
 	NSString *theString = @"Uninitialized";
@@ -820,6 +1029,49 @@ static BOOL isCoreStarted = NO;
 	return theString;
 }
 
+- (BOOL) startReplayRecording:(NSURL *)fileURL sramURL:(NSURL *)sramURL
+{
+	if (fileURL == nil)
+	{
+		return NO;
+	}
+	
+	std::string sramPath = (sramURL != nil) ? [[sramURL path] cStringUsingEncoding:NSUTF8StringEncoding] : "";
+	const char *fileName = [[fileURL path] cStringUsingEncoding:NSUTF8StringEncoding];
+	
+	NSDate *currentDate = [NSDate date];
+	NSString *currentDateStr = [currentDate descriptionWithCalendarFormat:@"%Y %m %d %H %M %S %F"
+																 timeZone:nil
+																   locale:[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]];
+	
+	int dateYear = 2009;
+	int dateMonth = 1;
+	int dateDay = 1;
+	int dateHour = 0;
+	int dateMinute = 0;
+	int dateSecond = 0;
+	int dateMillisecond = 0;
+	const char *dateCStr = [currentDateStr cStringUsingEncoding:NSUTF8StringEncoding];
+	sscanf(dateCStr, "%i %i %i %i %i %i %i", &dateYear, &dateMonth, &dateDay, &dateHour, &dateMinute, &dateSecond, &dateMillisecond);
+	
+	DateTime rtcDate = DateTime(dateYear,
+								dateMonth,
+								dateDay,
+								dateHour,
+								dateMinute,
+								dateSecond,
+								dateMillisecond);
+	
+	FCEUI_SaveMovie(fileName, L"Test Author", 0, sramPath, rtcDate);
+	
+	return YES;
+}
+
+- (void) stopReplay
+{
+	FCEUI_StopMovie();
+}
+
 @end
 
 static void* RunCoreThread(void *arg)
@@ -827,6 +1079,7 @@ static void* RunCoreThread(void *arg)
 	CoreThreadParam *param = (CoreThreadParam *)arg;
 	CocoaDSCore *cdsCore = (CocoaDSCore *)param->cdsCore;
 	NSMutableArray *cdsOutputList = [cdsCore cdsOutputList];
+	NSUInteger frameNum = 0;
 	uint64_t startTime = 0;
 	uint64_t timeBudget = 0; // Need local variable to ensure that param->timeBudgetMachAbsTime is thread-safe.
 	
@@ -836,32 +1089,33 @@ static void* RunCoreThread(void *arg)
 		pthread_mutex_lock(&param->mutexThreadExecute);
 		timeBudget = param->timeBudgetMachAbsTime;
 		
-		while (!(param->state == CORESTATE_EXECUTE && execute && !param->exitThread))
+		while (!(param->state != CORESTATE_PAUSE && execute))
 		{
 			pthread_cond_wait(&param->condThreadExecute, &param->mutexThreadExecute);
 			startTime = mach_absolute_time();
 			timeBudget = param->timeBudgetMachAbsTime;
 		}
 		
-		if (param->exitThread)
+		CocoaDSController *cdsController = [cdsCore cdsController];
+		if (param->state != CORESTATE_FRAMEJUMP)
 		{
-			pthread_mutex_unlock(&param->mutexThreadExecute);
-			break;
+			[cdsController flush];
+		}
+		else
+		{
+			[cdsController flushEmpty];
 		}
 		
-		// Get the user's input, execute a single emulation frame, and generate
-		// the frame output.
-		[(CocoaDSController *)param->cdsController flush];
-		
 		NDS_beginProcessingInput();
-		// Shouldn't need to do any special processing steps in between.
-		// We'll just jump directly to ending the input processing.
+		FCEUMOV_HandlePlayback();
 		NDS_endProcessingInput();
+		FCEUMOV_HandleRecording();
 		
 		// Execute the frame and increment the frame counter.
-		pthread_mutex_lock(&param->mutexCoreExecute);
+		pthread_rwlock_wrlock(&param->rwlockCoreExecute);
 		NDS_exec<false>();
-		pthread_mutex_unlock(&param->mutexCoreExecute);
+		frameNum = currFrameCounter;
+		pthread_rwlock_unlock(&param->rwlockCoreExecute);
 		
 		// Check if an internal execution error occurred that halted the emulation.
 		if (!execute)
@@ -877,81 +1131,274 @@ static void* RunCoreThread(void *arg)
 			param->frameCount++;
 		}
 		
+		// Make sure that the mic level is updated at least once per frame, regardless
+		// of whether the NDS actually reads the mic or not.
+		[cdsController updateMicLevel];
+		[cdsController clearMicLevelMeasure];
+		
 		pthread_mutex_lock(&param->mutexOutputList);
-		for(CocoaDSOutput *cdsOutput in cdsOutputList)
+		
+		switch (param->state)
 		{
-			if (param->framesToSkip > 0 && [cdsOutput isKindOfClass:[CocoaDSDisplay class]])
+			case CORESTATE_EXECUTE:
 			{
-				continue;
+				for(CocoaDSOutput *cdsOutput in cdsOutputList)
+				{
+					if (param->framesToSkip > 0 && [cdsOutput isKindOfClass:[CocoaDSDisplay class]])
+					{
+						continue;
+					}
+					
+					[cdsOutput doCoreEmuFrame];
+				}
+				break;
 			}
-			
-			[cdsOutput doCoreEmuFrame];
+				
+			case CORESTATE_FRAMEADVANCE:
+			{
+				for(CocoaDSOutput *cdsOutput in cdsOutputList)
+				{
+					[cdsOutput doCoreEmuFrame];
+				}
+				break;
+			}
+				
+			case CORESTATE_FRAMEJUMP:
+			{
+				for(CocoaDSOutput *cdsOutput in cdsOutputList)
+				{
+					if ([cdsOutput isKindOfClass:[CocoaDSDisplay class]] && (param->framesToSkip == 0 || frameNum >= param->frameJumpTarget))
+					{
+						[cdsOutput doCoreEmuFrame];
+					}
+				}
+				break;
+			}
+				
+			default:
+				break;
 		}
+		
 		pthread_mutex_unlock(&param->mutexOutputList);
 		
-		// Determine the number of frames to skip based on how much time "debt"
-		// we owe on timeBudget.
-		if (param->isFrameSkipEnabled)
+		switch (param->state)
 		{
-			CoreFrameSkip(timeBudget, startTime, &param->framesToSkip);
+			case CORESTATE_EXECUTE:
+			{
+				// Determine the number of frames to skip based on how much time "debt"
+				// we owe on timeBudget.
+				if (param->isFrameSkipEnabled)
+				{
+					if (param->framesToSkip > 0)
+					{
+						NDS_SkipNextFrame();
+						param->framesToSkip--;
+					}
+					else
+					{
+						param->framesToSkip = CalculateFrameSkip(timeBudget, startTime);
+					}
+				}
+				break;
+			}
+			
+			case CORESTATE_FRAMEJUMP:
+			{
+				if (param->framesToSkip > 0)
+				{
+					NDS_SkipNextFrame();
+					param->framesToSkip--;
+				}
+				else
+				{
+					param->framesToSkip = (int)((DS_FRAMES_PER_SECOND * 1.0) + 0.85);
+				}
+				break;
+			}
+				
+			default:
+				break;
 		}
 		
 		pthread_mutex_unlock(&param->mutexThreadExecute);
 		
-		// If there is any time left in the loop, go ahead and pad it.
-		mach_wait_until(startTime + timeBudget);
-		
-	} while (!param->exitThread);
-	
-	return nil;
-}
-
-static void CoreFrameSkip(uint64_t timeBudgetMachAbsTime, uint64_t frameStartMachAbsTime, unsigned int *outFramesToSkip)
-{
-	if (*outFramesToSkip > 0)
-	{
-		NDS_SkipNextFrame();
-		(*outFramesToSkip)--;
-	}
-	else
-	{
-		// Calculate the time remaining.
-		uint64_t elapsed = mach_absolute_time() - frameStartMachAbsTime;
-		
-		if (elapsed > timeBudgetMachAbsTime)
+		// If we doing a frame advance, switch back to pause state immediately
+		// after we're done with the frame.
+		if (param->state == CORESTATE_FRAMEADVANCE)
 		{
-			static unsigned int lastSetFrameSkip = 0;
-			unsigned int framesToSkip = 0;
-			
-			if (timeBudgetMachAbsTime > 0)
+			[cdsCore setCoreState:CORESTATE_PAUSE];
+		}
+		else if (param->state == CORESTATE_FRAMEJUMP)
+		{
+			if (frameNum == (param->frameJumpTarget - 1))
 			{
-				framesToSkip = (unsigned int)( (((double)(elapsed - timeBudgetMachAbsTime) * FRAME_SKIP_AGGRESSIVENESS) / (double)timeBudgetMachAbsTime) + FRAME_SKIP_BIAS );
-				if (framesToSkip < lastSetFrameSkip)
-				{
-					framesToSkip += (unsigned int)((double)(lastSetFrameSkip - framesToSkip) * FRAME_SKIP_SMOOTHNESS);
-				}
-				
-				lastSetFrameSkip = framesToSkip;
+				param->framesToSkip = 0;
+				NDS_OmitFrameSkip(2);
 			}
-			else
+			else if (frameNum >= param->frameJumpTarget)
 			{
-				framesToSkip = (unsigned int)( (((double)(elapsed - timeBudgetMachAbsTime) * FRAME_SKIP_AGGRESSIVENESS * 100.0) / DS_SECONDS_PER_FRAME) + FRAME_SKIP_BIAS );
-				// Don't need to save lastSetFrameSkip here since this code path assumes that
-				// the frame limiter is disabled.
+				[cdsCore restoreCoreState];
 			}
-			
-			// Bound the frame skip.
-			if (framesToSkip > (unsigned int)MAX_FRAME_SKIP)
-			{
-				framesToSkip = (unsigned int)MAX_FRAME_SKIP;
-				lastSetFrameSkip = framesToSkip;
-			}
-			
-			*outFramesToSkip = framesToSkip;
 		}
 		else
 		{
-			*outFramesToSkip = 0;
+			// If there is any time left in the loop, go ahead and pad it.
+			mach_wait_until(startTime + timeBudget);
 		}
+		
+	} while(true);
+	
+	return NULL;
+}
+
+static int CalculateFrameSkip(uint64_t timeBudgetMachAbsTime, uint64_t frameStartMachAbsTime)
+{
+	static const double skipCurve[10]	= {0.60, 0.58, 0.55, 0.51, 0.46, 0.40, 0.30, 0.20, 0.10, 0.00};
+	static const double unskipCurve[10]	= {0.75, 0.70, 0.65, 0.60, 0.50, 0.40, 0.30, 0.20, 0.10, 0.00};
+	static size_t skipStep = 0;
+	static size_t unskipStep = 0;
+	static int lastSetFrameSkip = 0;
+	
+	// Calculate the time remaining.
+	const uint64_t elapsed = mach_absolute_time() - frameStartMachAbsTime;
+	int framesToSkip = 0;
+	
+	if (elapsed > timeBudgetMachAbsTime)
+	{
+		if (timeBudgetMachAbsTime > 0)
+		{
+			framesToSkip = (int)( (((double)(elapsed - timeBudgetMachAbsTime) * FRAME_SKIP_AGGRESSIVENESS) / (double)timeBudgetMachAbsTime) + FRAME_SKIP_BIAS );
+			
+			if (framesToSkip > lastSetFrameSkip)
+			{
+				framesToSkip -= (int)((double)(framesToSkip - lastSetFrameSkip) * skipCurve[skipStep]);
+				if (skipStep < 9)
+				{
+					skipStep++;
+				}
+			}
+			else
+			{
+				framesToSkip += (int)((double)(lastSetFrameSkip - framesToSkip) * skipCurve[skipStep]);
+				if (skipStep > 0)
+				{
+					skipStep--;
+				}
+			}
+		}
+		else
+		{
+			static const double frameRate100x = (double)FRAME_SKIP_AGGRESSIVENESS / (double)GetFrameAbsoluteTime(1.0/100.0);
+			framesToSkip = (int)((double)elapsed * frameRate100x);
+		}
+		
+		unskipStep = 0;
+	}
+	else
+	{
+		framesToSkip = (int)((double)lastSetFrameSkip * unskipCurve[unskipStep]);
+		if (unskipStep < 9)
+		{
+			unskipStep++;
+		}
+		
+		skipStep = 0;
+	}
+	
+	// Bound the frame skip.
+	static const int kMaxFrameSkip = (int)MAX_FRAME_SKIP;
+	if (framesToSkip > kMaxFrameSkip)
+	{
+		framesToSkip = kMaxFrameSkip;
+	}
+	
+	lastSetFrameSkip = framesToSkip;
+	
+	return framesToSkip;
+}
+
+uint64_t GetFrameAbsoluteTime(const double frameTimeScalar)
+{
+	const uint64_t frameTimeNanoseconds = (uint64_t)(DS_SECONDS_PER_FRAME * 1000000000.0 * frameTimeScalar);
+	const AbsoluteTime frameTimeAbsTime = NanosecondsToAbsolute(*(Nanoseconds *)&frameTimeNanoseconds);
+	
+	return *(uint64_t *)&frameTimeAbsTime;
+}
+
+#pragma mark - OSXDriver
+
+pthread_mutex_t* OSXDriver::GetCoreThreadMutexLock()
+{
+	return this->mutexThreadExecute;
+}
+
+void OSXDriver::SetCoreThreadMutexLock(pthread_mutex_t *theMutex)
+{
+	this->mutexThreadExecute = theMutex;
+}
+
+pthread_rwlock_t* OSXDriver::GetCoreExecuteRWLock()
+{
+	return this->rwlockCoreExecute;
+}
+
+void OSXDriver::SetCoreExecuteRWLock(pthread_rwlock_t *theRwLock)
+{
+	this->rwlockCoreExecute = theRwLock;
+}
+
+void OSXDriver::SetCore(CocoaDSCore *theCore)
+{
+	this->cdsCore = theCore;
+}
+
+void OSXDriver::EMU_DebugIdleEnter()
+{
+	[this->cdsCore setIsInDebugTrap:YES];
+	pthread_rwlock_unlock(this->rwlockCoreExecute);
+	pthread_mutex_unlock(this->mutexThreadExecute);
+}
+
+void OSXDriver::EMU_DebugIdleUpdate()
+{
+	usleep(50);
+}
+
+void OSXDriver::EMU_DebugIdleWakeUp()
+{
+	pthread_mutex_lock(this->mutexThreadExecute);
+	pthread_rwlock_wrlock(this->rwlockCoreExecute);
+	[this->cdsCore setIsInDebugTrap:NO];
+}
+
+#pragma mark - GDB Stub implementation
+
+void* createThread_gdb(void (*thread_function)( void *data),void *thread_data)
+{
+	// Create the thread using POSIX routines.
+	pthread_attr_t  attr;
+	pthread_t*      posixThreadID = (pthread_t*)malloc(sizeof(pthread_t));
+	
+	assert(!pthread_attr_init(&attr));
+	assert(!pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE));
+	
+	int threadError = pthread_create(posixThreadID, &attr, (void* (*)(void *))thread_function, thread_data);
+	
+	assert(!pthread_attr_destroy(&attr));
+	
+	if (threadError != 0)
+	{
+		// Report an error.
+		return NULL;
+	}
+	else
+	{
+		return posixThreadID;
 	}
 }
+
+void joinThread_gdb(void *thread_handle)
+{
+	pthread_join(*((pthread_t*)thread_handle), NULL);
+	free(thread_handle);
+}
