@@ -34,6 +34,7 @@
 + (BOOL) loadState:(NSURL *)saveStateURL;
 + (BOOL) saveState:(NSURL *)saveStateURL;
 + (BOOL) loadRom:(NSURL *)romURL;
++ (BOOL) loadReplay:(NSURL *)replayURL;
 + (BOOL) importRomSave:(NSURL *)romSaveURL;
 + (BOOL) exportRomSaveToURL:(NSURL *)destinationURL romSaveURL:(NSURL *)romSaveURL fileType:(NSInteger)fileTypeID;
 + (NSURL *) romSaveURLFromRomURL:(NSURL *)romURL;
@@ -69,9 +70,6 @@
 #if MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_4
 + (BOOL) moveFileToCurrentDirectory:(NSURL *)fileURL;
 + (BOOL) copyFileToCurrentDirectory:(NSURL *)fileURL;
-+ (BOOL) moveFileListToCurrent:(NSMutableArray *)fileList;
-+ (BOOL) copyFileListToCurrent:(NSMutableArray *)fileList;
-+ (NSMutableArray *) completeFileList;
 + (NSMutableArray *) appFileList:(NSURL *)directoryURL;
 + (NSMutableArray *) appFileList:(NSURL *)directoryURL fileKind:(NSString *)theFileKind;
 #endif
