@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012 DeSmuME team
+	Copyright (C) 2012-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 
 #import "cocoa_firmware.h"
 #include "../NDSSystem.h"
+#include "../firmware.h"
+#undef BOOL
 
 
 @implementation CocoaDSFirmware
