@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012-2013 DeSmuME team
+	Copyright (C) 2012-2015 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -34,6 +34,10 @@
 #define NSSTRING_TITLE_SELECT_ARM7_IMAGE_PANEL		NSLocalizedString(@"Select ARM7 BIOS Image", nil)
 #define NSSTRING_TITLE_SELECT_ARM9_IMAGE_PANEL		NSLocalizedString(@"Select ARM9 BIOS Image", nil)
 #define NSSTRING_TITLE_SELECT_FIRMWARE_IMAGE_PANEL	NSLocalizedString(@"Select Firmware Image", nil)
+#define NSSTRING_TITLE_SELECT_MPCF_FOLDER_PANEL		NSLocalizedString(@"Select MPCF Folder", nil)
+#define NSSTRING_TITLE_SELECT_MPCF_DISK_IMAGE_PANEL	NSLocalizedString(@"Select MPCF Disk Image", nil)
+#define NSSTRING_TITLE_CHOOSE_GBA_CARTRIDGE_PANEL	NSLocalizedString(@"Choose GBA Cartridge", nil)
+#define NSSTRING_TITLE_CHOOSE_GBA_SRAM_PANEL		NSLocalizedString(@"Choose GBA SRAM File", nil)
 #define NSSTRING_TITLE_SAVE_SCREENSHOT_PANEL		NSLocalizedString(@"Save Screenshot", nil)
 
 #define NSSTRING_TITLE_EXECUTE_CONTROL				NSLocalizedString(@"Execute", nil)
@@ -84,6 +88,9 @@
 #define NSSTRING_STATUS_ROM_UNLOADED				NSLocalizedString(@"ROM unloaded.", nil)
 #define NSSTRING_STATUS_EMULATOR_RESETTING			NSLocalizedString(@"Emulator resetting...", nil)
 #define NSSTRING_STATUS_EMULATOR_RESET				NSLocalizedString(@"Emulator reset.", nil)
+#define NSSTRING_STATUS_AUTOHOLD_SET				NSLocalizedString(@"Autohold set...", nil)
+#define NSSTRING_STATUS_AUTOHOLD_SET_RELEASE		NSLocalizedString(@"Autohold set released.", nil)
+#define NSSTRING_STATUS_AUTOHOLD_CLEAR				NSLocalizedString(@"Autohold cleared.", nil)
 #define NSSTRING_STATUS_CANNOT_GENERATE_SAVE_PATH	NSLocalizedString(@"Cannot generate the save file path!", nil)
 #define NSSTRING_STATUS_CANNOT_CREATE_SAVE_DIRECTORY	NSLocalizedString(@"Cannot create the save directory!", nil)
 
@@ -107,7 +114,22 @@
 #define NSSTRING_STATUS_SLOT1_R4_INSERTED			NSLocalizedString(@"R4 cartridge interface inserted.", nil)
 #define NSSTRING_STATUS_SLOT1_STANDARD_INSERTED		NSLocalizedString(@"Standard retail cartridge inserted.", nil)
 
-#define NSSTRING_STATUS_AUTOLOAD_ROM_NAME_NONE		NSLocalizedString(@"No ROM chosen.", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_NONE			NSLocalizedString(@"No SLOT-2 device loaded.", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_AUTOMATIC		NSLocalizedString(@"Loaded SLOT-2 device using automatic selection.\nSelected device type: %@", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_MPCF_WITH_ROM		NSLocalizedString(@"Compact flash device loaded with data from the ROM's directory.", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_MPCF_DIRECTORY		NSLocalizedString(@"Compact flash device loaded with data from directory path:\n%s", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_MPCF_DISK_IMAGE	NSLocalizedString(@"Compact flash device loaded using disk image:\n%s", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_GBA_CART_WITH_SRAM	NSLocalizedString(@"GBA cartridge loaded with SRAM file:\n%s", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_GBA_CART_NO_SRAM	NSLocalizedString(@"GBA cartridge loaded. (No associated SRAM file loaded.)", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_GENERIC_DEVICE	NSLocalizedString(@"Loaded SLOT-2 device:\n%@", nil)
+#define NSSTRING_STATUS_SLOT2_LOADED_UNKNOWN		NSLocalizedString(@"An unknown SLOT-2 device has been loaded.", nil)
+
+#define NSSTRING_STATUS_NO_ROM_CHOSEN				NSLocalizedString(@"No ROM chosen.", nil)
+#define NSSTRING_STATUS_NO_FOLDER_CHOSEN			NSLocalizedString(@"No folder chosen.", nil)
+#define NSSTRING_STATUS_NO_DISK_IMAGE_CHOSEN		NSLocalizedString(@"No disk image chosen.", nil)
+#define NSSTRING_STATUS_NO_GBA_CART_CHOSEN			NSLocalizedString(@"No GBA cartridge chosen.", nil)
+#define NSSTRING_STATUS_NO_GBA_SRAM_CHOSEN			NSLocalizedString(@"No GBA SRAM chosen.", nil)
+#define NSSTRING_STATUS_NO_GBA_SRAM_FOUND			NSLocalizedString(@"No GBA SRAM found.", nil)
 
 #define NSSTRING_DISPLAYMODE_MAIN					NSLocalizedString(@"Main", nil)
 #define NSSTRING_DISPLAYMODE_TOUCH					NSLocalizedString(@"Touch", nil)
@@ -160,12 +182,15 @@
 #define FILE_EXT_CHEAT								"dct"
 #define FILE_EXT_ROM_SAVE_NOGBA						"sav*"
 #define FILE_EXT_ROM_SAVE_RAW						"sav"
-#define FILE_EXT_ACTION_REPLAY_SAVE					"duc"
+#define FILE_EXT_ACTION_REPLAY_SAVE					"dss"
+#define FILE_EXT_ACTION_REPLAY_MAX_SAVE				"duc"
 #define FILE_EXT_ROM_DS								"nds"
 #define FILE_EXT_ROM_GBA							"ds.gba"
 #define FILE_EXT_HW_IMAGE_FILE						"bin"
 #define FILE_EXT_ADVANSCENE_DB						"xml"
 #define FILE_EXT_R4_CHEAT_DB						"dat"
+#define FILE_EXT_GBA_ROM							"gba"
+#define FILE_EXT_GBA_SRAM							"sav"
 
 #define MAX_SAVESTATE_SLOTS							10
 
@@ -194,14 +219,12 @@
 #define DS_FRAMES_PER_SECOND						59.8261	// Number of DS frames per second.
 #define DS_SECONDS_PER_FRAME						(1.0 / DS_FRAMES_PER_SECOND) // The length of time in seconds that, ideally, a frame should be processed within.
 
-#define FRAME_SKIP_AGGRESSIVENESS					10.0	// Must be a value between 0.0 (inclusive) and positive infinity.
+#define FRAME_SKIP_AGGRESSIVENESS					9.0		// Must be a value between 0.0 (inclusive) and positive infinity.
 															// This value acts as a scalar multiple of the frame skip.
-#define FRAME_SKIP_SMOOTHNESS						0.90	// Must be a value between 0.00 (inclusive) and 1.00 (exclusive).
-															// Values closer to 0.00 give better video smoothness, but makes the emulation timing more "jumpy."
-															// Values closer to 1.00 makes the emulation timing more accurate, but makes the video look more "choppy."
-#define FRAME_SKIP_BIAS								0.5		// May be any real number. This value acts as a vector addition to the frame skip.
+#define FRAME_SKIP_BIAS								0.1		// May be any real number. This value acts as a vector addition to the frame skip.
 #define MAX_FRAME_SKIP								(DS_FRAMES_PER_SECOND / 3.0)
 
+//#define SPU_SAMPLE_RATE								(44100.0 * DS_FRAMES_PER_SECOND / 60.0)	// Samples per second
 #define SPU_SAMPLE_RATE								44100.0	// Samples per second
 #define SPU_SAMPLE_RESOLUTION						16		// Bits per sample; must be a multiple of 8
 #define SPU_NUMBER_CHANNELS							2		// Number of channels
@@ -220,14 +243,25 @@
 #define ROMINFO_GAME_CODE_LENGTH					4
 #define ROMINFO_GAME_BANNER_LENGTH					128
 
-#define MIC_SAMPLE_RATE								16000
-#define MIC_MAX_BUFFER_SAMPLES						(MIC_SAMPLE_RATE / DS_FRAMES_PER_SECOND)
+#define MIC_SAMPLE_RATE								16000.0
+#define MIC_SAMPLE_RESOLUTION						8		// Bits per sample; must be a multiple of 8
+#define MIC_NUMBER_CHANNELS							1		// Number of channels
+#define MIC_SAMPLE_SIZE								((MIC_SAMPLE_RESOLUTION / 8) * MIC_NUMBER_CHANNELS) // Bytes per sample, multiplied by the number of channels
+#define MIC_MAX_BUFFER_SAMPLES						((MIC_SAMPLE_RATE / DS_FRAMES_PER_SECOND) * MIC_SAMPLE_SIZE)
+#define MIC_CAPTURE_FRAMES							192		// The number of audio frames that the NDS microphone should pull. The lower this value, the lower the latency. Ensure that this value is not too low, or else audio frames may be lost.
+#define MIC_NULL_LEVEL_THRESHOLD					2.5
+#define MIC_CLIP_LEVEL_THRESHOLD					39.743665431525209 // ((2.0/pi) * MIC_NULL_SAMPLE_VALUE) - 1.0
+#define MIC_NULL_SAMPLE_VALUE						64
 
 #define COCOA_DIALOG_CANCEL							0
 #define COCOA_DIALOG_DEFAULT						1
 #define COCOA_DIALOG_OK								1
 #define COCOA_DIALOG_OPTION							2
 
+#define RUMBLE_ITERATIONS_RUMBLE_PAK				2
+#define RUMBLE_ITERATIONS_ENABLE					1
+#define RUMBLE_ITERATIONS_TEST						3
+
 enum
 {
 	ROMAUTOLOADOPTION_LOAD_LAST						= 0,
@@ -236,6 +270,15 @@ enum
 	ROMAUTOLOADOPTION_CHOOSE_ROM					= 10001
 };
 
+enum
+{
+	MPCF_OPTION_LOAD_WITH_ROM						= 0,
+	MPCF_OPTION_LOAD_DIRECTORY						= 1,
+	MPCF_OPTION_LOAD_DISK_IMAGE						= 2,
+	MPCF_ACTION_CHOOSE_DIRECTORY					= 10000,
+	MPCF_ACTION_CHOOSE_DISK_IMAGE					= 10001
+};
+
 enum
 {
 	REASONFORCLOSE_NORMAL = 0,
@@ -388,14 +431,12 @@ enum
 	MESSAGE_RESIZE_VIEW,
 	MESSAGE_TRANSFORM_VIEW,
 	MESSAGE_REDRAW_VIEW,
+	MESSAGE_REPROCESS_AND_REDRAW,
 	MESSAGE_SET_GPU_STATE_FLAGS,
 	MESSAGE_CHANGE_DISPLAY_TYPE,
 	MESSAGE_CHANGE_DISPLAY_ORIENTATION,
 	MESSAGE_CHANGE_DISPLAY_ORDER,
 	MESSAGE_CHANGE_DISPLAY_GAP,
-	MESSAGE_CHANGE_BILINEAR_OUTPUT,
-	MESSAGE_CHANGE_VERTICAL_SYNC,
-	MESSAGE_CHANGE_VIDEO_FILTER,
 	MESSAGE_SET_RENDER3D_METHOD,
 	MESSAGE_SET_RENDER3D_HIGH_PRECISION_COLOR_INTERPOLATION,
 	MESSAGE_SET_RENDER3D_EDGE_MARKING,
@@ -405,8 +446,6 @@ enum
 	MESSAGE_SET_RENDER3D_THREADS,
 	MESSAGE_SET_RENDER3D_LINE_HACK,
 	MESSAGE_SET_RENDER3D_MULTISAMPLE,
-	MESSAGE_SET_VIEW_TO_BLACK,
-	MESSAGE_SET_VIEW_TO_WHITE,
 	
 	MESSAGE_SET_AUDIO_PROCESS_METHOD,
 	MESSAGE_SET_SPU_ADVANCED_LOGIC,
@@ -429,6 +468,12 @@ enum
 	DS_DISPLAY_TYPE_DUAL
 };
 
+enum
+{
+	VIDEO_SOURCE_INTERNAL	= 0,
+	VIDEO_SOURCE_EMULATOR	= 1
+};
+
 enum
 {
 	DS_DISPLAY_ORIENTATION_VERTICAL = 0,
@@ -456,8 +501,10 @@ enum
  */
 enum
 {
-	CORESTATE_EXECUTE = 0,
-	CORESTATE_PAUSE
+	CORESTATE_PAUSE = 0,
+	CORESTATE_EXECUTE,
+	CORESTATE_FRAMEADVANCE,
+	CORESTATE_FRAMEJUMP
 };
 
 /*
@@ -482,3 +529,16 @@ enum
 	MICMODE_PHYSICAL,
 	MICMODE_SINE_WAVE
 };
+
+enum
+{
+	FRAMEJUMP_TYPE_FORWARD		= 0,
+	FRAMEJUMP_TYPE_TOFRAME		= 1,
+	FRAMEJUMP_TYPE_NEXTMARKER	= 2
+};
+
+enum
+{
+	PADDLE_CONTROL_RELATIVE = 0,
+	PADDLE_CONTROL_DIRECT
+};
