@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2011-2013 DeSmuME team
+	Copyright (C) 2011-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "sndOSX.h"
 
 #include "../NDSSystem.h"
+#include "../GPU.h"
 #include "../SPU.h"
 #include "../metaspu/metaspu.h"
 
@@ -35,11 +36,9 @@
 
 @synthesize isStateChanged;
 @synthesize frameCount;
-@synthesize frameData;
-@synthesize frameAttributesData;
 @synthesize property;
-@synthesize mutexProducer;
 @synthesize mutexConsume;
+@synthesize rwlockProducer;
 
 - (id)init
 {
@@ -51,8 +50,6 @@
 	
 	isStateChanged = NO;
 	frameCount = 0;
-	frameData = nil;
-	frameAttributesData = nil;
 	
 	property = [[NSMutableDictionary alloc] init];
 	[property setValue:[NSDate date] forKey:@"outputTime"];
@@ -68,8 +65,6 @@
 	// Force the thread to exit now so that we can safely release other resources.
 	[self forceThreadExit];
 	
-	self.frameData = nil;
-	self.frameAttributesData = nil;
 	[property release];
 	
 	pthread_mutex_destroy(mutexConsume);
@@ -92,7 +87,7 @@
 	switch (message)
 	{
 		case MESSAGE_EMU_FRAME_PROCESSED:
-			[self handleEmuFrameProcessed:[messageComponents objectAtIndex:0] attributes:[messageComponents objectAtIndex:1]];
+			[self handleEmuFrameProcessed];
 			break;
 			
 		default:
@@ -101,11 +96,9 @@
 	}
 }
 
-- (void) handleEmuFrameProcessed:(NSData *)mainData attributes:(NSData *)attributesData
+- (void) handleEmuFrameProcessed
 {
 	self.frameCount++;
-	self.frameData = mainData;
-	self.frameAttributesData = attributesData;
 }
 
 @end
@@ -134,6 +127,7 @@
 	spinlockSpuSyncMode = OS_SPINLOCK_INIT;
 	spinlockSpuSyncMethod = OS_SPINLOCK_INIT;
 	
+	_idleState = YES;
 	bufferSize = 0;
 	
 	// Set up properties.
@@ -154,6 +148,20 @@
 	[super dealloc];
 }
 
+- (void) setIdle:(BOOL)theState
+{
+	if (theState)
+	{
+		SNDOSXPauseAudio();
+	}
+	else
+	{
+		SNDOSXUnpauseAudio();
+	}
+	
+	[super setIdle:theState];
+}
+
 - (void) setVolume:(float)vol
 {
 	if (vol < 0.0f)
@@ -187,7 +195,7 @@
 	[property setValue:[NSNumber numberWithInteger:methodID] forKey:@"audioOutputEngine"];
 	OSSpinLockUnlock(&spinlockAudioOutputEngine);
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	
 	NSInteger result = -1;
 	
@@ -201,9 +209,9 @@
 		SPU_ChangeSoundCore(SNDCORE_DUMMY, 0);
 	}
 	
-	mutexAudioEmulateCore = self.mutexProducer;
+	rwlockAudioEmulateCore = self.rwlockProducer;
 	
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	// Force the volume back to it's original setting.
 	[self setVolume:[self volume]];
@@ -224,9 +232,9 @@
 	[property setValue:[NSNumber numberWithBool:state] forKey:@"spuAdvancedLogic"];
 	OSSpinLockUnlock(&spinlockSpuAdvancedLogic);
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.spu_advanced = state;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (BOOL) spuAdvancedLogic
@@ -244,9 +252,9 @@
 	[property setValue:[NSNumber numberWithInteger:modeID] forKey:@"spuInterpolationMode"];
 	OSSpinLockUnlock(&spinlockSpuInterpolationMode);
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.spuInterpolationMode = (SPUInterpolationMode)modeID;
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (NSInteger) spuInterpolationMode
@@ -264,10 +272,10 @@
 	[property setValue:[NSNumber numberWithInteger:modeID] forKey:@"spuSyncMode"];
 	OSSpinLockUnlock(&spinlockSpuSyncMode);
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.SPU_sync_mode = (int)modeID;
 	SPU_SetSynchMode(CommonSettings.SPU_sync_mode, CommonSettings.SPU_sync_method);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (NSInteger) spuSyncMode
@@ -285,10 +293,10 @@
 	[property setValue:[NSNumber numberWithInteger:methodID] forKey:@"spuSyncMethod"];
 	OSSpinLockUnlock(&spinlockSpuSyncMethod);
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_wrlock(self.rwlockProducer);
 	CommonSettings.SPU_sync_method = (int)methodID;
 	SPU_SetSynchMode(CommonSettings.SPU_sync_mode, CommonSettings.SPU_sync_method);
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 }
 
 - (NSInteger) spuSyncMethod
@@ -333,19 +341,19 @@
 {
 	NSString *theString = @"Uninitialized";
 	
-	pthread_mutex_lock(self.mutexProducer);
+	pthread_rwlock_rdlock(self.rwlockProducer);
 	
 	SoundInterface_struct *soundCore = SPU_SoundCore();
 	if(soundCore == NULL)
 	{
-		pthread_mutex_unlock(self.mutexProducer);
+		pthread_rwlock_unlock(self.rwlockProducer);
 		return theString;
 	}
 	
 	const char *theName = soundCore->Name;
 	theString = [NSString stringWithCString:theName encoding:NSUTF8StringEncoding];
 	
-	pthread_mutex_unlock(self.mutexProducer);
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 	return theString;
 }
@@ -417,10 +425,6 @@
 	
 	switch (message)
 	{
-		case MESSAGE_EMU_FRAME_PROCESSED:
-			[self handleEmuFrameProcessed:nil attributes:nil];
-			break;
-			
 		case MESSAGE_SET_AUDIO_PROCESS_METHOD:
 			[self handleSetAudioOutputEngine:[messageComponents objectAtIndex:0]];
 			break;
@@ -451,11 +455,11 @@
 	}
 }
 
-- (void) handleEmuFrameProcessed:(NSData *)mainData attributes:(NSData *)attributesData
+- (void) handleEmuFrameProcessed
 {
 	SPU_Emulate_user();
 	
-	[super handleEmuFrameProcessed:mainData attributes:attributesData];
+	[super handleEmuFrameProcessed];
 }
 
 - (void) handleSetVolume:(NSData *)volumeData
@@ -498,7 +502,7 @@
 
 @implementation CocoaDSDisplay
 
-@dynamic delegate;
+@synthesize delegate;
 @dynamic displayMode;
 @dynamic frameSize;
 
@@ -510,14 +514,19 @@
 	{
 		return self;
 	}
-		
-	spinlockDelegate = OS_SPINLOCK_INIT;
+	
 	spinlockDisplayType = OS_SPINLOCK_INIT;
 	
 	delegate = nil;
 	displayMode = DS_DISPLAY_TYPE_DUAL;
 	frameSize = NSMakeSize((CGFloat)GPU_DISPLAY_WIDTH, (CGFloat)GPU_DISPLAY_HEIGHT * 2);
 	
+	_gpuFrame.buffer = (uint16_t *)GPU_screen;
+	_gpuFrame.bufferSize = GPU_DISPLAY_WIDTH * GPU_DISPLAY_HEIGHT * 2 * sizeof(uint16_t);
+	_gpuFrame.displayModeID = DS_DISPLAY_TYPE_DUAL;
+	_gpuFrame.width = frameSize.width;
+	_gpuFrame.height = frameSize.height;
+	
 	[property setValue:[NSNumber numberWithInteger:displayMode] forKey:@"displayMode"];
 	[property setValue:NSSTRING_DISPLAYMODE_MAIN forKey:@"displayModeString"];
 	
@@ -526,41 +535,11 @@
 
 - (void)dealloc
 {
-	self.delegate = nil;
-			
+	[self setDelegate:nil];
+	
 	[super dealloc];
 }
 
-- (void) setDelegate:(id <CocoaDSDisplayDelegate>)theDelegate
-{
-	OSSpinLockLock(&spinlockDelegate);
-	
-	if (theDelegate == delegate)
-	{
-		OSSpinLockUnlock(&spinlockDelegate);
-		return;
-	}
-	
-	if (theDelegate != nil)
-	{
-		[theDelegate retain];
-	}
-	
-	[delegate release];
-	delegate = theDelegate;
-	
-	OSSpinLockUnlock(&spinlockDelegate);
-}
-
-- (id <CocoaDSDisplayDelegate>) delegate
-{
-	OSSpinLockLock(&spinlockDelegate);
-	id <CocoaDSDisplayDelegate> theDelegate = delegate;
-	OSSpinLockUnlock(&spinlockDelegate);
-	
-	return theDelegate;
-}
-
 - (void) setDisplayMode:(NSInteger)displayModeID
 {
 	NSString *newDispString = nil;
@@ -612,41 +591,6 @@
 	return size;
 }
 
-- (void) doCoreEmuFrame
-{
-	NSData *gpuData = nil;
-	NSInteger displayModeID = [self displayMode];
-	NSSize displayFrameSize = [self frameSize];
-	
-	// Here, we copy the raw GPU data from the emulation core.
-	// 
-	// The core data contains the GPU pixels from both the main and touch screens. So
-	// depending on the display type, we copy only the pixels from the respective screen.
-	if (displayModeID == DS_DISPLAY_TYPE_MAIN)
-	{
-		gpuData = [[NSData alloc] initWithBytes:GPU_screen length:GPU_SCREEN_SIZE_BYTES];
-	}
-	else if(displayModeID == DS_DISPLAY_TYPE_TOUCH)
-	{
-		gpuData = [[NSData alloc] initWithBytes:(GPU_screen + GPU_SCREEN_SIZE_BYTES) length:GPU_SCREEN_SIZE_BYTES];
-	}
-	else if(displayModeID == DS_DISPLAY_TYPE_DUAL)
-	{
-		gpuData = [[NSData alloc] initWithBytes:GPU_screen length:GPU_SCREEN_SIZE_BYTES * 2];
-	}
-	
-	DisplaySrcPixelAttributes attr = {displayModeID, (size_t)displayFrameSize.width, (size_t)displayFrameSize.height};
-	NSData *attributesData = [[NSData alloc] initWithBytes:&attr length:sizeof(DisplaySrcPixelAttributes)];
-	
-	NSArray *messageComponents = [[NSArray alloc] initWithObjects:gpuData, attributesData, nil];
-	[CocoaDSUtil messageSendOneWayWithMessageComponents:self.receivePort msgID:MESSAGE_EMU_FRAME_PROCESSED array:messageComponents];
-	
-	// Now that we've finished sending the GPU data, release the local copy.
-	[gpuData release];
-	[attributesData release];
-	[messageComponents release];
-}
-
 - (void)handlePortMessage:(NSPortMessage *)portMessage
 {
 	NSInteger message = (NSInteger)[portMessage msgid];
@@ -654,22 +598,10 @@
 	
 	switch (message)
 	{
-		case MESSAGE_EMU_FRAME_PROCESSED:
-			[self handleEmuFrameProcessed:[messageComponents objectAtIndex:0] attributes:[messageComponents objectAtIndex:1]];
-			break;
-			
 		case MESSAGE_CHANGE_DISPLAY_TYPE:
 			[self handleChangeDisplayMode:[messageComponents objectAtIndex:0]];
 			break;
 			
-		case MESSAGE_SET_VIEW_TO_BLACK:
-			[self handleSetViewToBlack];
-			break;
-			
-		case MESSAGE_SET_VIEW_TO_WHITE:
-			[self handleSetViewToWhite];
-			break;
-			
 		case MESSAGE_REQUEST_SCREENSHOT:
 			[self handleRequestScreenshot:[messageComponents objectAtIndex:0] fileTypeData:[messageComponents objectAtIndex:1]];
 			break;
@@ -684,9 +616,34 @@
 	}
 }
 
-- (void) handleEmuFrameProcessed:(NSData *)mainData attributes:(NSData *)attributesData
+- (void) handleEmuFrameProcessed
 {
-	[super handleEmuFrameProcessed:mainData attributes:attributesData];
+	_gpuFrame.displayModeID = [self displayMode];
+	_gpuFrame.width = [self frameSize].width;
+	_gpuFrame.height = [self frameSize].height;
+	
+	switch (_gpuFrame.displayModeID)
+	{
+		case DS_DISPLAY_TYPE_MAIN:
+			_gpuFrame.buffer = (uint16_t *)GPU_screen;
+			_gpuFrame.bufferSize = GPU_SCREEN_SIZE_BYTES;
+			break;
+			
+		case DS_DISPLAY_TYPE_TOUCH:
+			_gpuFrame.buffer = (uint16_t *)GPU_screen + (GPU_DISPLAY_WIDTH * GPU_DISPLAY_HEIGHT);
+			_gpuFrame.bufferSize = GPU_SCREEN_SIZE_BYTES;
+			break;
+			
+		case DS_DISPLAY_TYPE_DUAL:
+			_gpuFrame.buffer = (uint16_t *)GPU_screen;
+			_gpuFrame.bufferSize = GPU_SCREEN_SIZE_BYTES * 2;
+			break;
+			
+		default:
+			break;
+	}
+	
+	[super handleEmuFrameProcessed];
 }
 
 - (void) handleChangeDisplayMode:(NSData *)displayModeData
@@ -697,20 +654,10 @@
 	}
 	
 	const NSInteger displayModeID = *(NSInteger *)[displayModeData bytes];
-	self.displayMode = displayModeID;
+	[self setDisplayMode:displayModeID];
 	[delegate doDisplayModeChanged:displayModeID];
 }
 
-- (void) handleSetViewToBlack
-{
-	[self fillVideoFrameWithColor:0x8000];
-}
-
-- (void) handleSetViewToWhite
-{
-	[self fillVideoFrameWithColor:0xFFFF];
-}
-
 - (void) handleRequestScreenshot:(NSData *)fileURLStringData fileTypeData:(NSData *)fileTypeData
 {
 	NSString *fileURLString = [[NSString alloc] initWithData:fileURLStringData encoding:NSUTF8StringEncoding];
@@ -742,32 +689,6 @@
 	[pboard setData:[screenshot TIFFRepresentationUsingCompression:NSTIFFCompressionLZW factor:1.0f] forType:NSTIFFPboardType];
 }
 
-- (void) fillVideoFrameWithColor:(UInt16)colorValue
-{
-	const NSInteger displayModeID = [self displayMode];
-	const NSSize displayFrameSize = [self frameSize];
-	const size_t numberBytes = (displayModeID == DS_DISPLAY_TYPE_DUAL) ? GPU_SCREEN_SIZE_BYTES * 2 : GPU_SCREEN_SIZE_BYTES;
-	
-	UInt16 *gpuBytes = (UInt16 *)malloc(numberBytes);
-	if (gpuBytes == NULL)
-	{
-		return;
-	}
-	
-	const UInt16 colorValuePattern[] = {colorValue, colorValue, colorValue, colorValue, colorValue, colorValue, colorValue, colorValue};
-	memset_pattern16(gpuBytes, colorValuePattern, numberBytes);
-	NSData *gpuData = [[NSData alloc] initWithBytes:gpuBytes length:numberBytes];
-	
-	free(gpuBytes);
-	gpuBytes = nil;
-	
-	DisplaySrcPixelAttributes attr = {displayModeID, (size_t)displayFrameSize.width, (size_t)displayFrameSize.height};
-	NSData *attributesData = [[[NSData alloc] initWithBytes:&attr length:sizeof(DisplaySrcPixelAttributes)] autorelease];
-	
-	[self handleEmuFrameProcessed:gpuData attributes:attributesData];
-	[gpuData release];
-}
-
 - (NSImage *) image
 {
 	NSImage *newImage = [[NSImage alloc] initWithSize:self.frameSize];
@@ -793,7 +714,7 @@
 
 - (NSBitmapImageRep *) bitmapImageRep
 {
-	if (self.frameData == nil)
+	if (_gpuFrame.buffer == NULL)
 	{
 		return nil;
 	}
@@ -818,7 +739,9 @@
 	}
 	
 	uint32_t *bitmapData = (uint32_t *)[imageRep bitmapData];
-	RGB555ToRGBA8888Buffer((const uint16_t *)[self.frameData bytes], bitmapData, (w * h));
+	pthread_rwlock_rdlock(self.rwlockProducer);
+	RGB555ToRGBA8888Buffer((const uint16_t *)_gpuFrame.buffer, bitmapData, (w * h));
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
 #ifdef __BIG_ENDIAN__
 	uint32_t *bitmapDataEnd = bitmapData + (w * h);
@@ -835,8 +758,6 @@
 
 @implementation CocoaDSDisplayVideo
 
-@synthesize vf;
-
 - (id)init
 {
 	self = [super init];
@@ -845,21 +766,6 @@
 		return self;
 	}
 	
-	videoDelegate = nil;
-	lastDisplayMode = DS_DISPLAY_TYPE_DUAL;
-	
-	spinlockVideoFilterType = OS_SPINLOCK_INIT;
-	spinlockVFBuffers = OS_SPINLOCK_INIT;
-	
-	if ([[NSProcessInfo processInfo] activeProcessorCount] >= 2)
-	{
-		vf = [[CocoaVideoFilter alloc] initWithSize:frameSize typeID:VideoFilterTypeID_None numberThreads:2];
-	}
-	else
-	{
-		vf = [[CocoaVideoFilter alloc] initWithSize:frameSize typeID:VideoFilterTypeID_None numberThreads:0];
-	}
-	
 	[property setValue:[NSNumber numberWithInteger:(NSInteger)VideoFilterTypeID_None] forKey:@"videoFilterType"];
 	[property setValue:[CocoaVideoFilter typeStringByID:VideoFilterTypeID_None] forKey:@"videoFilterTypeString"];
 	
@@ -868,68 +774,13 @@
 
 - (void)dealloc
 {
-	[vf release];
-	
 	[super dealloc];
 }
 
-- (void) setDelegate:(id <CocoaDSDisplayVideoDelegate>)theDelegate
-{
-	OSSpinLockLock(&spinlockDelegate);
-	
-	if (theDelegate == videoDelegate)
-	{
-		OSSpinLockUnlock(&spinlockDelegate);
-		return;
-	}
-	
-	if (theDelegate != nil)
-	{
-		[theDelegate retain];
-	}
-	
-	[videoDelegate release];
-	videoDelegate = theDelegate;
-	
-	OSSpinLockUnlock(&spinlockDelegate);
-	
-	[super setDelegate:theDelegate];
-}
-
-- (id <CocoaDSDisplayVideoDelegate>) delegate
-{
-	OSSpinLockLock(&spinlockDelegate);
-	id <CocoaDSDisplayVideoDelegate> theDelegate = videoDelegate;
-	OSSpinLockUnlock(&spinlockDelegate);
-	
-	return theDelegate;
-}
-
-- (void) setVfType:(NSInteger)videoFilterTypeID
-{
-	OSSpinLockLock(&spinlockVFBuffers);
-	[vf changeFilter:(VideoFilterTypeID)videoFilterTypeID];
-	OSSpinLockUnlock(&spinlockVFBuffers);
-	
-	OSSpinLockLock(&spinlockVideoFilterType);
-	[property setValue:[NSNumber numberWithInteger:videoFilterTypeID] forKey:@"videoFilterType"];
-	[property setValue:NSLocalizedString([vf typeString], nil) forKey:@"videoFilterTypeString"];
-	OSSpinLockUnlock(&spinlockVideoFilterType);
-}
-
-- (NSInteger) vfType
-{
-	OSSpinLockLock(&spinlockVideoFilterType);
-	NSInteger theType = [(NSNumber *)[property valueForKey:@"videoFilterType"] integerValue];
-	OSSpinLockUnlock(&spinlockVideoFilterType);
-	
-	return theType;
-}
-
 - (void) runThread:(id)object
 {
 	NSAutoreleasePool *tempPool = [[NSAutoreleasePool alloc] init];
-	[videoDelegate doInitVideoOutput:self.property];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doInitVideoOutput:self.property];
 	[tempPool release];
 	
 	[super runThread:object];
@@ -942,8 +793,8 @@
 	
 	switch (message)
 	{
-		case MESSAGE_EMU_FRAME_PROCESSED:
-			[self handleEmuFrameProcessed:[messageComponents objectAtIndex:0] attributes:[messageComponents objectAtIndex:1]];
+		case MESSAGE_REPROCESS_AND_REDRAW:
+			[self handleReprocessAndRedraw];
 			break;
 			
 		case MESSAGE_RESIZE_VIEW:
@@ -970,161 +821,93 @@
 			[self handleChangeDisplayGap:[messageComponents objectAtIndex:0]];
 			break;
 			
-		case MESSAGE_CHANGE_BILINEAR_OUTPUT:
-			[self handleChangeBilinearOutput:[messageComponents objectAtIndex:0]];
-			break;
-			
-		case MESSAGE_CHANGE_VERTICAL_SYNC:
-			[self handleChangeVerticalSync:[messageComponents objectAtIndex:0]];
-			break;
-			
-		case MESSAGE_CHANGE_VIDEO_FILTER:
-			[self handleChangeVideoFilter:[messageComponents objectAtIndex:0]];
-			break;
-			
 		default:
 			[super handlePortMessage:portMessage];
 			break;
 	}
 }
 
-- (void) handleEmuFrameProcessed:(NSData *)mainData attributes:(NSData *)attributesData
+- (void) handleEmuFrameProcessed
 {
-	if (mainData == nil || attributesData == nil)
-	{
-		return;
-	}
+	[super handleEmuFrameProcessed];
 	
-	const DisplaySrcPixelAttributes attr = *(DisplaySrcPixelAttributes *)[attributesData bytes];
-	const NSInteger displayModeID = attr.displayModeID;
+	pthread_rwlock_rdlock(self.rwlockProducer);
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doLoadVideoFrame:_gpuFrame.buffer
+													displayMode:_gpuFrame.displayModeID
+														  width:_gpuFrame.width
+														 height:_gpuFrame.height];
+	pthread_rwlock_unlock(self.rwlockProducer);
 	
-	// Tell the video delegate to process the video frame with our copied GPU data.
-	OSSpinLockLock(&spinlockVFBuffers);
-	
-	if (lastDisplayMode != displayModeID)
-	{
-		const NSSize newSrcSize = NSMakeSize((CGFloat)attr.width, (CGFloat)attr.height);
-		[vf setSourceSize:newSrcSize];
-		lastDisplayMode = displayModeID;
-	}
-	
-	const NSInteger destWidth = (NSInteger)[vf destSize].width;
-	const NSInteger destHeight = (NSInteger)[vf destSize].height;
-	
-	if ([vf typeID] == VideoFilterTypeID_None)
-	{
-		[videoDelegate doProcessVideoFrame:[mainData bytes] displayMode:displayModeID width:destWidth height:destHeight];
-	}
-	else
-	{
-		RGB555ToRGBA8888Buffer((const uint16_t *)[mainData bytes], (uint32_t *)[vf srcBufferPtr], [mainData length] / sizeof(UInt16));
-		const UInt32 *vfDestBuffer = [vf runFilter];
-		[videoDelegate doProcessVideoFrame:vfDestBuffer displayMode:displayModeID width:destWidth height:destHeight];
-	}
-	
-	OSSpinLockUnlock(&spinlockVFBuffers);
-		
-	[super handleEmuFrameProcessed:mainData attributes:attributesData];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doProcessVideoFrame];
 }
 
 - (void) handleResizeView:(NSData *)rectData
 {
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doResizeView:)])
+	if (delegate == nil || ![delegate respondsToSelector:@selector(doResizeView:)])
 	{
 		return;
 	}
 	
 	const NSRect resizeRect = *(NSRect *)[rectData bytes];
-	[videoDelegate doResizeView:resizeRect];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doResizeView:resizeRect];
 }
 
 - (void) handleTransformView:(NSData *)transformData
 {
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doTransformView:)])
+	if (delegate == nil || ![delegate respondsToSelector:@selector(doTransformView:)])
 	{
 		return;
 	}
 	
-	[videoDelegate doTransformView:(DisplayOutputTransformData *)[transformData bytes]];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doTransformView:(DisplayOutputTransformData *)[transformData bytes]];
 }
 
 - (void) handleRedrawView
 {
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doRedraw)])
+	if (delegate == nil || ![delegate respondsToSelector:@selector(doRedraw)])
 	{
 		return;
 	}
 	
-	[videoDelegate doRedraw];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doRedraw];
+}
+
+- (void) handleReprocessAndRedraw
+{
+	[self handleEmuFrameProcessed];
 }
 
 - (void) handleChangeDisplayOrientation:(NSData *)displayOrientationIdData
 {
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doDisplayOrientationChanged:)])
+	if (delegate == nil || ![delegate respondsToSelector:@selector(doDisplayOrientationChanged:)])
 	{
 		return;
 	}
 	
 	const NSInteger theOrientation = *(NSInteger *)[displayOrientationIdData bytes];
-	[videoDelegate doDisplayOrientationChanged:theOrientation];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doDisplayOrientationChanged:theOrientation];
 }
 
 - (void) handleChangeDisplayOrder:(NSData *)displayOrderIdData
 {
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doDisplayOrderChanged:)])
+	if (delegate == nil || ![delegate respondsToSelector:@selector(doDisplayOrderChanged:)])
 	{
 		return;
 	}
 	
 	const NSInteger theOrder = *(NSInteger *)[displayOrderIdData bytes];
-	[videoDelegate doDisplayOrderChanged:theOrder];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doDisplayOrderChanged:theOrder];
 }
 
 - (void) handleChangeDisplayGap:(NSData *)displayGapScalarData
 {
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doDisplayGapChanged:)])
+	if (delegate == nil || ![delegate respondsToSelector:@selector(doDisplayGapChanged:)])
 	{
 		return;
 	}
 	
 	const float gapScalar = *(float *)[displayGapScalarData bytes];
-	[videoDelegate doDisplayGapChanged:gapScalar];
-}
-
-- (void) handleChangeBilinearOutput:(NSData *)bilinearStateData
-{
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doBilinearOutputChanged:)])
-	{
-		return;
-	}
-	
-	const BOOL theState = *(BOOL *)[bilinearStateData bytes];
-	[videoDelegate doBilinearOutputChanged:theState];
-	[self handleEmuFrameProcessed:self.frameData attributes:self.frameAttributesData];
-}
-
-- (void) handleChangeVerticalSync:(NSData *)verticalSyncStateData
-{
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doVerticalSyncChanged:)])
-	{
-		return;
-	}
-	
-	const BOOL theState = *(BOOL *)[verticalSyncStateData bytes];
-	[videoDelegate doVerticalSyncChanged:theState];
-}
-
-- (void) handleChangeVideoFilter:(NSData *)videoFilterTypeIdData
-{
-	if (videoDelegate == nil || ![videoDelegate respondsToSelector:@selector(doVideoFilterChanged:frameSize:)])
-	{
-		return;
-	}
-	
-	const NSInteger theType = *(NSInteger *)[videoFilterTypeIdData bytes];
-	[self setVfType:theType];
-	[videoDelegate doVideoFilterChanged:theType frameSize:[vf destSize]];
-	[self handleEmuFrameProcessed:self.frameData attributes:self.frameAttributesData];
+	[(id<CocoaDSDisplayVideoDelegate>)delegate doDisplayGapChanged:gapScalar];
 }
 
 @end
