@@ -223,6 +223,7 @@ static NSDate *distantFutureDate = [[NSDate distantFuture] retain];
 
 @synthesize thread;
 @synthesize threadExit;
+@dynamic idle;
 @synthesize autoreleaseInterval;
 @synthesize sendPort;
 @synthesize receivePort;
@@ -237,8 +238,11 @@ static NSDate *distantFutureDate = [[NSDate distantFuture] retain];
 	// Set up thread info.
 	thread = nil;
 	threadExit = NO;
+	_idleState = NO;
 	autoreleaseInterval = interval;
 	
+	spinlockIdle = OS_SPINLOCK_INIT;
+	
 	// Set up thread ports.
 	sendPort = [[NSPort port] retain];
 	[sendPort setDelegate:self];
@@ -255,6 +259,22 @@ static NSDate *distantFutureDate = [[NSDate distantFuture] retain];
 	[super dealloc];
 }
 
+- (void) setIdle:(BOOL)theState
+{
+	OSSpinLockLock(&spinlockIdle);
+	_idleState = theState;
+	OSSpinLockUnlock(&spinlockIdle);
+}
+
+- (BOOL) idle
+{
+	OSSpinLockLock(&spinlockIdle);
+	const BOOL theState = _idleState;
+	OSSpinLockUnlock(&spinlockIdle);
+	
+	return theState;
+}
+
 - (void) runThread:(id)object
 {
 	NSAutoreleasePool *threadPool = [[NSAutoreleasePool alloc] init];
@@ -294,6 +314,7 @@ static NSDate *distantFutureDate = [[NSDate distantFuture] retain];
 	}
 	
 	[self setThreadExit:YES];
+	[self setIdle:NO];
 	
 	// Wait until the thread has shut down.
 	while ([self thread] != nil)
