@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2012-2013 DeSmuME team
+	Copyright (C) 2012-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,9 +18,11 @@
 #import <Cocoa/Cocoa.h>
 #import <OpenEmuBase/OEGameCore.h>
 #import "OENDSSystemResponderClient.h"
+#import "../cocoa_input.h"
 #include <libkern/OSAtomic.h>
 #include <pthread.h>
 
+@class CocoaDSCheatManager;
 @class CocoaDSController;
 @class CocoaDSGPU;
 @class CocoaDSFirmware;
@@ -29,6 +31,8 @@
 @interface NDSGameCore : OEGameCore
 {
 	NSPoint touchLocation;
+	NSMutableDictionary *addedCheatsDict;
+	CocoaDSCheatManager *cdsCheats;
 	CocoaDSController *cdsController;
 	CocoaDSGPU *cdsGPU;
 	CocoaDSFirmware *cdsFirmware;
@@ -38,9 +42,10 @@
 	NSInteger inputID[OENDSButtonCount]; // Key = OpenEmu's input ID, Value = DeSmuME's input ID
 	
 	OSSpinLock spinlockDisplayMode;
-	pthread_mutex_t mutexCoreExecute;
+	pthread_rwlock_t rwlockCoreExecute;
 }
 
+@property (retain) CocoaDSCheatManager *cdsCheats;
 @property (retain) CocoaDSController *cdsController;
 @property (retain) CocoaDSGPU *cdsGPU;
 @property (retain) CocoaDSFirmware *cdsFirmware;
