@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2007 Jeff Bland
-	Copyright (C) 2007-2013 DeSmuME team
+	Copyright (C) 2007-2014 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
 // Global sound playback manager
 static CoreAudioOutput *coreAudioPlaybackManager = NULL;
 static pthread_mutex_t *mutexAudioSampleReadWrite = NULL;
-pthread_mutex_t *mutexAudioEmulateCore = NULL;
+pthread_rwlock_t *rwlockAudioEmulateCore = NULL;
 
 // Sound interface to the SPU
 SoundInterface_struct SNDOSX = {
@@ -52,7 +52,7 @@ SoundInterface_struct *SNDCoreList[] = {
 int SNDOSXInit(int buffer_size)
 {
 	CoreAudioOutput *oldcoreAudioPlaybackManager = coreAudioPlaybackManager;
-	coreAudioPlaybackManager = new CoreAudioOutput(buffer_size / SPU_SAMPLE_SIZE, SPU_SAMPLE_SIZE);
+	coreAudioPlaybackManager = new CoreAudioOutput(buffer_size * 4 / SPU_SAMPLE_SIZE, SPU_SAMPLE_SIZE);
 	delete oldcoreAudioPlaybackManager;
 	
 	if (mutexAudioSampleReadWrite == NULL)
@@ -62,6 +62,7 @@ int SNDOSXInit(int buffer_size)
 	}
 	
 	coreAudioPlaybackManager->start();
+	coreAudioPlaybackManager->pause();
 	
 	return 0;
 }
@@ -126,6 +127,26 @@ void SNDOSXUnMuteAudio()
 	coreAudioPlaybackManager->unmute();
 }
 
+void SNDOSXPauseAudio()
+{
+	if (coreAudioPlaybackManager == NULL)
+	{
+		return;
+	}
+	
+	coreAudioPlaybackManager->pause();
+}
+
+void SNDOSXUnpauseAudio()
+{
+	if (coreAudioPlaybackManager == NULL)
+	{
+		return;
+	}
+	
+	coreAudioPlaybackManager->unpause();
+}
+
 void SNDOSXSetVolume(int volume)
 {
 	if (coreAudioPlaybackManager == NULL)
@@ -176,11 +197,11 @@ size_t SNDOSXPostProcessSamples(s16 *postProcessBuffer, size_t requestedSampleCo
 	switch (synchMode)
 	{
 		case ESynchMode_DualSynchAsynch:
-			if (mutexAudioEmulateCore != NULL)
+			if (rwlockAudioEmulateCore != NULL)
 			{
-				pthread_mutex_lock(mutexAudioEmulateCore);
+				pthread_rwlock_wrlock(rwlockAudioEmulateCore);
 				processedSampleCount = SPU_DefaultPostProcessSamples(postProcessBuffer, requestedSampleCount, synchMode, theSynchronizer);
-				pthread_mutex_unlock(mutexAudioEmulateCore);
+				pthread_rwlock_unlock(rwlockAudioEmulateCore);
 			}
 			break;
 			
