@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  */
 
 #import <Cocoa/Cocoa.h>
-#include <OpenGL/OpenGL.h>
+#import <OpenGL/OpenGL.h>
 #include <libkern/OSAtomic.h>
 
 #import "InputManager.h"
@@ -24,6 +24,7 @@
 
 @class CocoaDSController;
 @class EmuControllerDelegate;
+class OGLVideoOutput;
 
 
 // Subclass NSWindow for full screen windows so that we can override some methods.
@@ -34,65 +35,36 @@
 @interface DisplayView : NSView <CocoaDSDisplayVideoDelegate, InputHIDManagerTarget>
 {
 	InputManager *inputManager;
+	OGLVideoOutput *oglv;
+	CGFloat _displayRotation;
+	BOOL canUseShaderBasedFilters;
 	
-	// Display thread
-	BOOL isHudEnabled;
-	BOOL isHudEditingModeEnabled;
+	BOOL _useVerticalSync;
+	
+	OSSpinLock spinlockUseVerticalSync;
+	OSSpinLock spinlockVideoFiltersPreferGPU;
+	OSSpinLock spinlockOutputFilter;
+	OSSpinLock spinlockSourceDeposterize;
+	OSSpinLock spinlockPixelScaler;
 	
 	// OpenGL context
 	NSOpenGLContext *context;
 	CGLContextObj cglDisplayContext;
-	
-	NSSize _currentNormalSize;
-	NSInteger _currentDisplayMode;
-	NSInteger _currentDisplayOrientation;
-	GLfloat _currentGapScalar;
-	GLfloat _currentRotation;
-	GLenum glTexPixelFormat;
-	GLvoid *glTexBack;
-	NSSize glTexBackSize;
-	
-	GLuint displayTexID;
-	GLuint vboVertexID;
-	GLuint vboTexCoordID;
-	GLuint vboElementID;
-	GLuint vaoMainStatesID;
-	GLuint vertexShaderID;
-	GLuint fragmentShaderID;
-	GLuint shaderProgram;
-	
-	GLint uniformAngleDegrees;
-	GLint uniformScalar;
-	GLint uniformViewSize;
-	
-	GLint vtxBuffer[4 * 8];
-	GLfloat texCoordBuffer[2 * 8];
-	GLubyte vtxIndexBuffer[12];
-	
-	BOOL isShaderSupported;
-	size_t vtxBufferOffset;
 }
 
 @property (retain) InputManager *inputManager;
+@property (readonly) BOOL canUseShaderBasedFilters;
+@property (assign) BOOL useVerticalSync;
+@property (assign) BOOL videoFiltersPreferGPU;
+@property (assign) BOOL sourceDeposterize;
+@property (assign) NSInteger outputFilter;
+@property (assign) NSInteger pixelScaler;
 
-- (void) startupOpenGL;
-- (void) shutdownOpenGL;
-- (void) setupShaderIO;
-- (BOOL) setupShadersWithVertexProgram:(const char *)vertShaderProgram fragmentProgram:(const char *)fragShaderProgram;
 - (void) drawVideoFrame;
-- (void) uploadVertices;
-- (void) uploadTexCoords;
-- (void) uploadDisplayTextures:(const GLvoid *)textureData displayMode:(const NSInteger)displayModeID width:(const GLsizei)texWidth height:(const GLsizei)texHeight;
-- (void) renderDisplayUsingDisplayMode:(const NSInteger)displayModeID;
-- (void) updateDisplayVerticesUsingDisplayMode:(const NSInteger)displayModeID orientation:(const NSInteger)displayOrientationID gap:(const GLfloat)gapScalar;
-- (void) updateTexCoordS:(GLfloat)s T:(GLfloat)t;
-
 - (NSPoint) dsPointFromEvent:(NSEvent *)theEvent;
 - (NSPoint) convertPointToDS:(NSPoint)clickLoc;
 - (BOOL) handleKeyPress:(NSEvent *)theEvent keyPressed:(BOOL)keyPressed;
 - (BOOL) handleMouseButton:(NSEvent *)theEvent buttonPressed:(BOOL)buttonPressed;
-- (void) clearToBlack;
-- (void) clearToWhite;
 - (void) requestScreenshot:(NSURL *)fileURL fileType:(NSBitmapImageFileType)fileType;
 
 @end
@@ -103,8 +75,16 @@
 @interface DisplayWindowController : NSWindowController
 #endif
 {
+	NSObject *dummyObject;
+	
 	DisplayView *view;
 	NSView *saveScreenshotPanelAccessoryView;
+	NSView *outputVolumeControlView;
+	NSView *microphoneGainControlView;
+	NSMenuItem *outputVolumeMenuItem;
+	NSMenuItem *microphoneGainMenuItem;
+	NSSlider *microphoneGainSlider;
+	NSButton *microphoneMuteButton;
 	
 	EmuControllerDelegate *emuControl;
 	CocoaDSDisplayVideo *cdsVideoOutput;
@@ -114,13 +94,10 @@
 	NSSize _normalSize;
 	double _displayScale;
 	double _displayRotation;
-	BOOL _useBilinearOutput;
-	BOOL _useVerticalSync;
 	NSInteger _displayMode;
 	NSInteger _displayOrientation;
 	NSInteger _displayOrder;
 	double _displayGap;
-	NSInteger _videoFilterType;
 	NSInteger screenshotFileFormat;
 	
 	NSSize _minDisplayViewSize;
@@ -131,17 +108,22 @@
 	OSSpinLock spinlockNormalSize;
 	OSSpinLock spinlockScale;
 	OSSpinLock spinlockRotation;
-	OSSpinLock spinlockUseBilinearOutput;
-	OSSpinLock spinlockUseVerticalSync;
 	OSSpinLock spinlockDisplayMode;
 	OSSpinLock spinlockDisplayOrientation;
 	OSSpinLock spinlockDisplayOrder;
 	OSSpinLock spinlockDisplayGap;
-	OSSpinLock spinlockVideoFilterType;
 }
 
+@property (readonly) IBOutlet NSObject *dummyObject;
+
 @property (readonly) IBOutlet DisplayView *view;
 @property (readonly) IBOutlet NSView *saveScreenshotPanelAccessoryView;
+@property (readonly) IBOutlet NSView *outputVolumeControlView;
+@property (readonly) IBOutlet NSView *microphoneGainControlView;
+@property (readonly) IBOutlet NSMenuItem *outputVolumeMenuItem;
+@property (readonly) IBOutlet NSMenuItem *microphoneGainMenuItem;
+@property (readonly) IBOutlet NSSlider *microphoneGainSlider;
+@property (readonly) IBOutlet NSButton *microphoneMuteButton;
 
 @property (retain) EmuControllerDelegate *emuControl;
 @property (assign) CocoaDSDisplayVideo *cdsVideoOutput;
@@ -151,13 +133,14 @@
 @property (readonly) NSSize normalSize;
 @property (assign) double displayScale;
 @property (assign) double displayRotation;
-@property (assign) BOOL useBilinearOutput;
-@property (assign) BOOL useVerticalSync;
+@property (assign) BOOL videoFiltersPreferGPU;
+@property (assign) BOOL videoSourceDeposterize;
+@property (assign) NSInteger videoOutputFilter;
+@property (assign) NSInteger videoPixelScaler;
 @property (assign) NSInteger displayMode;
 @property (assign) NSInteger displayOrientation;
 @property (assign) NSInteger displayOrder;
 @property (assign) double displayGap;
-@property (assign) NSInteger videoFilterType;
 @property (assign) NSInteger screenshotFileFormat;
 @property (assign) BOOL isMinSizeNormal;
 @property (assign) BOOL isShowingStatusBar;
@@ -171,6 +154,7 @@
 - (void) exitFullScreen;
 
 - (IBAction) copy:(id)sender;
+- (IBAction) changeHardwareMicGain:(id)sender;
 - (IBAction) changeVolume:(id)sender;
 - (IBAction) toggleKeepMinDisplaySizeAtNormal:(id)sender;
 - (IBAction) toggleStatusBar:(id)sender;
@@ -190,9 +174,9 @@
 - (IBAction) changeDisplayOrientation:(id)sender;
 - (IBAction) changeDisplayOrder:(id)sender;
 - (IBAction) changeDisplayGap:(id)sender;
-- (IBAction) toggleBilinearFilteredOutput:(id)sender;
-- (IBAction) toggleVerticalSync:(id)sender;
-- (IBAction) changeVideoFilter:(id)sender;
+- (IBAction) toggleVideoSourceDeposterize:(id)sender;
+- (IBAction) changeVideoOutputFilter:(id)sender;
+- (IBAction) changeVideoPixelScaler:(id)sender;
 
 - (IBAction) writeDefaultsDisplayRotation:(id)sender;
 - (IBAction) writeDefaultsDisplayGap:(id)sender;
