@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,10 +17,12 @@
 
 #import <Cocoa/Cocoa.h>
 #include <libkern/OSAtomic.h>
+#import "../cocoa_input.h"
 
 @class InputManager;
 @class CocoaDSRom;
 @class CocoaDSFirmware;
+@class CocoaDSController;
 @class CocoaDSOutput;
 @class CocoaDSSpeaker;
 @class CocoaDSCheatManager;
@@ -28,7 +30,7 @@
 @class DisplayWindowController;
 class AudioSampleBlockGenerator;
 
-@interface EmuControllerDelegate : NSObject <NSUserInterfaceValidations>
+@interface EmuControllerDelegate : NSObject <NSUserInterfaceValidations, CocoaDSControllerDelegate>
 {
 	InputManager *inputManager;
 	
@@ -44,9 +46,18 @@ class AudioSampleBlockGenerator;
 	NSObjectController *cdsCoreController;
 	NSObjectController *cdsSoundController;
 	NSObjectController *cheatWindowController;
+	NSObjectController *slot2WindowController;
 	NSArrayController *cheatListController;
 	NSArrayController *cheatDatabaseController;
 	
+	NSWindow *displayRotationPanel;
+	NSWindow *displaySeparationPanel;
+	NSWindow *displayVideoSettingsPanel;
+	NSString *displayRotationPanelTitle;
+	NSString *displaySeparationPanelTitle;
+	NSString *displayVideoSettingsPanelTitle;
+	
+	NSWindow *executionControlWindow;
 	NSWindow *slot1ManagerWindow;
 	NSWindow *saveFileMigrationSheet;
 	NSWindow *saveStatePrecloseSheet;
@@ -58,6 +69,7 @@ class AudioSampleBlockGenerator;
 	BOOL isRomLoading;
 	NSString *statusText;
 	float currentVolumeValue;
+	NSImage *currentMicStatusIcon;
 	NSImage *currentVolumeIcon;
 	BOOL isShowingSaveStateDialog;
 	BOOL isShowingFileMigrationDialog;
@@ -65,11 +77,24 @@ class AudioSampleBlockGenerator;
 	NSURL *currentSaveStateURL;
 	NSInteger selectedExportRomSaveID;
 	NSInteger selectedRomSaveTypeID;
+	NSInteger frameJumpType;
+	NSInteger frameJumpFramesForward;
+	NSInteger frameJumpToFrame;
 	
 	CGFloat lastSetSpeedScalar;
+	BOOL isSoftwareMicActive;
+	BOOL isHardwareMicAvailable;
+	BOOL isHardwareMicIdle;
+	BOOL isHardwareMicInClip;
+	float currentMicGainValue;
 	BOOL isSoundMuted;
 	float lastSetVolumeValue;
 	
+	NSImage *iconMicDisabled;
+	NSImage *iconMicIdle;
+	NSImage *iconMicActive;
+	NSImage *iconMicInClip;
+	NSImage *iconMicManualOverride;
 	NSImage *iconVolumeFull;
 	NSImage *iconVolumeTwoThird;
 	NSImage *iconVolumeOneThird;
@@ -99,9 +124,15 @@ class AudioSampleBlockGenerator;
 @property (readonly) IBOutlet NSObjectController *cdsCoreController;
 @property (readonly) IBOutlet NSObjectController *cdsSoundController;
 @property (readonly) IBOutlet NSObjectController *cheatWindowController;
+@property (readonly) IBOutlet NSObjectController *slot2WindowController;
 @property (readonly) IBOutlet NSArrayController *cheatListController;
 @property (readonly) IBOutlet NSArrayController *cheatDatabaseController;
 
+@property (readonly) IBOutlet NSWindow *displayRotationPanel;
+@property (readonly) IBOutlet NSWindow *displaySeparationPanel;
+@property (readonly) IBOutlet NSWindow *displayVideoSettingsPanel;
+
+@property (readonly) IBOutlet NSWindow *executionControlWindow;
 @property (readonly) IBOutlet NSWindow *slot1ManagerWindow;
 @property (readonly) IBOutlet NSWindow *saveFileMigrationSheet;
 @property (readonly) IBOutlet NSWindow *saveStatePrecloseSheet;
@@ -120,7 +151,13 @@ class AudioSampleBlockGenerator;
 @property (assign) BOOL isWorking;
 @property (assign) BOOL isRomLoading;
 @property (assign) NSString *statusText;
+@property (assign) BOOL isSoftwareMicActive;
+@property (assign) BOOL isHardwareMicAvailable;
+@property (assign) BOOL isHardwareMicIdle;
+@property (assign) BOOL isHardwareMicInClip;
+@property (assign) float currentMicGainValue;
 @property (assign) float currentVolumeValue;
+@property (retain) NSImage *currentMicStatusIcon;
 @property (retain) NSImage *currentVolumeIcon;
 @property (assign) BOOL isShowingSaveStateDialog;
 @property (assign) BOOL isShowingFileMigrationDialog;
@@ -128,16 +165,9 @@ class AudioSampleBlockGenerator;
 @property (retain) NSURL *currentSaveStateURL;
 @property (assign) NSInteger selectedExportRomSaveID;
 @property (assign) NSInteger selectedRomSaveTypeID;
-
-@property (assign) NSInteger render3DRenderingEngine;
-@property (assign) BOOL render3DHighPrecisionColorInterpolation;
-@property (assign) BOOL render3DEdgeMarking;
-@property (assign) BOOL render3DFog;
-@property (assign) BOOL render3DTextures;
-@property (assign) NSInteger render3DDepthComparisonThreshold;
-@property (assign) NSInteger render3DThreads;
-@property (assign) BOOL render3DLineHack;
-@property (assign) BOOL render3DMultisample;
+@property (assign) NSInteger frameJumpType;
+@property (assign) NSInteger frameJumpFramesForward;
+@property (assign) NSInteger frameJumpToFrame;
 
 @property (retain) DisplayWindowController *mainWindow;
 @property (readonly) NSMutableArray *windowList;
@@ -155,6 +185,9 @@ class AudioSampleBlockGenerator;
 - (IBAction) revertEmuSaveState:(id)sender;
 - (IBAction) loadEmuSaveStateSlot:(id)sender;
 - (IBAction) saveEmuSaveStateSlot:(id)sender;
+- (IBAction) openReplay:(id)sender;
+- (IBAction) recordReplay:(id)sender;
+- (IBAction) stopReplay:(id)sender;
 - (IBAction) importRomSave:(id)sender;
 - (IBAction) exportRomSave:(id)sender;
 
@@ -163,15 +196,26 @@ class AudioSampleBlockGenerator;
 - (IBAction) toggleAutoFrameSkip:(id)sender;
 - (IBAction) toggleCheats:(id)sender;
 - (IBAction) toggleExecutePause:(id)sender;
+- (IBAction) coreExecute:(id)sender;
+- (IBAction) corePause:(id)sender;
+- (IBAction) frameAdvance:(id)sender;
+- (IBAction) frameJump:(id)sender;
 - (IBAction) reset:(id)sender;
 - (IBAction) changeRomSaveType:(id)sender;
 
+// View Menu
+- (IBAction) toggleAllDisplays:(id)sender;
+
 // Tools Menu
+- (IBAction) autoholdSet:(id)sender;
 - (IBAction) toggleGPUState:(id)sender;
+- (IBAction) toggleGDBStubActivate:(id)sender;
 
 - (IBAction) changeCoreSpeed:(id)sender;
 - (IBAction) changeCoreEmuFlags:(id)sender;
 - (IBAction) changeFirmwareSettings:(id)sender;
+- (IBAction) changeHardwareMicGain:(id)sender;
+- (IBAction) changeHardwareMicMute:(id)sender;
 - (IBAction) changeVolume:(id)sender;
 - (IBAction) changeAudioEngine:(id)sender;
 - (IBAction) changeSpuAdvancedLogic:(id)sender;
@@ -191,18 +235,27 @@ class AudioSampleBlockGenerator;
 - (IBAction) closeSheet:(id)sender;
 
 - (void) cmdUpdateDSController:(NSValue *)cmdAttrValue;
+- (void) cmdUpdateDSControllerWithTurbo:(NSValue *)cmdAttrValue;
+- (void) cmdUpdateDSTouch:(NSValue *)cmdAttrValue;
+- (void) cmdUpdateDSMicrophone:(NSValue *)cmdAttrValue;
+- (void) cmdUpdateDSPaddle:(NSValue *)cmdAttrValue;
 
 - (void) cmdLoadEmuSaveStateSlot:(NSValue *)cmdAttrValue;
 - (void) cmdSaveEmuSaveStateSlot:(NSValue *)cmdAttrValue;
 
 - (void) cmdCopyScreen:(NSValue *)cmdAttrValue;
 - (void) cmdRotateDisplayRelative:(NSValue *)cmdAttrValue;
+- (void) cmdToggleAllDisplays:(NSValue *)cmdAttrValue;
 
 - (void) cmdHoldToggleSpeedScalar:(NSValue *)cmdAttrValue;
 - (void) cmdToggleSpeedLimiter:(NSValue *)cmdAttrValue;
 - (void) cmdToggleAutoFrameSkip:(NSValue *)cmdAttrValue;
 - (void) cmdToggleCheats:(NSValue *)cmdAttrValue;
 - (void) cmdToggleExecutePause:(NSValue *)cmdAttrValue;
+- (void) cmdCoreExecute:(NSValue *)cmdAttrValue;
+- (void) cmdCorePause:(NSValue *)cmdAttrValue;
+- (void) cmdFrameAdvance:(NSValue *)cmdAttrValue;
+- (void) cmdFrameJump:(NSValue *)cmdAttrValue;
 - (void) cmdReset:(NSValue *)cmdAttrValue;
 - (void) cmdToggleGPUState:(NSValue *)cmdAttrValue;
 
@@ -218,6 +271,7 @@ class AudioSampleBlockGenerator;
 - (void) executeCore;
 - (void) pauseCore;
 - (void) restoreCoreState;
+- (void) updateMicStatusIcon;
 
 - (AudioSampleBlockGenerator *) selectedAudioFileGenerator;
 - (void) setSelectedAudioFileGenerator:(AudioSampleBlockGenerator *)theGenerator;
@@ -229,6 +283,7 @@ class AudioSampleBlockGenerator;
 - (void) didEndChooseSlot1R4Directory:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
 
 - (void) updateAllWindowTitles;
+- (void) updateDisplayPanelTitles;
 - (void) setupUserDefaults;
 
 @end
