@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -18,17 +18,19 @@
 #import <Cocoa/Cocoa.h>
 #include <libkern/OSAtomic.h>
 #include <IOKit/hid/IOHIDManager.h>
+#include <ForceFeedback/ForceFeedback.h>
 
 #if defined(__ppc__) || defined(__ppc64__)
 #include <map>
-#else
+#elif !defined(MAC_OS_X_VERSION_10_7) || (MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_6)
 #include <tr1/unordered_map>
+#else
+#include <unordered_map>
 #endif
 #include <string>
 #include <vector>
 
-#include "mic_ext.h"
-#undef BOOL
+#include "../audiosamplegenerator.h"
 
 #define INPUT_HANDLER_STRING_LENGTH 256
 
@@ -46,7 +48,7 @@ enum InputAttributeState
 @protocol InputHIDManagerTarget <NSObject>
 
 @required
-- (BOOL) handleHIDQueue:(IOHIDQueueRef)hidQueue;
+- (BOOL) handleHIDQueue:(IOHIDQueueRef)hidQueue hidManager:(InputHIDManager *)hidManager;
 
 @end
 
@@ -56,6 +58,7 @@ typedef struct
 	char deviceCode[INPUT_HANDLER_STRING_LENGTH];
 	char elementName[INPUT_HANDLER_STRING_LENGTH];
 	char elementCode[INPUT_HANDLER_STRING_LENGTH];
+	bool isAnalog;					// This is an analog input, as opposed to being a digital input.
 	
 	InputAttributeState state;		// The input state that is sent on command dispatch
 	int32_t intCoordX;				// The X-coordinate as an int for commands that require a location
@@ -80,6 +83,7 @@ typedef struct
 	bool useInputForSender;			// The command will prefer the input device's sender
 	
 	InputAttributes input;			// The input device's attributes
+	bool allowAnalogInput;			// Flag for allowing a command to accept analog inputs
 } CommandAttributes;
 
 typedef std::vector<InputAttributes> InputAttributesList;
@@ -90,11 +94,16 @@ typedef std::map<std::string, CommandAttributes> InputCommandMap; // Key = Input
 typedef std::map<std::string, CommandAttributes> CommandAttributesMap; // Key = Command Tag, Value = CommandAttributes
 typedef std::map<std::string, SEL> CommandSelectorMap; // Key = Command Tag, Value = Obj-C Selector
 typedef std::map<std::string, AudioSampleBlockGenerator> AudioFileSampleGeneratorMap; // Key = File path to audio file, Value = AudioSampleBlockGenerator
-#else
+#elif !defined(MAC_OS_X_VERSION_10_7) || (MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_6)
 typedef std::tr1::unordered_map<std::string, CommandAttributes> InputCommandMap; // Key = Input key in deviceCode:elementCode format, Value = CommandAttributes
 typedef std::tr1::unordered_map<std::string, CommandAttributes> CommandAttributesMap; // Key = Command Tag, Value = CommandAttributes
 typedef std::tr1::unordered_map<std::string, SEL> CommandSelectorMap; // Key = Command Tag, Value = Obj-C Selector
 typedef std::tr1::unordered_map<std::string, AudioSampleBlockGenerator> AudioFileSampleGeneratorMap; // Key = File path to audio file, Value = AudioSampleBlockGenerator
+#else
+typedef std::unordered_map<std::string, CommandAttributes> InputCommandMap; // Key = Input key in deviceCode:elementCode format, Value = CommandAttributes
+typedef std::unordered_map<std::string, CommandAttributes> CommandAttributesMap; // Key = Command Tag, Value = CommandAttributes
+typedef std::unordered_map<std::string, SEL> CommandSelectorMap; // Key = Command Tag, Value = Obj-C Selector
+typedef std::unordered_map<std::string, AudioSampleBlockGenerator> AudioFileSampleGeneratorMap; // Key = File path to audio file, Value = AudioSampleBlockGenerator
 #endif
 
 #pragma mark -
@@ -103,29 +112,49 @@ typedef std::tr1::unordered_map<std::string, AudioSampleBlockGenerator> AudioFil
 	InputHIDManager *hidManager;
 	IOHIDDeviceRef hidDeviceRef;
 	IOHIDQueueRef hidQueueRef;
+	
+	NSString *identifier;
+	
+	io_service_t ioService;
+	FFDeviceObjectReference ffDevice;
+	FFEffectObjectReference ffEffect;
+	BOOL supportsForceFeedback;
+	BOOL isForceFeedbackEnabled;
+	
 	NSRunLoop *runLoop;
 	OSSpinLock spinlockRunLoop;
 }
 
 @property (retain) InputHIDManager *hidManager;
 @property (readonly) IOHIDDeviceRef hidDeviceRef;
+@property (readonly) NSString *manufacturerName;
+@property (readonly) NSString *productName;
+@property (readonly) NSString *serialNumber;
+@property (readonly) NSString *identifier;
+@property (readonly) BOOL supportsForceFeedback;
+@property (assign) BOOL isForceFeedbackEnabled;
 @property (retain) NSRunLoop *runLoop;
 
 - (id) initWithDevice:(IOHIDDeviceRef)theDevice hidManager:(InputHIDManager *)theHIDManager;
 
+- (void) setPropertiesUsingDictionary:(NSDictionary *)theProperties;
+- (NSDictionary *) propertiesDictionary;
+- (void) writeDefaults;
+
 - (void) start;
 - (void) stop;
 
-- (NSString *) manufacturerName;
-- (NSString *) productName;
-- (NSString *) serialNumber;
+- (void) startForceFeedbackAndIterate:(UInt32)iterations flags:(UInt32)ffFlags;
+- (void) stopForceFeedback;
 
 @end
 
-BOOL GetOnStateFromHIDValueRef(IOHIDValueRef hidValueRef);
-InputAttributes InputAttributesOfHIDValue(IOHIDValueRef hidValueRef, const char *altElementCode, const char *altElementName, bool *altOnState);
-InputAttributesList InputListFromHIDValue(IOHIDValueRef hidValueRef);
-InputAttributesList InputListFromHatSwitchValue(IOHIDValueRef hidValueRef, bool useEightDirection);
+bool InputElementCodeFromHIDElement(const IOHIDElementRef hidElementRef, char *charBuffer);
+bool InputElementNameFromHIDElement(const IOHIDElementRef hidElementRef, char *charBuffer);
+bool InputDeviceCodeFromHIDDevice(const IOHIDDeviceRef hidDeviceRef, char *charBuffer);
+bool InputDeviceNameFromHIDDevice(const IOHIDDeviceRef hidDeviceRef, char *charBuffer);
+InputAttributes InputAttributesOfHIDValue(IOHIDValueRef hidValueRef);
+InputAttributesList InputListFromHIDValue(IOHIDValueRef hidValueRef, InputManager *inputManager, bool forceDigitalInput);
 
 size_t ClearHIDQueue(const IOHIDQueueRef hidQueue);
 void HandleQueueValueAvailableCallback(void *inContext, IOReturn inResult, void *inSender);
@@ -136,15 +165,15 @@ void HandleQueueValueAvailableCallback(void *inContext, IOReturn inResult, void
 	InputManager *inputManager;
 	IOHIDManagerRef hidManagerRef;
 	NSRunLoop *runLoop;
-	NSMutableSet *deviceList;
+	NSArrayController *deviceListController;
 	id<InputHIDManagerTarget> target;
 	
 	OSSpinLock spinlockRunLoop;
 }
 
+@property (retain) NSArrayController *deviceListController;
 @property (retain) InputManager *inputManager;
 @property (readonly) IOHIDManagerRef hidManagerRef;
-@property (readonly) NSMutableSet *deviceList;
 @property (assign) id target;
 @property (retain) NSRunLoop *runLoop;
 
@@ -173,6 +202,7 @@ void HandleDeviceRemovalCallback(void *inContext, IOReturn inResult, void *inSen
 
 @property (readonly) IBOutlet EmuControllerDelegate *emuControl;
 @property (retain) id<InputHIDManagerTarget> hidInputTarget;
+@property (readonly) InputHIDManager *hidManager;
 @property (readonly) NSMutableDictionary *inputMappings;
 @property (readonly) NSArray *commandTagList;
 @property (readonly) NSDictionary *commandIcon;
@@ -207,7 +237,7 @@ void HandleDeviceRemovalCallback(void *inContext, IOReturn inResult, void *inSen
 
 CommandAttributes NewDefaultCommandAttributes(const char *commandTag);
 CommandAttributes NewCommandAttributesForSelector(const char *commandTag, const SEL theSelector);
-CommandAttributes NewCommandAttributesForDSControl(const char *commandTag, const NSUInteger controlID);
+CommandAttributes NewCommandAttributesForDSControl(const char *commandTag, const NSUInteger controlID, const bool supportTurbo);
 void UpdateCommandAttributesWithDeviceInfoDictionary(CommandAttributes *cmdAttr, NSDictionary *deviceInfo);
 
 NSMutableDictionary* DeviceInfoDictionaryWithCommandAttributes(const CommandAttributes *cmdAttr,
@@ -216,7 +246,7 @@ NSMutableDictionary* DeviceInfoDictionaryWithCommandAttributes(const CommandAttr
 															   NSString *elementCode,
 															   NSString *elementName);
 
-InputAttributesList InputManagerEncodeHIDQueue(const IOHIDQueueRef hidQueue);
+InputAttributesList InputManagerEncodeHIDQueue(const IOHIDQueueRef hidQueue, InputManager *inputManager, bool forceDigitalInput);
 InputAttributes InputManagerEncodeKeyboardInput(const unsigned short keyCode, BOOL keyPressed);
 InputAttributes InputManagerEncodeMouseButtonInput(const NSInteger buttonNumber, const NSPoint touchLoc, BOOL buttonPressed);
 InputAttributes InputManagerEncodeIBAction(const SEL theSelector, id sender);
