@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2011-2013 DeSmuME team
+	Copyright (C) 2011-2015 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 #import "DisplayWindowController.h"
 #import "EmuControllerDelegate.h"
 #import "FileMigrationDelegate.h"
+#import "RomInfoPanel.h"
+#import "Slot2WindowDelegate.h"
 #import "preferencesWindowDelegate.h"
 #import "troubleshootingWindowDelegate.h"
 #import "cheatWindowDelegate.h"
@@ -40,6 +42,7 @@
 @synthesize prefWindow;
 @synthesize troubleshootingWindow;
 @synthesize cheatListWindow;
+@synthesize slot2Window;
 @synthesize prefGeneralView;
 @synthesize mLoadStateSlot;
 @synthesize mSaveStateSlot;
@@ -50,17 +53,14 @@
 @synthesize romInfoPanelController;
 @synthesize prefWindowController;
 @synthesize cdsCoreController;
+@synthesize inputDeviceListController;
 @synthesize cheatWindowController;
 @synthesize migrationDelegate;
 @synthesize inputManager;
-
-@synthesize boxGeneralInfo;
-@synthesize boxTitles;
-@synthesize boxARMBinaries;
-@synthesize boxFileSystem;
-@synthesize boxMisc;
+@synthesize romInfoPanel;
 
 @synthesize isAppRunningOnIntel;
+@synthesize isDeveloperPlusBuild;
 
 
 - (id)init
@@ -77,6 +77,12 @@
 #else
 	isAppRunningOnIntel = NO;
 #endif
+    
+#if defined(GDB_STUB)
+    isDeveloperPlusBuild = YES;
+#else
+    isDeveloperPlusBuild = NO;
+#endif
 	
 	return self;
 }
@@ -112,9 +118,21 @@
 
 - (void)applicationWillFinishLaunching:(NSNotification *)notification
 {
+	// Register the application's defaults.
+	NSDictionary *prefsDict = [NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"DefaultUserPrefs" ofType:@"plist"]];
+	if (prefsDict == nil)
+	{
+		[[NSAlert alertWithMessageText:NSSTRING_ALERT_CRITICAL_FILE_MISSING_PRI defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:NSSTRING_ALERT_CRITICAL_FILE_MISSING_SEC] runModal];
+		[NSApp terminate:nil];
+		return;
+	}
+	
+	[[NSUserDefaults standardUserDefaults] registerDefaults:prefsDict];
+	
 	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	PreferencesWindowDelegate *prefWindowDelegate = (PreferencesWindowDelegate *)[prefWindow delegate];
 	CheatWindowDelegate *cheatWindowDelegate = (CheatWindowDelegate *)[cheatListWindow delegate];
+	Slot2WindowDelegate *slot2WindowDelegate = (Slot2WindowDelegate *)[slot2Window delegate];
 	
 	// Create the needed directories in Application Support if they haven't already
 	// been created.
@@ -150,23 +168,9 @@
 	
 	[aboutWindowController setContent:aboutWindowProperties];
 	
-	// Register the application's defaults.
-	NSDictionary *prefsDict = [NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"DefaultUserPrefs" ofType:@"plist"]];
-	if (prefsDict == nil)
-	{
-		[[NSAlert alertWithMessageText:NSSTRING_ALERT_CRITICAL_FILE_MISSING_PRI defaultButton:nil alternateButton:nil otherButton:nil informativeTextWithFormat:NSSTRING_ALERT_CRITICAL_FILE_MISSING_SEC] runModal];
-		[NSApp terminate:nil];
-		return;
-	}
-	
-	[[NSUserDefaults standardUserDefaults] registerDefaults:prefsDict];
-	
-	// Change the title colors of the NSBox objects in the ROM Info panel. We change the
-	// colors manually here because you can't change them in Interface Builder. Boo!!!
-	[self setRomInfoPanelBoxTitleColors];
-	
 	// Set the preferences window to the general view by default.
-	[prefWindowDelegate switchContentView:prefGeneralView];
+	[[prefWindowDelegate toolbar] setSelectedItemIdentifier:@"General"];
+	[prefWindowDelegate changePrefView:self];
 	
 	// Setup the slot menu items. We set this up manually instead of through Interface
 	// Builder because we're assuming an arbitrary number of slot items.
@@ -178,20 +182,18 @@
 	// Init the DS controller.
 	CocoaDSController *newController = [[[CocoaDSController alloc] init] autorelease];
 	[newCore setCdsController:newController];
+	[newController setDelegate:emuControl];
+	[newController setHardwareMicEnabled:YES];
 	
 	// Init the DS speakers.
 	CocoaDSSpeaker *newSpeaker = [[[CocoaDSSpeaker alloc] init] autorelease];
 	[newCore addOutput:newSpeaker];
 	[emuControl setCdsSpeaker:newSpeaker];
 	
-	// Start the core thread.
-	[NSThread detachNewThreadSelector:@selector(runThread:) toTarget:newCore withObject:nil];
-	
-	// Wait until the emulation core has finished starting up.
-	while (!([CocoaDSCore isCoreStarted] && [newCore thread] != nil))
-	{
-		[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.01]];
-	}
+	// Update the SLOT-2 device list after the emulation core is initialized.
+	[slot2WindowDelegate update];
+	[slot2WindowDelegate setHidManager:[inputManager hidManager]];
+	[slot2WindowDelegate setAutoSelectedDeviceText:[[slot2WindowDelegate deviceManager] autoSelectedDeviceName]];
 	
 	// Start up the threads for our outputs.
 	[NSThread detachNewThreadSelector:@selector(runThread:) toTarget:newSpeaker withObject:nil];
@@ -207,17 +209,11 @@
 	[romInfoPanelController setContent:[CocoaDSRom romNotLoadedBindings]];
 	[prefWindowController setContent:[prefWindowDelegate bindings]];
 	[cheatWindowController setContent:[cheatWindowDelegate bindings]];
-	
-	// Setup the applications settings from the user defaults file.
-	[self setupUserDefaults];
-	
-	[inputPrefsView initSettingsSheets];
-	[inputPrefsView populateInputProfileMenu];
-	[[inputPrefsView inputProfileMenu] selectItemAtIndex:0];
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotification
 {
+	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
 	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	
 	// Determine if the app was run for the first time.
@@ -238,6 +234,19 @@
 		appFirstTimeRunDict = [[NSMutableDictionary alloc] initWithObjectsAndKeys:isFirstTimeRunNumber, bundleVersionString, nil];
 	}
 	
+	// Setup the applications settings from the user defaults file.
+	[self setupUserDefaults];
+	
+	// Set up the input preferences view.
+	[inputPrefsView initSettingsSheets];
+	[inputPrefsView populateInputProfileMenu];
+	[[inputPrefsView inputPrefOutlineView] expandItem:nil expandChildren:YES];
+	[[inputPrefsView inputProfileMenu] selectItemAtIndex:0];
+	
+	// Make sure that the mic is paused to start with.
+	[[cdsCore cdsController] setHardwareMicPause:YES];
+	[emuControl updateMicStatusIcon];
+	
 	//Bring the application to the front
 	[NSApp activateIgnoringOtherApps:TRUE];
 	[self restoreDisplayWindowStates];
@@ -337,11 +346,21 @@
 	
 	// Save some settings to user defaults before app termination
 	[self saveDisplayWindowStates];
+	[romInfoPanel writeDefaults];
+	[[NSUserDefaults standardUserDefaults] setBool:[[cdsCore cdsController] hardwareMicMute] forKey:@"Microphone_HardwareMicMute"];
+	[[NSUserDefaults standardUserDefaults] setDouble:[emuControl currentVolumeValue] forKey:@"Sound_Volume"];
 	[[NSUserDefaults standardUserDefaults] setDouble:[emuControl lastSetSpeedScalar] forKey:@"CoreControl_SpeedScalar"];
 	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore isSpeedLimitEnabled] forKey:@"CoreControl_EnableSpeedLimit"];
 	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore isFrameSkipEnabled] forKey:@"CoreControl_EnableAutoFrameSkip"];
 	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore isCheatingEnabled] forKey:@"CoreControl_EnableCheats"];
 	
+#ifdef GDB_STUB
+	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore enableGdbStubARM9] forKey:@"Debug_GDBStubEnableARM9"];
+	[[NSUserDefaults standardUserDefaults] setBool:[cdsCore enableGdbStubARM7] forKey:@"Debug_GDBStubEnableARM7"];
+	[[NSUserDefaults standardUserDefaults] setInteger:[cdsCore gdbStubPortARM9] forKey:@"Debug_GDBStubPortARM9"];
+	[[NSUserDefaults standardUserDefaults] setInteger:[cdsCore gdbStubPortARM7] forKey:@"Debug_GDBStubPortARM7"];
+#endif
+	
 	[cdsCoreController setContent:nil];
 }
 
@@ -417,7 +436,7 @@
 {
 	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
 	PreferencesWindowDelegate *prefWindowDelegate = [prefWindow delegate];
-	NSMutableDictionary *prefBindings = [prefWindowDelegate bindings];
+	Slot2WindowDelegate *slot2WindowDelegate = (Slot2WindowDelegate *)[slot2Window delegate];
 	CocoaDSCore *cdsCore = (CocoaDSCore *)[cdsCoreController content];
 	
 	// Set the emulation flags.
@@ -509,77 +528,44 @@
 	if (arm7BiosImagePath != nil)
 	{
 		[cdsCore setArm7ImageURL:[NSURL fileURLWithPath:arm7BiosImagePath]];
-		[prefBindings setValue:[arm7BiosImagePath lastPathComponent] forKey:@"Arm7BiosImageName"];
 	}
 	else
 	{
 		[cdsCore setArm7ImageURL:nil];
-		[prefBindings setValue:nil forKey:@"Arm7BiosImageName"];
 	}
 	
 	NSString *arm9BiosImagePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"BIOS_ARM9ImagePath"];
 	if (arm9BiosImagePath != nil)
 	{
 		[cdsCore setArm9ImageURL:[NSURL fileURLWithPath:arm9BiosImagePath]];
-		[prefBindings setValue:[arm9BiosImagePath lastPathComponent] forKey:@"Arm9BiosImageName"];
 	}
 	else
 	{
 		[cdsCore setArm9ImageURL:nil];
-		[prefBindings setValue:nil forKey:@"Arm9BiosImageName"];
 	}
 	
 	NSString *firmwareImagePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"Emulation_FirmwareImagePath"];
 	if (firmwareImagePath != nil)
 	{
 		[cdsCore setFirmwareImageURL:[NSURL fileURLWithPath:firmwareImagePath]];
-		[prefBindings setValue:[firmwareImagePath lastPathComponent] forKey:@"FirmwareImageName"];
 	}
 	else
 	{
 		[cdsCore setFirmwareImageURL:nil];
-		[prefBindings setValue:nil forKey:@"FirmwareImageName"];
 	}
 	
-	NSString *advansceneDatabasePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"Advanscene_DatabasePath"];
-	if (advansceneDatabasePath != nil)
-	{
-		[prefBindings setValue:[advansceneDatabasePath lastPathComponent] forKey:@"AdvansceneDatabaseName"];
-	}
-	
-	NSString *cheatDatabasePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"R4Cheat_DatabasePath"];
-	if (cheatDatabasePath != nil)
-	{
-		[prefBindings setValue:[cheatDatabasePath lastPathComponent] forKey:@"R4CheatDatabaseName"];
-	}
-	
-	// Update the SPU Sync controls in the Preferences window.
-	if ([[NSUserDefaults standardUserDefaults] integerForKey:@"SPU_SyncMode"] == SPU_SYNC_MODE_DUAL_SYNC_ASYNC)
-	{
-		[[prefWindowDelegate spuSyncMethodMenu] setEnabled:NO];
-	}
-	else
-	{
-		[[prefWindowDelegate spuSyncMethodMenu] setEnabled:YES];
-	}
-	
-	// Set the name of the autoloaded ROM.
-	NSString *autoloadRomPath = [[NSUserDefaults standardUserDefaults] stringForKey:@"General_AutoloadROMSelectedPath"];
-	if (autoloadRomPath != nil)
-	{
-		[prefBindings setValue:[autoloadRomPath lastPathComponent] forKey:@"AutoloadRomName"];
-	}
-	else
-	{
-		[prefBindings setValue:NSSTRING_STATUS_AUTOLOAD_ROM_NAME_NONE forKey:@"AutoloadRomName"];
-	}
-	
-	// Set the menu for the display rotation.
-	const double displayRotation = (double)[[NSUserDefaults standardUserDefaults] floatForKey:@"DisplayView_Rotation"];
-	[prefWindowDelegate updateDisplayRotationMenu:displayRotation];
-	
-	// Set the default sound volume per user preferences.
-	[prefWindowDelegate updateVolumeIcon:nil];
+	// Set up GDB stub settings per user preferences.
+#ifdef GDB_STUB
+	[cdsCore setEnableGdbStubARM9:[[NSUserDefaults standardUserDefaults] boolForKey:@"Debug_GDBStubEnableARM9"]];
+	[cdsCore setEnableGdbStubARM7:[[NSUserDefaults standardUserDefaults] boolForKey:@"Debug_GDBStubEnableARM7"]];
+	[cdsCore setGdbStubPortARM9:[[NSUserDefaults standardUserDefaults] integerForKey:@"Debug_GDBStubPortARM9"]];
+	[cdsCore setGdbStubPortARM7:[[NSUserDefaults standardUserDefaults] integerForKey:@"Debug_GDBStubPortARM7"]];
+#else
+	[cdsCore setEnableGdbStubARM9:NO];
+	[cdsCore setEnableGdbStubARM7:NO];
+	[cdsCore setGdbStubPortARM9:0];
+	[cdsCore setGdbStubPortARM7:0];
+#endif
 	
 	// Set up the user's default input settings.
 	NSDictionary *userMappings = [[NSUserDefaults standardUserDefaults] dictionaryForKey:@"Input_ControllerMappings"];
@@ -591,28 +577,21 @@
 	}
 	
 	[inputManager setMappingsWithMappings:userMappings];
+	[[inputManager hidManager] setDeviceListController:inputDeviceListController];
+	
+	// Set up the ROM Info panel.
+	[romInfoPanel setupUserDefaults];
+	
+	// Set up the preferences window.
+	[prefWindowDelegate setupUserDefaults];
+	
+	// Set up the default SLOT-2 device.
+	[slot2WindowDelegate setupUserDefaults];
 	
 	// Set up the rest of the emulation-related user defaults.
 	[emuControl setupUserDefaults];
 }
 
-- (void) setRomInfoPanelBoxTitleColors
-{
-	NSColor *boxTitleColor = [NSColor whiteColor];
-	
-	[[boxGeneralInfo titleCell] setTextColor:boxTitleColor];
-	[[boxTitles titleCell] setTextColor:boxTitleColor];
-	[[boxARMBinaries titleCell] setTextColor:boxTitleColor];
-	[[boxFileSystem titleCell] setTextColor:boxTitleColor];
-	[[boxMisc titleCell] setTextColor:boxTitleColor];
-	
-	[boxGeneralInfo setNeedsDisplay:YES];
-	[boxTitles setNeedsDisplay:YES];
-	[boxARMBinaries setNeedsDisplay:YES];
-	[boxFileSystem setNeedsDisplay:YES];
-	[boxMisc setNeedsDisplay:YES];
-}
-
 - (void) restoreDisplayWindowStates
 {
 	EmuControllerDelegate *emuControl = (EmuControllerDelegate *)[emuControlController content];
@@ -642,9 +621,11 @@
 			const NSInteger displayOrientation		= [(NSNumber *)[windowProperties valueForKey:@"displayOrientation"] integerValue];
 			const NSInteger displayOrder			= [(NSNumber *)[windowProperties valueForKey:@"displayOrder"] integerValue];
 			const double displayGap					= [(NSNumber *)[windowProperties valueForKey:@"displayGap"] doubleValue];
-			const NSInteger videoFilterType			= [(NSNumber *)[windowProperties valueForKey:@"videoFilterType"] integerValue];
+			const BOOL videoFiltersPreferGPU		= [(NSNumber *)[windowProperties valueForKey:@"videoFiltersPreferGPU"] boolValue];
+			const BOOL videoSourceDeposterize		= [(NSNumber *)[windowProperties valueForKey:@"videoSourceDeposterize"] boolValue];
+			const NSInteger videoPixelScaler		= [(NSNumber *)[windowProperties valueForKey:@"videoFilterType"] integerValue];
+			const NSInteger videoOutputFilter		= [(NSNumber *)[windowProperties valueForKey:@"videoOutputFilter"] integerValue];
 			const NSInteger screenshotFileFormat	= [(NSNumber *)[windowProperties valueForKey:@"screenshotFileFormat"] integerValue];
-			const BOOL useBilinearOutput			= [(NSNumber *)[windowProperties valueForKey:@"useBilinearOutput"] boolValue];
 			const BOOL useVerticalSync				= [(NSNumber *)[windowProperties valueForKey:@"useVerticalSync"] boolValue];
 			const BOOL isMinSizeNormal				= [(NSNumber *)[windowProperties valueForKey:@"isMinSizeNormal"] boolValue];
 			const BOOL isShowingStatusBar			= [(NSNumber *)[windowProperties valueForKey:@"isShowingStatusBar"] boolValue];
@@ -661,15 +642,17 @@
 			
 			[windowController setIsMinSizeNormal:isMinSizeNormal];
 			[windowController setIsShowingStatusBar:isShowingStatusBar];
-			[windowController setVideoFilterType:videoFilterType];
+			[windowController setVideoFiltersPreferGPU:videoFiltersPreferGPU];
+			[windowController setVideoSourceDeposterize:videoSourceDeposterize];
+			[windowController setVideoPixelScaler:videoPixelScaler];
+			[windowController setVideoOutputFilter:videoOutputFilter];
 			[windowController setDisplayMode:displayMode];
 			[windowController setDisplayRotation:displayRotation];
 			[windowController setDisplayOrientation:displayOrientation];
 			[windowController setDisplayOrder:displayOrder];
 			[windowController setDisplayGap:displayGap];
 			[windowController setScreenshotFileFormat:screenshotFileFormat];
-			[windowController setUseBilinearOutput:useBilinearOutput];
-			[windowController setUseVerticalSync:useVerticalSync];
+			[[windowController view] setUseVerticalSync:useVerticalSync];
 			[windowController setDisplayScale:displayScale];
 			
 			[[windowController masterWindow] setFrameOrigin:NSMakePoint(frameX, frameY)];
@@ -688,14 +671,7 @@
 			}
 			
 			// Draw the display view now so that we guarantee that its drawn at least once.
-			if ([emuControl currentRom] == nil)
-			{
-				[[windowController view] clearToBlack];
-			}
-			else
-			{
-				[[windowController view] setNeedsDisplay:YES];
-			}
+			[CocoaDSUtil messageSendOneWay:[[windowController cdsVideoOutput] receivePort] msgID:MESSAGE_REPROCESS_AND_REDRAW];
 			
 			// If this window is set to full screen mode, its associated screen index must
 			// exist. If not, this window will not enter full screen mode. This is necessary,
@@ -738,10 +714,12 @@
 											  [NSNumber numberWithInteger:[windowController displayOrientation]], @"displayOrientation",
 											  [NSNumber numberWithInteger:[windowController displayOrder]], @"displayOrder",
 											  [NSNumber numberWithDouble:[windowController displayGap]], @"displayGap",
-											  [NSNumber numberWithInteger:[windowController videoFilterType]], @"videoFilterType",
+											  [NSNumber numberWithBool:[windowController videoFiltersPreferGPU]], @"videoFiltersPreferGPU",
+											  [NSNumber numberWithInteger:[windowController videoPixelScaler]], @"videoFilterType",
 											  [NSNumber numberWithInteger:[windowController screenshotFileFormat]], @"screenshotFileFormat",
-											  [NSNumber numberWithBool:[windowController useBilinearOutput]], @"useBilinearOutput",
-											  [NSNumber numberWithBool:[windowController useVerticalSync]], @"useVerticalSync",
+											  [NSNumber numberWithInteger:[windowController videoOutputFilter]], @"videoOutputFilter",
+											  [NSNumber numberWithBool:[windowController videoSourceDeposterize]], @"videoSourceDeposterize",
+											  [NSNumber numberWithBool:[[windowController view] useVerticalSync]], @"useVerticalSync",
 											  [NSNumber numberWithBool:[windowController isMinSizeNormal]], @"isMinSizeNormal",
 											  [NSNumber numberWithBool:[windowController isShowingStatusBar]], @"isShowingStatusBar",
 											  [NSNumber numberWithBool:isInFullScreenMode], @"isInFullScreenMode",
