@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012-2013 DeSmuME team
+	Copyright (C) 2012-2014 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -39,12 +39,14 @@
 	
 	NSWindow *inputProfileSheet;
 	NSWindow *inputProfileRenameSheet;
+	NSWindow *inputSettingsNDSInput;
 	NSWindow *inputSettingsMicrophone;
 	NSWindow *inputSettingsTouch;
 	NSWindow *inputSettingsLoadStateSlot;
 	NSWindow *inputSettingsSaveStateSlot;
 	NSWindow *inputSettingsSetSpeedLimit;
 	NSWindow *inputSettingsGPUState;
+	NSWindow *inputSettingsPaddleController;
 	
 	InputManager *inputManager;
 	NSString *configInputTargetID;
@@ -69,18 +71,21 @@
 
 @property (readonly) IBOutlet NSWindow *inputProfileSheet;
 @property (readonly) IBOutlet NSWindow *inputProfileRenameSheet;
+@property (readonly) IBOutlet NSWindow *inputSettingsNDSInput;
 @property (readonly) IBOutlet NSWindow *inputSettingsMicrophone;
 @property (readonly) IBOutlet NSWindow *inputSettingsTouch;
 @property (readonly) IBOutlet NSWindow *inputSettingsLoadStateSlot;
 @property (readonly) IBOutlet NSWindow *inputSettingsSaveStateSlot;
 @property (readonly) IBOutlet NSWindow *inputSettingsSetSpeedLimit;
 @property (readonly) IBOutlet NSWindow *inputSettingsGPUState;
+@property (readonly) IBOutlet NSWindow *inputSettingsPaddleController;
 
 @property (readonly) IBOutlet InputManager *inputManager;
 @property (retain) NSString *configInputTargetID;
 @property (retain) NSMutableDictionary *inputSettingsInEdit;
 
 - (void) initSettingsSheets;
+- (void) loadSavedProfilesList;
 - (void) populateInputProfileMenu;
 
 - (BOOL) handleKeyboardEvent:(NSEvent *)theEvent keyPressed:(BOOL)keyPressed;
