@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012-2013 DeSmuME team
+	Copyright (C) 2012-2014 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -36,12 +36,14 @@
 @synthesize inputPrefOutlineView;
 @synthesize inputSettingsController;
 @synthesize inputProfileController;
+@synthesize inputSettingsNDSInput;
 @synthesize inputSettingsMicrophone;
 @synthesize inputSettingsTouch;
 @synthesize inputSettingsLoadStateSlot;
 @synthesize inputSettingsSaveStateSlot;
 @synthesize inputSettingsSetSpeedLimit;
 @synthesize inputSettingsGPUState;
+@synthesize inputSettingsPaddleController;
 @synthesize inputProfileSheet;
 @synthesize inputProfileRenameSheet;
 @synthesize inputManager;
@@ -97,38 +99,7 @@
 		[defaultProfilesList addObject:[reconstructedProfile autorelease]];
 	}
 	
-	NSArray *userDefaultsSavedProfilesList = (NSArray *)[[NSUserDefaults standardUserDefaults] arrayForKey:@"Input_SavedProfiles"];
 	savedProfilesList = [[NSMutableArray alloc] initWithCapacity:32];
-	
-	for (NSDictionary *theProfile in userDefaultsSavedProfilesList)
-	{
-		NSMutableDictionary *reconstructedProfile = [[NSMutableDictionary alloc] initWithCapacity:[theProfile count]];
-		
-		for (NSString *profileKey in theProfile)
-		{
-			if ([profileKey isEqualToString:@"Mappings"])
-			{
-				NSDictionary *profileMappings = (NSDictionary *)[theProfile objectForKey:profileKey];
-				NSMutableDictionary *reconstructedMappings = [[NSMutableDictionary alloc] initWithCapacity:[profileMappings count]];
-				
-				for (NSString *mappingKey in profileMappings)
-				{
-					NSArray *inputList = (NSArray *)[profileMappings objectForKey:mappingKey];
-					NSMutableArray *newInputList = [[NSMutableArray alloc] initWithArray:inputList copyItems:YES];
-					[reconstructedMappings setObject:[newInputList autorelease] forKey:mappingKey];
-				}
-				
-				[reconstructedProfile setObject:[reconstructedMappings autorelease] forKey:profileKey];
-			}
-			else
-			{
-				[reconstructedProfile setObject:[theProfile objectForKey:profileKey] forKey:profileKey];
-			}
-		}
-		
-		[savedProfilesList addObject:[reconstructedProfile autorelease]];
-	}
-	
 	configInputTargetID = nil;
 	configInputList = [[NSMutableDictionary alloc] initWithCapacity:128];
 	inputSettingsInEdit = nil;
@@ -167,15 +138,63 @@
 - (void) initSettingsSheets
 {
 	inputSettingsMappings = [[NSDictionary alloc] initWithObjectsAndKeys:
+							 inputSettingsNDSInput,			@"Up",
+							 inputSettingsNDSInput,			@"Down",
+							 inputSettingsNDSInput,			@"Right",
+							 inputSettingsNDSInput,			@"Left",
+							 inputSettingsNDSInput,			@"A",
+							 inputSettingsNDSInput,			@"B",
+							 inputSettingsNDSInput,			@"X",
+							 inputSettingsNDSInput,			@"Y",
+							 inputSettingsNDSInput,			@"L",
+							 inputSettingsNDSInput,			@"R",
+							 inputSettingsNDSInput,			@"Start",
+							 inputSettingsNDSInput,			@"Select",
 							 inputSettingsMicrophone,		@"Microphone",
 							 inputSettingsTouch,			@"Touch",
 							 inputSettingsLoadStateSlot,	@"Load State Slot",
 							 inputSettingsSaveStateSlot,	@"Save State Slot",
 							 inputSettingsSetSpeedLimit,	@"Set Speed",
 							 inputSettingsGPUState,			@"Enable/Disable GPU State",
+							 inputSettingsPaddleController,	@"Paddle",
 							 nil];
 }
 
+- (void) loadSavedProfilesList
+{
+	NSArray *userDefaultsSavedProfilesList = (NSArray *)[[NSUserDefaults standardUserDefaults] arrayForKey:@"Input_SavedProfiles"];
+	[savedProfilesList removeAllObjects];
+	
+	for (NSDictionary *theProfile in userDefaultsSavedProfilesList)
+	{
+		NSMutableDictionary *reconstructedProfile = [[NSMutableDictionary alloc] initWithCapacity:[theProfile count]];
+		
+		for (NSString *profileKey in theProfile)
+		{
+			if ([profileKey isEqualToString:@"Mappings"])
+			{
+				NSDictionary *profileMappings = (NSDictionary *)[theProfile objectForKey:profileKey];
+				NSMutableDictionary *reconstructedMappings = [[NSMutableDictionary alloc] initWithCapacity:[profileMappings count]];
+				
+				for (NSString *mappingKey in profileMappings)
+				{
+					NSArray *inputList = (NSArray *)[profileMappings objectForKey:mappingKey];
+					NSMutableArray *newInputList = [[NSMutableArray alloc] initWithArray:inputList copyItems:YES];
+					[reconstructedMappings setObject:[newInputList autorelease] forKey:mappingKey];
+				}
+				
+				[reconstructedProfile setObject:[reconstructedMappings autorelease] forKey:profileKey];
+			}
+			else
+			{
+				[reconstructedProfile setObject:[theProfile objectForKey:profileKey] forKey:profileKey];
+			}
+		}
+		
+		[savedProfilesList addObject:[reconstructedProfile autorelease]];
+	}
+}
+
 - (void) populateInputProfileMenu
 {
 	NSDictionary *defaultKeyMappingsDict = [NSDictionary dictionaryWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"DefaultKeyMappings" ofType:@"plist"]];
@@ -414,35 +433,33 @@
 }
 
 #pragma mark InputHIDManagerTarget Protocol
-- (BOOL) handleHIDQueue:(IOHIDQueueRef)hidQueue
+- (BOOL) handleHIDQueue:(IOHIDQueueRef)hidQueue hidManager:(InputHIDManager *)hidManager
 {
 	BOOL isHandled = NO;
+	NSString *cmdTagTarget = [self configInputTargetID];
 	
-	if ([self configInputTargetID] == nil)
+	if (cmdTagTarget == nil)
 	{
 		ClearHIDQueue(hidQueue);
 		return isHandled;
 	}
 	
-	InputAttributesList inputList = InputManagerEncodeHIDQueue(hidQueue);
+	CommandAttributes cmdAttr = [[hidManager inputManager] defaultCommandAttributesForCommandTag:[cmdTagTarget cStringUsingEncoding:NSUTF8StringEncoding]];
+	bool forceDigitalInput = !cmdAttr.allowAnalogInput;
+	InputAttributesList inputList = InputManagerEncodeHIDQueue(hidQueue, [hidManager inputManager], forceDigitalInput);
 	const size_t inputCount = inputList.size();
 	
 	for (size_t i = 0; i < inputCount; i++)
 	{
 		const InputAttributes &inputAttr = inputList[i];
-		char inputKey[INPUT_HANDLER_STRING_LENGTH*2];
-		strlcpy(inputKey, inputAttr.deviceCode, INPUT_HANDLER_STRING_LENGTH*2);
-		strlcat(inputKey, ":", INPUT_HANDLER_STRING_LENGTH*2);
-		strlcat(inputKey, inputAttr.elementCode, INPUT_HANDLER_STRING_LENGTH*2);
-		
-		NSString *inputKeyStr = [NSString stringWithCString:inputKey encoding:NSUTF8StringEncoding];
-		NSDate *inputOnDate = [configInputList valueForKey:inputKeyStr];
+		NSString *inputKey = [NSString stringWithFormat:@"%s:%s", inputAttr.deviceCode, inputAttr.elementCode];
+		NSDate *inputOnDate = [configInputList valueForKey:inputKey];
 		
 		if (inputAttr.state == INPUT_ATTRIBUTE_STATE_ON)
 		{
 			if (inputOnDate == nil)
 			{
-				[configInputList setValue:[NSDate date] forKey:inputKeyStr];
+				[configInputList setValue:[NSDate date] forKey:inputKey];
 			}
 		}
 		else
@@ -452,11 +469,11 @@
 				if (([inputOnDate timeIntervalSinceNow] * -1.0) < INPUT_HOLD_TIME)
 				{
 					// If the button isn't held for at least INPUT_HOLD_TIME seconds, then reject the input.
-					[configInputList setValue:nil forKey:inputKeyStr];
+					[configInputList setValue:nil forKey:inputKey];
 				}
 				else
 				{
-					isHandled = [self addMappingUsingInputAttributes:&inputAttr commandTag:[self configInputTargetID]];
+					isHandled = [self addMappingUsingInputAttributes:&inputAttr commandTag:cmdTagTarget];
 					break;
 				}
 			}
@@ -521,7 +538,7 @@
 	{
 		NSFont *newFont = [[NSFontManager sharedFontManager] fontWithFamily:[[outCell font] familyName]
 																	 traits:([item isKindOfClass:[NSArray class]]) ? NSBoldFontMask : 0
-																	 weight:0
+																	 weight:5
 																	   size:[[outCell font] pointSize]];
 		[outCell setFont:newFont];
 	}
@@ -529,7 +546,7 @@
 	{
 		NSFont *newFont = [[NSFontManager sharedFontManager] fontWithFamily:[[outCell font] familyName]
 																	 traits:([item isKindOfClass:[NSArray class]]) ? NSBoldFontMask : 0
-																	 weight:0
+																	 weight:5
 																	   size:[[outCell font] pointSize]];
 		[outCell setFont:newFont];
 	}
