@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011 Roger Manuel
-	Copyright (C) 2012-2013 DeSmuME team
+	Copyright (C) 2012-2015 DeSmuME Team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -27,11 +27,230 @@
 #import "cocoa_videofilter.h"
 #import "cocoa_util.h"
 
+#ifdef MAC_OS_X_VERSION_10_7
+#include "../OGLDisplayOutput_3_2.h"
+#else
+#include "../OGLDisplayOutput.h"
+#endif
+
+
+#pragma mark -
+@implementation DisplayPreviewView
+
+@dynamic filtersPreferGPU;
+@dynamic sourceDeposterize;
+@dynamic pixelScaler;
+@dynamic outputFilter;
+
+- (id)initWithFrame:(NSRect)frameRect
+{
+	self = [super initWithFrame:frameRect];
+	if (self == nil)
+	{
+		return self;
+	}
+	
+	isPreviewImageLoaded = false;
+		
+	// Initialize the OpenGL context
+	NSOpenGLPixelFormatAttribute attributes[] = {
+		NSOpenGLPFAColorSize, (NSOpenGLPixelFormatAttribute)24,
+		NSOpenGLPFAAlphaSize, (NSOpenGLPixelFormatAttribute)8,
+		NSOpenGLPFADepthSize, (NSOpenGLPixelFormatAttribute)0,
+		NSOpenGLPFAStencilSize, (NSOpenGLPixelFormatAttribute)0,
+		(NSOpenGLPixelFormatAttribute)0, (NSOpenGLPixelFormatAttribute)0,
+		(NSOpenGLPixelFormatAttribute)0 };
+	
+#ifdef _OGLDISPLAYOUTPUT_3_2_H_
+	// If we can support a 3.2 Core Profile context, then request that in our
+	// pixel format attributes.
+	if (IsOSXVersionSupported(10, 7, 0))
+	{
+		attributes[8] = NSOpenGLPFAOpenGLProfile;
+		attributes[9] = NSOpenGLProfileVersion3_2Core;
+		OGLInfoCreate_Func = &OGLInfoCreate_3_2;
+	}
+#endif
+	
+	NSOpenGLPixelFormat *format = [[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];
+	if (format == nil)
+	{
+		// If we can't get a 3.2 Core Profile context, then switch to using a
+		// legacy context instead.
+		attributes[8] = (NSOpenGLPixelFormatAttribute)0;
+		attributes[9] = (NSOpenGLPixelFormatAttribute)0;
+		OGLInfoCreate_Func = &OGLInfoCreate_Legacy;
+		format = [[NSOpenGLPixelFormat alloc] initWithAttributes:attributes];
+	}
+	
+	context = [[NSOpenGLContext alloc] initWithFormat:format shareContext:nil];
+	[format release];
+	cglDisplayContext = (CGLContextObj)[context CGLContextObj];
+	
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	
+	OGLInfo *oglInfo = OGLInfoCreate_Func();
+	oglImage = new OGLImage(oglInfo, 64, 64, frameRect.size.width, frameRect.size.height);
+	oglImage->SetFiltersPreferGPUOGL(true);
+	oglImage->SetSourceDeposterize(false);
+	oglImage->SetOutputFilterOGL(OutputFilterTypeID_Bilinear);
+	oglImage->SetPixelScalerOGL(VideoFilterTypeID_None);
+	
+	CGLSetCurrentContext(prevContext);
+	
+	return self;
+}
+
+- (void)dealloc
+{
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	delete oglImage;
+	CGLSetCurrentContext(prevContext);
+	
+	[context clearDrawable];
+	[context release];
+	
+	[super dealloc];
+}
+
+- (void) setFiltersPreferGPU:(BOOL)theState
+{
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	oglImage->SetFiltersPreferGPUOGL(theState ? true : false);
+	oglImage->ProcessOGL();
+	CGLSetCurrentContext(prevContext);
+}
+
+- (BOOL) filtersPreferGPU
+{
+	return (oglImage->GetFiltersPreferGPU() ? YES : NO);
+}
+
+- (void) setSourceDeposterize:(BOOL)theState
+{
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	oglImage->SetSourceDeposterize(theState ? true : false);
+	oglImage->ProcessOGL();
+	CGLSetCurrentContext(prevContext);
+}
+
+- (BOOL) sourceDeposterize
+{
+	return (oglImage->GetSourceDeposterize() ? YES : NO);
+}
+
+- (void) setPixelScaler:(NSInteger)scalerID
+{
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	oglImage->SetPixelScalerOGL(scalerID);
+	oglImage->ProcessOGL();
+	CGLSetCurrentContext(prevContext);
+}
+
+- (NSInteger) pixelScaler
+{
+	return (NSInteger)oglImage->GetPixelScaler();
+}
+
+- (void) setOutputFilter:(NSInteger)outputFilterID
+{
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	oglImage->SetOutputFilterOGL(outputFilterID);
+	CGLSetCurrentContext(prevContext);
+}
+
+- (NSInteger) outputFilter
+{
+	return (NSInteger)oglImage->GetOutputFilter();
+}
+
+- (void) loadPreviewImage:(NSImage *)previewImage
+{
+	NSArray *imageRepArray = [previewImage representations];
+	const NSBitmapImageRep *imageRep = [imageRepArray objectAtIndex:0];
+	const size_t previewWidth = (GLsizei)[previewImage size].width;
+	const size_t previewHeight = (GLsizei)[previewImage size].height;
+	
+	// When an NSImage is loaded from file, it is loaded as ABGR (or RGBA for big-endian).
+	// However, the OpenGL blitter takes BGRA format. Therefore, we need to convert the
+	// pixel format before sending to OpenGL.
+	
+	uint32_t *bitmapData = (uint32_t *)[imageRep bitmapData];
+	for (size_t i = 0; i < previewWidth * previewHeight; i++)
+	{
+		const uint32_t color = bitmapData[i];
+		
+#if defined(__i386__) || defined(__x86_64__)
+		bitmapData[i]	=           0xFF000000         | // lA
+						  ((color & 0x00FF0000) >> 16) | // lB -> lR
+						   (color & 0x0000FF00)        | // lG
+						  ((color & 0x000000FF) << 16);  // lR -> lB
+#else
+		bitmapData[i]	=           0xFF000000         | // lA
+						  ((color & 0xFF000000) >>  8) | // bR -> lR
+						   (color & 0x00FF0000) >>  8  | // bG -> lG
+						  ((color & 0x0000FF00) >>  8);  // bB -> lB
+#endif
+		
+	}
+	
+	// Send the NSImage to OpenGL.
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	oglImage->LoadFrameOGL(bitmapData, 0, 0, previewWidth, previewHeight);
+	oglImage->ProcessOGL();
+	CGLSetCurrentContext(prevContext);
+}
+
+- (BOOL)isOpaque
+{
+	return YES;
+}
+
+- (void)lockFocus
+{
+	[super lockFocus];
+	
+	if ([context view] != self)
+	{
+		[context setView:self];
+	}
+}
+
+- (void)drawRect:(NSRect)dirtyRect
+{
+	if (!isPreviewImageLoaded)
+	{
+		// Load the preview image.
+		NSImage *previewImage = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"VideoFilterPreview_64x64" ofType:@"png"]];
+		[self loadPreviewImage:previewImage];
+		[previewImage release];
+		
+		isPreviewImageLoaded = true;
+	}
+	
+	CGLContextObj prevContext = CGLGetCurrentContext();
+	CGLSetCurrentContext(cglDisplayContext);
+	oglImage->RenderOGL();
+	glFlush();
+	CGLSetCurrentContext(prevContext);
+}
+
+@end
+
+#pragma mark -
 
 @implementation PreferencesWindowDelegate
 
 @synthesize dummyObject;
 @synthesize window;
+@synthesize toolbar;
 @synthesize firmwareConfigSheet;
 @synthesize cdsCoreController;
 @synthesize emuController;
@@ -50,7 +269,7 @@
 @synthesize displayRotationField;
 @synthesize spuSyncMethodMenu;
 
-@synthesize previewImageView;
+@synthesize previewView;
 
 @synthesize bindings;
 
@@ -70,49 +289,15 @@
 		return self;
 	}
 	
-	VideoFilterTypeID vfType = (VideoFilterTypeID)[[NSUserDefaults standardUserDefaults] integerForKey:@"DisplayView_VideoFilter"];
-	NSImage *videoFilterImage = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"VideoFilterPreview_64x64" ofType:@"png"]];
-	
-	videoFilter = [[CocoaVideoFilter alloc] initWithSize:[videoFilterImage size] typeID:vfType];
-	NSSize vfDestSize = [videoFilter destSize];
-	NSUInteger vfWidth = (NSUInteger)vfDestSize.width;
-	NSUInteger vfHeight = (NSUInteger)vfDestSize.height;
-	
-	NSArray *imageRepArray = [videoFilterImage representations];
-	const NSBitmapImageRep *imageRep = [imageRepArray objectAtIndex:0];
-	RGBA8888ForceOpaqueBuffer((const uint32_t *)[imageRep bitmapData], (uint32_t *)[videoFilter srcBufferPtr], (64 * 64));
-	[videoFilterImage release];
-	
-	BOOL useBilinear = [[NSUserDefaults standardUserDefaults] boolForKey:@"DisplayView_UseBilinearOutput"];
-	
-	if (vfWidth <= 64 || vfHeight <= 64)
-	{
-		[videoFilter changeFilter:VideoFilterTypeID_Nearest2X];
-		vfDestSize = [videoFilter destSize];
-		vfWidth = (NSUInteger)vfDestSize.width;
-		vfHeight = (NSUInteger)vfDestSize.height;
-		
-		bilinearVideoFilter = [[CocoaVideoFilter alloc] initWithSize:vfDestSize typeID:(useBilinear) ? VideoFilterTypeID_Bilinear : VideoFilterTypeID_Nearest2X];
-	}
-	else if  (vfWidth >= 256 || vfHeight >= 256)
-	{
-		bilinearVideoFilter = [[CocoaVideoFilter alloc] initWithSize:vfDestSize typeID:VideoFilterTypeID_None];
-	}
-	else
-	{
-		bilinearVideoFilter = [[CocoaVideoFilter alloc] initWithSize:vfDestSize typeID:(useBilinear) ? VideoFilterTypeID_Bilinear : VideoFilterTypeID_Nearest2X];
-	}
-	
-	RGBA8888ForceOpaqueBuffer((const uint32_t *)[videoFilter runFilter], (uint32_t *)[bilinearVideoFilter srcBufferPtr], (vfWidth * vfHeight));
-	[bindings setObject:[bilinearVideoFilter image] forKey:@"VideoFilterPreviewImage"];	
-		
+	// Load the volume icons.
 	iconVolumeFull = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon_VolumeFull_16x16" ofType:@"png"]];
 	iconVolumeTwoThird = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon_VolumeTwoThird_16x16" ofType:@"png"]];
 	iconVolumeOneThird = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon_VolumeOneThird_16x16" ofType:@"png"]];
 	iconVolumeMute = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Icon_VolumeMute_16x16" ofType:@"png"]];
-	
 	[bindings setObject:iconVolumeFull forKey:@"volumeIconImage"];
 	
+	prefViewDict = nil;
+	
 	return self;
 }
 
@@ -123,35 +308,36 @@
 	[iconVolumeOneThird release];
 	[iconVolumeMute release];
 	[bindings release];
-	[videoFilter release];
+	[prefViewDict release];
 	
 	[super dealloc];
 }
 
-- (IBAction) showGeneralView:(id)sender
+- (IBAction) changePrefView:(id)sender
 {
-	[self switchContentView:viewGeneral];
-}
-
-- (IBAction) showInputView:(id)sender
-{
-	[self switchContentView:(NSView *)viewInput];
-	[window makeFirstResponder:(NSView *)viewInput];
-}
-
-- (IBAction) showDisplayView:(id)sender
-{
-	[self switchContentView:viewDisplay];
-}
-
-- (IBAction) showSoundView:(id)sender
-{
-	[self switchContentView:viewSound];
-}
-
-- (IBAction) showEmulationView:(id)sender
-{
-	[self switchContentView:viewEmulation];
+	if (prefViewDict == nil)
+	{
+		// Associates NSView objects to their respective toolbar identifiers.
+		prefViewDict = [[NSDictionary alloc] initWithObjectsAndKeys:
+						viewGeneral, @"General",
+						viewInput, @"Input",
+						viewDisplay, @"Display",
+						viewSound, @"Sound",
+						viewEmulation, @"Emulation",
+						nil];
+	}
+	
+	NSString *toolbarItemIdentifier = [[self toolbar] selectedItemIdentifier];
+	NSView *theView = [prefViewDict objectForKey:toolbarItemIdentifier];
+	if (theView != nil)
+	{
+		[self switchContentView:theView];
+		
+		if ([toolbarItemIdentifier isEqualToString:@"Input"])
+		{
+			[window makeFirstResponder:theView];
+		}
+	}
 }
 
 - (IBAction) chooseRomForAutoload:(id)sender
@@ -377,116 +563,65 @@
 	}
 }
 
-- (IBAction) setUseBilinear:(id)sender
+- (IBAction) updateFiltersPreferGPU:(id)sender
 {
-	const BOOL useBilinear = [CocoaDSUtil getIBActionSenderButtonStateBool:sender];
-	const NSUInteger previewSrcWidth = (NSUInteger)[bilinearVideoFilter srcSize].width;
-	const NSUInteger previewSrcHeight = (NSUInteger)[bilinearVideoFilter srcSize].height;
-	
-	if (previewSrcWidth <= 128 || previewSrcHeight <= 128)
-	{
-		[bilinearVideoFilter changeFilter:(useBilinear) ? VideoFilterTypeID_Bilinear : VideoFilterTypeID_Nearest2X];
-	}
-	
-	NSBitmapImageRep *newPreviewImageRep = [bilinearVideoFilter bitmapImageRep];
-	
-	NSImage *videoFilterPreviewImage = [bindings objectForKey:@"VideoFilterPreviewImage"];
-	NSArray *imageRepArray = [videoFilterPreviewImage representations];
-	NSImageRep *oldImageRep = [imageRepArray objectAtIndex:0];
-	[videoFilterPreviewImage removeRepresentation:oldImageRep];
-	[videoFilterPreviewImage addRepresentation:newPreviewImageRep];
-	
-	[previewImageView setNeedsDisplay:YES];
+	const BOOL theState = [CocoaDSUtil getIBActionSenderButtonStateBool:sender];
+	[[self previewView] setFiltersPreferGPU:theState];
+	[previewView setNeedsDisplay:YES];
 }
 
-- (IBAction) selectVideoFilterType:(id)sender
+- (IBAction) updateSourceDeposterize:(id)sender
 {
-	const VideoFilterTypeID vfType = (VideoFilterTypeID)[CocoaDSUtil getIBActionSenderTag:sender];
-	const BOOL useBilinear = [[NSUserDefaults standardUserDefaults] boolForKey:@"DisplayView_UseBilinearOutput"];
-	
-	[[NSUserDefaults standardUserDefaults] setInteger:vfType forKey:@"DisplayView_VideoFilter"];
-	
-	[videoFilter changeFilter:vfType];
-	NSSize vfDestSize = [videoFilter destSize];
-	NSUInteger vfWidth = (NSUInteger)[videoFilter destSize].width;
-	NSUInteger vfHeight = (NSUInteger)[videoFilter destSize].height;
-	
-	if (vfWidth <= 64 || vfHeight <= 64)
-	{
-		[videoFilter changeFilter:VideoFilterTypeID_Nearest2X];
-		vfDestSize = [videoFilter destSize];
-		vfWidth = (NSUInteger)vfDestSize.width;
-		vfHeight = (NSUInteger)vfDestSize.height;
-		
-		[bilinearVideoFilter setSourceSize:vfDestSize];
-		[bilinearVideoFilter changeFilter:(useBilinear) ? VideoFilterTypeID_Bilinear : VideoFilterTypeID_Nearest2X];
-	}
-	else if (vfWidth >= 256 || vfHeight >= 256)
-	{
-		[bilinearVideoFilter setSourceSize:vfDestSize];
-		[bilinearVideoFilter changeFilter:VideoFilterTypeID_None];
-	}
-	else
-	{
-		[bilinearVideoFilter setSourceSize:vfDestSize];
-		[bilinearVideoFilter changeFilter:(useBilinear) ? VideoFilterTypeID_Bilinear : VideoFilterTypeID_Nearest2X];
-	}
-	
-	RGBA8888ForceOpaqueBuffer((const uint32_t *)[videoFilter runFilter], (uint32_t *)[bilinearVideoFilter srcBufferPtr], (vfWidth * vfHeight));
-	NSBitmapImageRep *newPreviewImageRep = [bilinearVideoFilter bitmapImageRep];
-	
-	NSImage *videoFilterPreviewImage = [bindings objectForKey:@"VideoFilterPreviewImage"];
-	NSArray *imageRepArray = [videoFilterPreviewImage representations];
-	NSImageRep *oldImageRep = [imageRepArray objectAtIndex:0];
-	[videoFilterPreviewImage removeRepresentation:oldImageRep];
-	[videoFilterPreviewImage addRepresentation:newPreviewImageRep];
-	
-	[previewImageView setNeedsDisplay:YES];
+	const BOOL theState = [CocoaDSUtil getIBActionSenderButtonStateBool:sender];
+	[[self previewView] setSourceDeposterize:theState];
+	[previewView setNeedsDisplay:YES];
+}
+
+- (IBAction) selectOutputFilter:(id)sender
+{
+	const NSInteger filterID = [CocoaDSUtil getIBActionSenderTag:sender];
+	[[NSUserDefaults standardUserDefaults] setInteger:filterID forKey:@"DisplayView_OutputFilter"];
+	[[self previewView] setOutputFilter:filterID];
+	[previewView setNeedsDisplay:YES];
+}
+
+- (IBAction) selectPixelScaler:(id)sender
+{
+	const NSInteger filterID = [CocoaDSUtil getIBActionSenderTag:sender];
+	[[NSUserDefaults standardUserDefaults] setInteger:filterID forKey:@"DisplayView_VideoFilter"];
+	[[self previewView] setPixelScaler:filterID];
+	[previewView setNeedsDisplay:YES];
 }
 
 - (IBAction) updateVolumeIcon:(id)sender
 {
 	NSImage *iconImage = (NSImage *)[bindings objectForKey:@"volumeIconImage"];
+	NSImage *newIconImage = nil;
 	const float vol = [[NSUserDefaults standardUserDefaults] floatForKey:@"Sound_Volume"];
 	
 	if (vol <= 0.0f)
 	{
-		if (iconImage == iconVolumeMute)
-		{
-			return;
-		}
-		
-		iconImage = iconVolumeMute;
+		newIconImage = iconVolumeMute;
 	}
 	else if (vol > 0.0f && vol <= VOLUME_THRESHOLD_LOW)
 	{
-		if (iconImage == iconVolumeOneThird)
-		{
-			return;
-		}
-		
-		iconImage = iconVolumeOneThird;
+		newIconImage = iconVolumeOneThird;
 	}
 	else if (vol > VOLUME_THRESHOLD_LOW && vol <= VOLUME_THRESHOLD_HIGH)
 	{
-		if (iconImage == iconVolumeTwoThird)
-		{
-			return;
-		}
-		
-		iconImage = iconVolumeTwoThird;
+		newIconImage = iconVolumeTwoThird;
 	}
 	else
 	{
-		if (iconImage == iconVolumeFull)
-		{
-			return;
-		}
-		
-		iconImage = iconVolumeFull;
+		newIconImage = iconVolumeFull;
 	}
 	
-	[bindings setObject:iconImage forKey:@"volumeIconImage"];
+	if (newIconImage == iconImage)
+	{
+		return;
+	}
+	
+	[bindings setObject:newIconImage forKey:@"volumeIconImage"];
 }
 
 - (IBAction) selectSPUSyncMode:(id)sender
@@ -731,6 +866,85 @@
 	[tempView release];
 }
 
+- (void) setupUserDefaults
+{
+	// Update input preferences.
+	[viewInput loadSavedProfilesList];
+	
+	// Update the SPU Sync controls in the Preferences window.
+	if ([[NSUserDefaults standardUserDefaults] integerForKey:@"SPU_SyncMode"] == SPU_SYNC_MODE_DUAL_SYNC_ASYNC)
+	{
+		[[self spuSyncMethodMenu] setEnabled:NO];
+	}
+	else
+	{
+		[[self spuSyncMethodMenu] setEnabled:YES];
+	}
+	
+	// Set the menu for the display rotation.
+	const double displayRotation = (double)[[NSUserDefaults standardUserDefaults] floatForKey:@"DisplayView_Rotation"];
+	[self updateDisplayRotationMenu:displayRotation];
+	
+	// Set the default sound volume per user preferences.
+	[self updateVolumeIcon:nil];
+	
+	// Set the default video settings per user preferences.
+	const BOOL filtersPreferGPU = [[NSUserDefaults standardUserDefaults] integerForKey:@"DisplayView_FiltersPreferGPU"];
+	[[self previewView] setFiltersPreferGPU:filtersPreferGPU];
+	
+	const BOOL sourceDeposterize = [[NSUserDefaults standardUserDefaults] integerForKey:@"DisplayView_Deposterize"];
+	[[self previewView] setSourceDeposterize:sourceDeposterize];
+	
+	const NSInteger pixelScalerID = [[NSUserDefaults standardUserDefaults] integerForKey:@"DisplayView_VideoFilter"];
+	[[self previewView] setPixelScaler:pixelScalerID];
+	
+	const NSInteger outputFilterID = [[NSUserDefaults standardUserDefaults] integerForKey:@"DisplayView_OutputFilter"];
+	[[self previewView] setOutputFilter:outputFilterID];
+	
+	// Set up file paths.
+	NSString *arm7BiosImagePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"BIOS_ARM7ImagePath"];
+	if (arm7BiosImagePath != nil)
+	{
+		[bindings setValue:[arm7BiosImagePath lastPathComponent] forKey:@"Arm7BiosImageName"];
+	}
+	
+	NSString *arm9BiosImagePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"BIOS_ARM9ImagePath"];
+	if (arm9BiosImagePath != nil)
+	{
+		[bindings setValue:[arm9BiosImagePath lastPathComponent] forKey:@"Arm9BiosImageName"];
+	}
+	
+	NSString *firmwareImagePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"Emulation_FirmwareImagePath"];
+	if (firmwareImagePath != nil)
+	{
+		[bindings setValue:[firmwareImagePath lastPathComponent] forKey:@"FirmwareImageName"];
+	}
+	
+	NSString *advansceneDatabasePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"Advanscene_DatabasePath"];
+	if (advansceneDatabasePath != nil)
+	{
+		[bindings setValue:[advansceneDatabasePath lastPathComponent] forKey:@"AdvansceneDatabaseName"];
+	}
+	
+	NSString *cheatDatabasePath = [[NSUserDefaults standardUserDefaults] stringForKey:@"R4Cheat_DatabasePath"];
+	if (cheatDatabasePath != nil)
+	{
+		[bindings setValue:[cheatDatabasePath lastPathComponent] forKey:@"R4CheatDatabaseName"];
+	}
+	
+	NSString *autoloadRomPath = [[NSUserDefaults standardUserDefaults] stringForKey:@"General_AutoloadROMSelectedPath"];
+	if (autoloadRomPath != nil)
+	{
+		[bindings setValue:[autoloadRomPath lastPathComponent] forKey:@"AutoloadRomName"];
+	}
+	else
+	{
+		[bindings setValue:NSSTRING_STATUS_NO_ROM_CHOSEN forKey:@"AutoloadRomName"];
+	}
+}
+
+#pragma mark NSWindowDelegate Protocol
+
 - (void)windowDidBecomeKey:(NSNotification *)notification
 {
 	[prefWindowController setContent:bindings];
