@@ -18,6 +18,12 @@
 #include "utilities.h"
 #include <Accelerate/Accelerate.h>
 
+static const uint8_t bits5to8[] = {
+	0x00, 0x08, 0x10, 0x19, 0x21, 0x29, 0x31, 0x3A,
+	0x42, 0x4A, 0x52, 0x5A, 0x63, 0x6B, 0x73, 0x7B,
+	0x84, 0x8C, 0x94, 0x9C, 0xA5, 0xAD, 0xB5, 0xBD,
+	0xC5, 0xCE, 0xD6, 0xDE, 0xE6, 0xEF, 0xF7, 0xFF
+};
 
 static CFStringRef OSXProductName = NULL;
 static CFStringRef OSXProductVersion = NULL;
@@ -136,9 +142,31 @@ bool IsOSXVersionSupported(const unsigned int major, const unsigned int minor, c
  ********************************************************************************************/
 inline uint32_t RGB555ToRGBA8888(const uint16_t color16)
 {
-	return	((color16 & 0x001F) << 3) |
-			((color16 & 0x03E0) << 6) |
-			((color16 & 0x7C00) << 9) |
+	return	(bits5to8[((color16 >> 0) & 0x001F)] << 0) |
+			(bits5to8[((color16 >> 5) & 0x001F)] << 8) |
+			(bits5to8[((color16 >> 10) & 0x001F)] << 16) |
+			0xFF000000;
+}
+
+/********************************************************************************************
+	RGB555ToBGRA8888() - INLINE
+
+	Converts a color from 15-bit RGB555 format into 32-bit BGRA8888 format.
+
+	Takes:
+		color16 - The pixel in 15-bit RGB555 format.
+
+	Returns:
+		A 32-bit unsigned integer containing the BGRA8888 formatted color.
+
+	Details:
+		The input and output pixels are expected to have little-endian byte order.
+ ********************************************************************************************/
+inline uint32_t RGB555ToBGRA8888(const uint16_t color16)
+{
+	return	(bits5to8[((color16 >> 10) & 0x001F)] << 0) |
+			(bits5to8[((color16 >> 5) & 0x001F)] << 8) |
+			(bits5to8[((color16 >> 0) & 0x001F)] << 16) |
 			0xFF000000;
 }
 
@@ -191,6 +219,36 @@ void RGB555ToRGBA8888Buffer(const uint16_t *__restrict__ srcBuffer, uint32_t *__
 	}
 }
 
+/********************************************************************************************
+	RGB555ToBGRA8888Buffer()
+
+	Copies a 15-bit RGB555 pixel buffer into a 32-bit BGRA8888 pixel buffer.
+
+	Takes:
+		srcBuffer - Pointer to the source 15-bit RGB555 pixel buffer.
+
+		destBuffer - Pointer to the destination 32-bit BGRA8888 pixel buffer.
+
+		pixelCount - The number of pixels to copy.
+
+	Returns:
+		Nothing.
+
+	Details:
+		The source and destination pixels are expected to have little-endian byte order.
+		Also, it is the caller's responsibility to ensure that the source and destination
+		buffers are large enough to accomodate the requested number of pixels.
+ ********************************************************************************************/
+void RGB555ToBGRA8888Buffer(const uint16_t *__restrict__ srcBuffer, uint32_t *__restrict__ destBuffer, size_t pixelCount)
+{
+	const uint32_t *__restrict__ destBufferEnd = destBuffer + pixelCount;
+	
+	while (destBuffer < destBufferEnd)
+	{
+		*destBuffer++ = RGB555ToBGRA8888(*srcBuffer++);
+	}
+}
+
 /********************************************************************************************
 	RGBA8888ForceOpaqueBuffer()
 
