@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011-2012 Roger Manuel
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -19,6 +19,37 @@
 #include "videofilter.h"
 #include <string.h>
 
+
+// This function is called when running a filter in multithreaded mode.
+static void* RunVideoFilterTask(void *arg);
+
+// Attributes list of known video filters, indexed using VideoFilterTypeID.
+// Use VideoFilter::GetAttributesByID() to retrieve a filter's attributes.
+const VideoFilterAttributes VideoFilterAttributesList[] = {
+	{VideoFilterTypeID_None,			"None",				NULL,							1,	1,	0},
+	{VideoFilterTypeID_LQ2X,			"LQ2x",				&RenderLQ2X,					2,	1,	0},
+	{VideoFilterTypeID_LQ2XS,			"LQ2xS",			&RenderLQ2XS,					2,	1,	0},
+	{VideoFilterTypeID_HQ2X,			"HQ2x",				&RenderHQ2X,					2,	1,	0},
+	{VideoFilterTypeID_HQ2XS,			"HQ2xS",			&RenderHQ2XS,					2,	1,	0},
+	{VideoFilterTypeID_HQ4X,			"HQ4x",				&RenderHQ4X,					4,	1,	0},
+	{VideoFilterTypeID_2xSaI,			"2xSaI",			&Render2xSaI,					2,	1,	0},
+	{VideoFilterTypeID_Super2xSaI,		"Super 2xSaI",		&RenderSuper2xSaI,				2,	1,	0},
+	{VideoFilterTypeID_SuperEagle,		"Super Eagle",		&RenderSuperEagle,				2,	1,	0},
+	{VideoFilterTypeID_Scanline,		"Scanline",			&RenderScanline,				2,	1,	0},
+	{VideoFilterTypeID_Bilinear,		"Bilinear",			&RenderBilinear,				2,	1,	0},
+	{VideoFilterTypeID_Nearest2X,		"Nearest 2x",		&RenderNearest2X,				2,	1,	0},
+	{VideoFilterTypeID_Nearest1_5X,		"Nearest 1.5x",		&RenderNearest_1Point5x,		3,	2,	0},
+	{VideoFilterTypeID_NearestPlus1_5X,	"Nearest+ 1.5x",	&RenderNearestPlus_1Point5x,	3,	2,	0},
+	{VideoFilterTypeID_EPX,				"EPX",				&RenderEPX,						2,	1,	0},
+	{VideoFilterTypeID_EPXPlus,			"EPX+",				&RenderEPXPlus,					2,	1,	0},
+	{VideoFilterTypeID_EPX1_5X,			"EPX 1.5x",			&RenderEPX_1Point5x,			3,	2,	0},
+	{VideoFilterTypeID_EPXPlus1_5X,		"EPX+ 1.5x",		&RenderEPXPlus_1Point5x,		3,	2,	0},
+	{VideoFilterTypeID_HQ4XS,			"HQ4xS",			&RenderHQ4XS,					4,	1,	0},
+	{VideoFilterTypeID_2xBRZ,			"2xBRZ",			&Render2xBRZ,					2,	1,	0},
+	{VideoFilterTypeID_3xBRZ,			"3xBRZ",			&Render3xBRZ,					3,	1,	0},
+	{VideoFilterTypeID_4xBRZ,			"4xBRZ",			&Render4xBRZ,					4,	1,	0},
+	{VideoFilterTypeID_5xBRZ,			"5xBRZ",			&Render5xBRZ,					5,	1,	0} };
+
 // Parameters for Scanline filter
 int scanline_filter_a = 0;
 int scanline_filter_b = 2;
@@ -52,10 +83,17 @@ VideoFilter::VideoFilter(size_t srcWidth,
 	newSurface.Pitch = srcWidth*2;
 	newSurface.Width = srcWidth;
 	newSurface.Height = srcHeight;
+	newSurface.userData = NULL;
+	
+	for (size_t i = 0; i < FILTER_MAX_WORKING_SURFACE_COUNT; i++)
+	{
+		newSurface.workingSurface[i] = NULL;
+	}
 	
 	_vfSrcSurface = newSurface;
 	_vfDstSurface = newSurface;
 	
+	_useInternalDstBuffer = true;
 	_isFilterRunning = false;
 	_vfSrcSurfacePixBuffer = NULL;
 	
@@ -86,6 +124,7 @@ VideoFilter::VideoFilter(size_t srcWidth,
 		_vfThread[i].task->start(false);
 	}
 	
+	_vfFunc = _vfAttributes.filterFunction;
 	SetSourceSize(srcWidth, srcHeight);
 }
 
@@ -114,8 +153,17 @@ VideoFilter::~VideoFilter()
 		ThreadCondWait(&_condRunning, &_lockDst);
 	}
 	
-	free(_vfDstSurface.Surface);
-	_vfDstSurface.Surface = NULL;
+	if (_useInternalDstBuffer)
+	{
+		free(_vfDstSurface.Surface);
+		_vfDstSurface.Surface = NULL;
+	}
+	
+	for (size_t i = 0; i < _vfAttributes.workingSurfaceCount; i++)
+	{
+		free(_vfDstSurface.workingSurface[i]);
+		_vfDstSurface.workingSurface[i] = NULL;
+	}
 	
 	ThreadLockUnlock(&_lockDst);
 	
@@ -131,6 +179,78 @@ VideoFilter::~VideoFilter()
 	ThreadCondDestroy(&_condRunning);
 }
 
+bool VideoFilter::AllocateDstBuffer(const size_t dstWidth, const size_t dstHeight, const size_t workingSurfaceCount)
+{
+	bool result = false;
+	unsigned char *newSurfaceBuffer = NULL;
+	
+	// Allocate the destination buffer if necessary.
+	if (_useInternalDstBuffer)
+	{
+		newSurfaceBuffer = (unsigned char *)calloc(dstWidth * dstHeight, sizeof(uint32_t));
+		if (newSurfaceBuffer == NULL)
+		{
+			return result;
+		}
+	}
+	
+	ThreadLockLock(&this->_lockDst);
+	
+	for (size_t i = 0; i < FILTER_MAX_WORKING_SURFACE_COUNT; i++)
+	{
+		unsigned char *oldWorkingSurface = this->_vfDstSurface.workingSurface[i];
+		this->_vfDstSurface.workingSurface[i] = (i < workingSurfaceCount) ? (unsigned char *)calloc(dstWidth * dstHeight, sizeof(uint32_t)) : NULL;
+		free(oldWorkingSurface);
+	}
+	
+	// Set up SSurface structure.
+	this->_vfDstSurface.Width = dstWidth;
+	this->_vfDstSurface.Height = dstHeight;
+	this->_vfDstSurface.Pitch = dstWidth * 2;
+	
+	if (_useInternalDstBuffer)
+	{
+		unsigned char *oldBuffer = this->_vfDstSurface.Surface;
+		this->_vfDstSurface.Surface = newSurfaceBuffer;
+		free(oldBuffer);
+	}
+	
+	// Update the surfaces on threads.
+	const size_t threadCount = this->_vfThread.size();
+	const unsigned int linesPerThread = (threadCount > 1) ? dstHeight/threadCount : dstHeight;
+	unsigned int remainingLines = dstHeight;
+	
+	for (size_t i = 0; i < threadCount; i++)
+	{
+		SSurface &threadDstSurface = this->_vfThread[i].param.dstSurface;
+		threadDstSurface = this->_vfDstSurface;
+		threadDstSurface.Height = (linesPerThread < remainingLines) ? linesPerThread : remainingLines;
+		remainingLines -= threadDstSurface.Height;
+		
+		// Add any remaining lines to the last thread.
+		if (i == threadCount-1)
+		{
+			threadDstSurface.Height += remainingLines;
+		}
+		
+		if (i > 0)
+		{
+			SSurface &prevThreadDstSurface = this->_vfThread[i - 1].param.dstSurface;
+			threadDstSurface.Surface = (unsigned char *)((uint32_t *)prevThreadDstSurface.Surface + (prevThreadDstSurface.Width * prevThreadDstSurface.Height));
+			
+			for (size_t j = 0; j < workingSurfaceCount; j++)
+			{
+				threadDstSurface.workingSurface[j] = (unsigned char *)((uint32_t *)prevThreadDstSurface.workingSurface[j] + (prevThreadDstSurface.Width * prevThreadDstSurface.Height));
+			}
+		}
+	}
+	
+	ThreadLockUnlock(&this->_lockDst);
+	
+	result = true;
+	return result;
+}
+
 /********************************************************************************************
 	SetSourceSize()
 
@@ -149,6 +269,7 @@ VideoFilter::~VideoFilter()
 bool VideoFilter::SetSourceSize(const size_t width, const size_t height)
 {
 	bool result = false;
+	bool sizeChanged = false;
 	
 	ThreadLockLock(&this->_lockSrc);
 	
@@ -160,6 +281,11 @@ bool VideoFilter::SetSourceSize(const size_t width, const size_t height)
 		return result;
 	}
 	
+	if (this->_vfSrcSurface.Surface == NULL || this->_vfSrcSurface.Width != width || this->_vfSrcSurface.Height != height)
+	{
+		sizeChanged = true;
+	}
+	
 	this->_vfSrcSurface.Width = width;
 	this->_vfSrcSurface.Height = height;
 	this->_vfSrcSurface.Pitch = width * 2;
@@ -172,12 +298,21 @@ bool VideoFilter::SetSourceSize(const size_t width, const size_t height)
 	
 	// Update the surfaces on threads.
 	size_t threadCount = this->_vfThread.size();
+	const unsigned int linesPerThread = (threadCount > 1) ? this->_vfSrcSurface.Height/threadCount : this->_vfSrcSurface.Height;
+	unsigned int remainingLines = this->_vfSrcSurface.Height;
 	
 	for (size_t i = 0; i < threadCount; i++)
 	{
 		SSurface &threadSrcSurface = this->_vfThread[i].param.srcSurface;
 		threadSrcSurface = this->_vfSrcSurface;
-		threadSrcSurface.Height /= threadCount;
+		threadSrcSurface.Height = (linesPerThread < remainingLines) ? linesPerThread : remainingLines;
+		remainingLines -= threadSrcSurface.Height;
+		
+		// Add any remaining lines to the last thread.
+		if (i == threadCount-1)
+		{
+			threadSrcSurface.Height += remainingLines;
+		}
 		
 		if (i > 0)
 		{
@@ -188,9 +323,20 @@ bool VideoFilter::SetSourceSize(const size_t width, const size_t height)
 	
 	ThreadLockUnlock(&this->_lockSrc);
 	
-	const VideoFilterAttributes vfAttr = this->GetAttributes();
-	result = this->ChangeFilterByAttributes(&vfAttr);
+	if (sizeChanged)
+	{
+		const VideoFilterAttributes vfAttr = this->GetAttributes();
+		const size_t dstWidth = width * vfAttr.scaleMultiply / vfAttr.scaleDivide;
+		const size_t dstHeight = height * vfAttr.scaleMultiply / vfAttr.scaleDivide;
+		
+		result = this->AllocateDstBuffer(dstWidth, dstHeight, vfAttr.workingSurfaceCount);
+		if (!result)
+		{
+			return result;
+		}
+	}
 	
+	result = true;
 	return result;
 }
 
@@ -216,7 +362,7 @@ bool VideoFilter::ChangeFilterByID(const VideoFilterTypeID typeID)
 		return result;
 	}
 	
-	result = this->ChangeFilterByAttributes(&VideoFilterAttributesList[typeID]);
+	result = this->ChangeFilterByAttributes(VideoFilterAttributesList[typeID]);
 	
 	return result;
 }
@@ -233,70 +379,76 @@ bool VideoFilter::ChangeFilterByID(const VideoFilterTypeID typeID)
 		A bool that reports if the filter change was successful. A value of true means
 		success, while a value of false means failure.
  ********************************************************************************************/
-bool VideoFilter::ChangeFilterByAttributes(const VideoFilterAttributes *vfAttr)
+bool VideoFilter::ChangeFilterByAttributes(const VideoFilterAttributes &vfAttr)
 {
 	bool result = false;
 	
-	if (vfAttr == NULL)
+	if (vfAttr.scaleMultiply == 0 || vfAttr.scaleDivide < 1)
 	{
 		return result;
 	}
 	
-	if (vfAttr->scaleMultiply == 0 || vfAttr->scaleDivide < 1)
-	{
-		return result;
-	}
-	
-	this->SetAttributes(*vfAttr);
-	VideoFilterAttributes &workingAttributes = this->_vfAttributes;
-	
-	ThreadLockLock(&this->_lockSrc);
-	const size_t srcWidth = this->_vfSrcSurface.Width;
-	const size_t srcHeight = this->_vfSrcSurface.Height;
-	ThreadLockUnlock(&this->_lockSrc);
-	
-	const size_t dstWidth = srcWidth * workingAttributes.scaleMultiply / workingAttributes.scaleDivide;
-	const size_t dstHeight = srcHeight * workingAttributes.scaleMultiply / workingAttributes.scaleDivide;
-	const VideoFilterFunc filterFunction = workingAttributes.filterFunction;
-	
 	ThreadLockLock(&this->_lockDst);
-	
-	uint32_t *newSurfaceBuffer = (uint32_t *)calloc(dstWidth * dstHeight, sizeof(uint32_t));
-	if (newSurfaceBuffer == NULL)
-	{
-		return result;
-	}
-	
-	this->_vfFunc = filterFunction;
-	this->_vfDstSurface.Width = dstWidth;
-	this->_vfDstSurface.Height = dstHeight;
-	this->_vfDstSurface.Pitch = dstWidth * 2;
-	
-	free(this->_vfDstSurface.Surface);
-	this->_vfDstSurface.Surface = (unsigned char *)newSurfaceBuffer;
-	
-	// Update the surfaces on threads.
-	const size_t threadCount = this->_vfThread.size();
-	
-	for (size_t i = 0; i < threadCount; i++)
-	{
-		SSurface &threadDstSurface = this->_vfThread[i].param.dstSurface;
-		threadDstSurface = this->_vfDstSurface;
-		threadDstSurface.Height /= threadCount;
-		
-		if (i > 0)
-		{
-			SSurface &prevThreadDstSurface = this->_vfThread[i - 1].param.dstSurface;
-			threadDstSurface.Surface = (unsigned char *)((uint32_t *)prevThreadDstSurface.Surface + (prevThreadDstSurface.Width * prevThreadDstSurface.Height));
-		}
-		
-		this->_vfThread[i].param.filterFunction = this->_vfFunc;
-	}
-	
+	unsigned char *dstSurface = this->_vfDstSurface.Surface;
 	ThreadLockUnlock(&this->_lockDst);
 	
-	result = true;
+	const VideoFilterAttributes currentAttr = this->GetAttributes();
+	const size_t threadCount = this->_vfThread.size();
 	
+	if (dstSurface != NULL &&
+		currentAttr.scaleMultiply == vfAttr.scaleMultiply &&
+		currentAttr.scaleDivide == vfAttr.scaleDivide &&
+		currentAttr.workingSurfaceCount == vfAttr.workingSurfaceCount)
+	{
+		// If we have existing buffers and the new size is identical to the old size, we
+		// can skip the costly construction of the buffers and simply clear them instead.
+		
+		ThreadLockLock(&this->_lockDst);
+		
+		const size_t bufferSizeBytes = this->_vfDstSurface.Width * this->_vfDstSurface.Height * sizeof(uint32_t);
+		
+		memset(this->_vfDstSurface.Surface, 0, bufferSizeBytes);
+		for (size_t i = 0; i < currentAttr.workingSurfaceCount; i++)
+		{
+			memset(this->_vfDstSurface.workingSurface[i], 0, bufferSizeBytes);
+		}
+		
+		this->_vfFunc = vfAttr.filterFunction;
+		for (size_t i = 0; i < threadCount; i++)
+		{
+			this->_vfThread[i].param.filterFunction = this->_vfFunc;
+		}
+		
+		ThreadLockUnlock(&this->_lockDst);
+	}
+	else
+	{
+		// Construct a new destination buffer per filter attributes.
+		ThreadLockLock(&this->_lockSrc);
+		const size_t dstWidth = this->_vfSrcSurface.Width * vfAttr.scaleMultiply / vfAttr.scaleDivide;
+		const size_t dstHeight = this->_vfSrcSurface.Height * vfAttr.scaleMultiply / vfAttr.scaleDivide;
+		ThreadLockUnlock(&this->_lockSrc);
+		
+		ThreadLockLock(&this->_lockDst);
+		
+		this->_vfFunc = vfAttr.filterFunction;
+		for (size_t i = 0; i < threadCount; i++)
+		{
+			this->_vfThread[i].param.filterFunction = this->_vfFunc;
+		}
+		
+		ThreadLockUnlock(&this->_lockDst);
+		
+		result = this->AllocateDstBuffer(dstWidth, dstHeight, vfAttr.workingSurfaceCount);
+		if (!result)
+		{
+			return result;
+		}
+	}
+	
+	this->SetAttributes(vfAttr);
+	
+	result = true;
 	return result;
 }
 
@@ -390,7 +542,7 @@ void VideoFilter::RunFilterCustomByID(const uint32_t *__restrict srcBuffer, uint
 		return;
 	}
 	
-	VideoFilter::RunFilterCustomByAttributes(srcBuffer, dstBuffer, srcWidth, srcHeight, &VideoFilterAttributesList[typeID]);
+	VideoFilter::RunFilterCustomByAttributes(srcBuffer, dstBuffer, srcWidth, srcHeight, VideoFilterAttributesList[typeID]);
 }
 
 /********************************************************************************************
@@ -422,24 +574,23 @@ void VideoFilter::RunFilterCustomByID(const uint32_t *__restrict srcBuffer, uint
  ********************************************************************************************/
 void VideoFilter::RunFilterCustomByAttributes(const uint32_t *__restrict srcBuffer, uint32_t *__restrict dstBuffer,
 											  const size_t srcWidth, const size_t srcHeight,
-											  const VideoFilterAttributes *vfAttr)
+											  const VideoFilterAttributes &vfAttr)
 {
 	// Parameter check
 	if (srcBuffer == NULL ||
 		dstBuffer == NULL ||
 		srcWidth < 1 ||
 		srcHeight < 1 ||
-		vfAttr == NULL ||
-		vfAttr->scaleMultiply == 0 ||
-		vfAttr->scaleDivide < 1)
+		vfAttr.scaleMultiply == 0 ||
+		vfAttr.scaleDivide < 1)
 	{
 		return;
 	}
 	
 	// Get the filter attributes
-	const size_t dstWidth = srcWidth * vfAttr->scaleMultiply / vfAttr->scaleDivide;
-	const size_t dstHeight = srcHeight * vfAttr->scaleMultiply / vfAttr->scaleDivide;
-	const VideoFilterFunc filterFunction = vfAttr->filterFunction;
+	const size_t dstWidth = srcWidth * vfAttr.scaleMultiply / vfAttr.scaleDivide;
+	const size_t dstHeight = srcHeight * vfAttr.scaleMultiply / vfAttr.scaleDivide;
+	const VideoFilterFunc filterFunction = vfAttr.filterFunction;
 	
 	// Assign the surfaces and run the filter
 	SSurface srcSurface;
@@ -460,10 +611,43 @@ void VideoFilter::RunFilterCustomByAttributes(const uint32_t *__restrict srcBuff
 	}
 	else
 	{
+		for (size_t i = 0; i < vfAttr.workingSurfaceCount; i++)
+		{
+			dstSurface.workingSurface[i] = (unsigned char *)calloc(dstWidth * dstHeight, sizeof(uint32_t));
+		}
+		
 		filterFunction(srcSurface, dstSurface);
+		
+		for (size_t i = 0; i < vfAttr.workingSurfaceCount; i++)
+		{
+			free(dstSurface.workingSurface[i]);
+		}
 	}
 }
 
+/********************************************************************************************
+	GetAttributesByID() - STATIC
+
+	Returns the filter attributes associated with the given type ID.
+
+	Takes:
+		typeID - The type ID of the video filter. See the VideoFilterTypeID
+			enumeration for possible values.
+
+	Returns:
+		A copy of the filter attributes of the given type ID. If typeID is
+		invalid, this method returns the attributes of VideoFilterTypeID_None.
+ ********************************************************************************************/
+VideoFilterAttributes VideoFilter::GetAttributesByID(const VideoFilterTypeID typeID)
+{
+	if (typeID >= VideoFilterTypeIDCount)
+	{
+		return VideoFilterAttributesList[VideoFilterTypeID_None];
+	}
+	
+	return VideoFilterAttributesList[typeID];
+}
+
 /********************************************************************************************
 	GetTypeStringByID() - STATIC
 
@@ -499,7 +683,7 @@ VideoFilterAttributes VideoFilter::GetAttributes()
 	return vfAttr;
 }
 
-void VideoFilter::SetAttributes(const VideoFilterAttributes vfAttr)
+void VideoFilter::SetAttributes(const VideoFilterAttributes &vfAttr)
 {
 	ThreadLockLock(&this->_lockAttributes);
 	this->_vfAttributes = vfAttr;
@@ -542,6 +726,52 @@ uint32_t* VideoFilter::GetDstBufferPtr()
 	return ptr;
 }
 
+/********************************************************************************************
+	SetDstBufferPtr()
+
+	Returns a C-string representation of the passed in video filter type.
+
+	Takes:
+		theBuffer - A pointer to the destination pixel buffer. If a value of NULL is
+			passed in, then VideoFilter will use its own internal buffer (this is the
+			default behavior). Otherwise, VideoFilter will use the passed in pointer
+			for the destination buffer. The caller is responsible for ensuring that
+			this buffer is valid and large enough to store all of the destination
+			pixels under all circumstances. Methods that may change the destination
+			buffer's size requirements are:
+				SetSourceSize()
+				ChangeFilterByID()
+				ChangeFilterByAttributes()
+
+	Returns:
+		Nothing.
+ ********************************************************************************************/
+void VideoFilter::SetDstBufferPtr(uint32_t *theBuffer)
+{
+	ThreadLockLock(&this->_lockDst);
+	
+	if (theBuffer == NULL)
+	{
+		this->_useInternalDstBuffer = true;
+	}
+	else
+	{
+		unsigned char *oldDstBuffer = this->_vfDstSurface.Surface;
+		this->_vfDstSurface.Surface = (unsigned char *)theBuffer;
+		
+		if (this->_useInternalDstBuffer)
+		{
+			free(oldDstBuffer);
+		}
+		
+		this->_useInternalDstBuffer = false;
+	}
+	
+	ThreadLockUnlock(&this->_lockDst);
+	
+	this->AllocateDstBuffer(this->_vfDstSurface.Width, this->_vfDstSurface.Height, this->_vfAttributes.workingSurfaceCount);
+}
+
 size_t VideoFilter::GetSrcWidth()
 {
 	ThreadLockLock(&this->_lockSrc);
@@ -578,7 +808,7 @@ size_t VideoFilter::GetDstHeight()
 	return height;
 }
 
-VideoFilterParamType VideoFilter::GetFilterParameterType(VideoFilterParamID paramID)
+VideoFilterParamType VideoFilter::GetFilterParameterType(VideoFilterParamID paramID) const
 {
 	return _VideoFilterParamAttributesList[paramID].type;
 }
