@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2011-2012 Roger Manuel
-	Copyright (C) 2013 DeSmuME team
+	Copyright (C) 2013-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -19,8 +19,17 @@
 #ifndef _VIDEOFILTER_
 #define _VIDEOFILTER_
 
+#include <stdlib.h>
+#include <string>
+#include <vector>
+
+#include "types.h"
+#include "filter.h"
+#include "../utils/task.h"
+
 #ifdef HOST_WINDOWS
 	typedef unsigned __int32 uint32_t;
+	#define WIN32_LEAN_AND_MEAN
 	#include <windows.h>
 	typedef CRITICAL_SECTION ThreadLock;
 	typedef HANDLE ThreadCond;
@@ -31,13 +40,6 @@
 	typedef pthread_cond_t ThreadCond;
 #endif
 
-#include <stdlib.h>
-#include <string>
-#include <vector>
-#include "filter.h"
-#include "../utils/task.h"
-
-
 #define VIDEOFILTERTYPE_UNKNOWN_STRING "Unknown"
 
 typedef void (*VideoFilterFunc)(SSurface Src, SSurface Dst);
@@ -64,6 +66,10 @@ enum VideoFilterTypeID
 	VideoFilterTypeID_EPX1_5X,
 	VideoFilterTypeID_EPXPlus1_5X,
 	VideoFilterTypeID_HQ4XS,
+	VideoFilterTypeID_2xBRZ,
+	VideoFilterTypeID_3xBRZ,
+	VideoFilterTypeID_4xBRZ,
+	VideoFilterTypeID_5xBRZ,
 	
 	VideoFilterTypeIDCount // Make sure this one is always last
 };
@@ -75,29 +81,13 @@ typedef struct
 	VideoFilterFunc filterFunction;
 	size_t scaleMultiply;
 	size_t scaleDivide;
+	size_t workingSurfaceCount;
 } VideoFilterAttributes;
 
-// Attributes list of known video filters, indexed using VideoFilterTypeID.
-const VideoFilterAttributes VideoFilterAttributesList[] = {
-	{VideoFilterTypeID_None,			"None",				NULL,							1,	1},
-	{VideoFilterTypeID_LQ2X,			"LQ2x",				&RenderLQ2X,					2,	1},
-	{VideoFilterTypeID_LQ2XS,			"LQ2xS",			&RenderLQ2XS,					2,	1},
-	{VideoFilterTypeID_HQ2X,			"HQ2x",				&RenderHQ2X,					2,	1},
-	{VideoFilterTypeID_HQ2XS,			"HQ2xS",			&RenderHQ2XS,					2,	1},
-	{VideoFilterTypeID_HQ4X,			"HQ4x",				&RenderHQ4X,					4,	1},
-	{VideoFilterTypeID_2xSaI,			"2xSaI",			&Render2xSaI,					2,	1},
-	{VideoFilterTypeID_Super2xSaI,		"Super 2xSaI",		&RenderSuper2xSaI,				2,	1},
-	{VideoFilterTypeID_SuperEagle,		"Super Eagle",		&RenderSuperEagle,				2,	1},
-	{VideoFilterTypeID_Scanline,		"Scanline",			&RenderScanline,				2,	1},
-	{VideoFilterTypeID_Bilinear,		"Bilinear",			&RenderBilinear,				2,	1},
-	{VideoFilterTypeID_Nearest2X,		"Nearest 2x",		&RenderNearest2X,				2,	1},
-	{VideoFilterTypeID_Nearest1_5X,		"Nearest 1.5x",		&RenderNearest_1Point5x,		3,	2},
-	{VideoFilterTypeID_NearestPlus1_5X,	"Nearest+ 1.5x",	&RenderNearestPlus_1Point5x,	3,	2},
-	{VideoFilterTypeID_EPX,				"EPX",				&RenderEPX,						2,	1},
-	{VideoFilterTypeID_EPXPlus,			"EPX+",				&RenderEPXPlus,					2,	1},
-	{VideoFilterTypeID_EPX1_5X,			"EPX 1.5x",			&RenderEPX_1Point5x,			3,	2},
-	{VideoFilterTypeID_EPXPlus1_5X,		"EPX+ 1.5x",		&RenderEPXPlus_1Point5x,		3,	2},
-	{VideoFilterTypeID_HQ4XS,			"HQ4xS",			&RenderHQ4XS,					4,	1} };
+// Users should really be using VideoFilter::GetAttributesByID() to retrieve
+// a filter's attributes. However, extern this out just in case someone
+// wants to statically assign some attributes info at compile time.
+extern const VideoFilterAttributes VideoFilterAttributesList[];
 
 // VIDEO FILTER PARAMETER DATA TYPES
 enum VideoFilterParamType
@@ -165,6 +155,7 @@ private:
 	uint32_t *_vfSrcSurfacePixBuffer;
 	VideoFilterFunc _vfFunc;
 	std::vector<VideoFilterThread> _vfThread;
+	bool _useInternalDstBuffer;
 	
 	bool _isFilterRunning;
 	ThreadLock _lockSrc;
@@ -172,7 +163,8 @@ private:
 	ThreadLock _lockAttributes;
 	ThreadCond _condRunning;
 	
-	void SetAttributes(const VideoFilterAttributes vfAttr);
+	bool AllocateDstBuffer(const size_t dstWidth, const size_t dstHeight, const size_t workingSurfaceCount);
+	void SetAttributes(const VideoFilterAttributes &vfAttr);
 	
 public:
 	VideoFilter(size_t srcWidth, size_t srcHeight, VideoFilterTypeID typeID, size_t threadCount);
@@ -180,11 +172,12 @@ public:
 	
 	bool SetSourceSize(const size_t width, const size_t height);
 	bool ChangeFilterByID(const VideoFilterTypeID typeID);
-	bool ChangeFilterByAttributes(const VideoFilterAttributes *vfAttr);
+	bool ChangeFilterByAttributes(const VideoFilterAttributes &vfAttr);
 	uint32_t* RunFilter();
 	
 	static void RunFilterCustomByID(const uint32_t *__restrict srcBuffer, uint32_t *__restrict dstBuffer, const size_t srcWidth, const size_t srcHeight, const VideoFilterTypeID typeID);
-	static void RunFilterCustomByAttributes(const uint32_t *__restrict srcBuffer, uint32_t *__restrict dstBuffer, const size_t srcWidth, const size_t srcHeight, const VideoFilterAttributes *vfAttr);
+	static void RunFilterCustomByAttributes(const uint32_t *__restrict srcBuffer, uint32_t *__restrict dstBuffer, const size_t srcWidth, const size_t srcHeight, const VideoFilterAttributes &vfAttr);
+	static VideoFilterAttributes GetAttributesByID(const VideoFilterTypeID typeID);
 	static const char* GetTypeStringByID(const VideoFilterTypeID typeID);
 	
 	VideoFilterAttributes GetAttributes();
@@ -192,11 +185,12 @@ public:
 	const char* GetTypeString();
 	uint32_t* GetSrcBufferPtr();
 	uint32_t* GetDstBufferPtr();
+	void SetDstBufferPtr(uint32_t *theBuffer);
 	size_t GetSrcWidth();
 	size_t GetSrcHeight();
 	size_t GetDstWidth();
 	size_t GetDstHeight();
-	VideoFilterParamType GetFilterParameterType(VideoFilterParamID paramID);
+	VideoFilterParamType GetFilterParameterType(VideoFilterParamID paramID) const;
 	int GetFilterParameteri(VideoFilterParamID paramID);
 	unsigned int GetFilterParameterui(VideoFilterParamID paramID);
 	float GetFilterParameterf(VideoFilterParamID paramID);
@@ -205,8 +199,6 @@ public:
 	void SetFilterParameterf(VideoFilterParamID paramID, float value);
 };
 
-static void* RunVideoFilterTask(void *arg);
-
 void ThreadLockInit(ThreadLock *theLock);
 void ThreadLockDestroy(ThreadLock *theLock);
 void ThreadLockLock(ThreadLock *theLock);
