@@ -1,6 +1,6 @@
 /*	
 	Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2013 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -28,13 +28,18 @@
 //if they do, then we need to copy them out in doFlush!!!
 //---------------
 
-#include <algorithm>
+#include "gfx3d.h"
+
 #include <assert.h>
 #include <math.h>
 #include <string.h>
+#include <algorithm>
+#include <queue>
+
 #include "armcpu.h"
 #include "debug.h"
-#include "gfx3d.h"
+#include "driver.h"
+#include "emufile.h"
 #include "matrix.h"
 #include "bits.h"
 #include "MMU.h"
@@ -46,7 +51,6 @@
 #include "readwrite.h"
 #include "FIFO.h"
 #include "movie.h" //only for currframecounter which really ought to be moved into the core emu....
-#include <queue>
 
 //#define _SHOW_VTX_COUNTERS	// show polygon/vertex counters on screen
 #ifdef _SHOW_VTX_COUNTERS
@@ -98,135 +102,135 @@ static const u8 gfx3d_commandTypes[] = {
 class GXF_Hardware
 {
 public:
+
 	GXF_Hardware()
 	{
 		reset();
 	}
-	void reset() {
-		countdown = 0;
-		commandCursor = 4;
-		for(int i=0;i<4;i++) {
-			commandsPending[i].command = 0;
-			commandsPending[i].countdown = 0;
-		}
-		size = 0;
+
+	void reset()
+	{
+		shiftCommand = 0;
+		paramCounter = 0;
 	}
 
 	void receive(u32 val) 
 	{
-		if (size > 0)
-		{
-			GFX_FIFOsend(front().command, val);
-			front().countdown--;
-			if (front().countdown == 0) 
-			{
-				size--; 
-				if (size == 0) return;
-				dequeue();
+		//so, it seems as if the dummy values and restrictions on the highest-order command in the packed command set 
+		//is solely about some unknown internal timing quirk, and not about the logical behaviour of the state machine.
+		//it's possible that writing some values too quickly can result in the gxfifo not being ready. 
+		//this would be especially troublesome when they're getting DMA'd nonstop with no delay.
+		//so, since the timing is not emulated rigorously here, and only the logic, we shouldn't depend on or expect the dummy values.
+		//indeed, some games aren't issuing them, and will break if they are expected.
+		//however, since the dummy values seem to be 0x00000000 always, theyre benign to the state machine, even if the HW timing doesnt require them.
+		//the actual logical rule seem to be:
+		//1. commands with no arguments are executed as soon as possible; when the packed command is first received, or immediately after the preceding N-arg command.
+		//1a. for example, a 0-arg command following an N-arg command doesn't require a dummy value
+		//1b. for example, two 0-arg commands constituting a packed command will execute immediately when the packed command is received.
+		//
+		//as an example, DQ6 entity rendering will issue:
+		//0x00151110
+		//0x00000000
+		//0x00171012 (next packed command)
+		//this 0 param is meant for the 0x10 mtxMode command. dummy args aren't supplied for the 0x11 or 0x15.
+		//but other times, the game will issue dummy parameters.
+		//Either this is because the rules are more complex (do only certain 0-arg commands require dummies?),
+		//or the game made a mistake in applying the rules, which didnt seem to irritate the hypothetical HW timing quirk.
+		//yet, another time, it will issue the dummy:
+		//0x00004123 (XYZ vertex)
+		//0x0c000400 (arg1)
+		//0x00000000 (arg2)
+		//0x00000000 (dummy for 0x41)
+		//0x00000017 (next packed command)
 
-				while (gfx3d_commandTypes[front().command] == GFX_NOARG_COMMAND)
-				{
-					GFX_FIFOsend(front().command, 0);
-					size--; 
-					if (size == 0) break;
-					dequeue();
-				}
-			}
+		u8 currCommand = shiftCommand & 0xFF;
+		u8 currCommandType = gfx3d_commandTypes[currCommand];
+
+		//if the current command is invalid, receive a new packed command.
+		if(currCommandType == GFX_INVALID_COMMAND)
+		{
+			shiftCommand = val;
 		}
-		else
+
+		//finish receiving args
+		if(paramCounter>0)
 		{
-			if (val == 0) return;	// nop
+			GFX_FIFOsend(currCommand, val);
+			paramCounter--;
+			if(paramCounter <= 0)
+				shiftCommand >>= 8;
+			else return;
+		}
 
-			const u8 commands[] = { val&0xFF, (val>>8)&0xFF, (val>>16)&0xFF, (val>>24)&0xFF };
-			const u8 commandTypes[] = { gfx3d_commandTypes[commands[0]], gfx3d_commandTypes[commands[1]], gfx3d_commandTypes[commands[2]], gfx3d_commandTypes[commands[3]] };
+		//analyze current packed commands
+		for(;;)
+		{
+			currCommand = shiftCommand & 0xFF;
+			currCommandType = gfx3d_commandTypes[currCommand];
 
-			commandCursor = 0;
-			size = 0;
-
-			// A "command without parameters" is one of the four following commands:
-			// - PushMatrix
-			// - LoadIdentity
-			// - End
-			// - Commands undefined within the region between 0x10 and 0xFF
-
-			for (u8 i = 0; i < 4; i++)
+			if(currCommandType == GFX_UNDEFINED_COMMAND)
+				shiftCommand >>= 8;
+			else if(currCommandType == GFX_NOARG_COMMAND)
 			{
-				if (commandTypes[i] == GFX_INVALID_COMMAND)
-				{
-					//printf("gfx3D: invalid command (%02X)\n", commands[i]);
-					continue;
-				}
-
-				if (commandTypes[i] == GFX_UNDEFINED_COMMAND)
-				{
-					//printf("gfx3D: undefined command (%02X)\n", commands[i]);
-					continue;
-				}
-				commandsPending[size].command = commands[i];
-				commandsPending[size].countdown = commandTypes[i];
-				//printf("%i: CMD %02X size %i\n", i, commands[i], commandsPending[size].countdown);
-				if ((commandsPending[size].countdown == 0) && (size == 0)) // cmd 0x11, 0x15, 0x41 - no params
-				{
-					GFX_FIFOsend(commands[i], 0);
-
-					while ((i < 4) && commands[i+1] != 0 && gfx3d_commandTypes[commands[i+1]] == GFX_NOARG_COMMAND)
-						GFX_FIFOsend(commands[++i], 0);
-				}
-				else
-					size++;
+				GFX_FIFOsend(currCommand, 0);
+				shiftCommand >>= 8;
+			}
+			else if(currCommandType == GFX_INVALID_COMMAND)
+				break;
+			else 
+			{
+				paramCounter = currCommandType;
+				break;
 			}
 		}
 	}
 
-	struct CommandItem {
-		u8 command, countdown;
-	} commandsPending[4];
-
-	u32 commandCursor;
-	u8 countdown;
-
 private:
-	void dequeue() { commandCursor++; }
-	CommandItem& front() { return commandsPending[commandCursor]; }
-	u32 size;
+
+	u32 shiftCommand;
+	u32 paramCounter;
+
 public:
 
 	void savestate(EMUFILE *f)
 	{
-		//TODO - next time we invalidate savestates, simplify this format.
-		write32le(1,f); //version
-		write32le(size,f);
-		write32le(commandCursor,f);
-		for(u32 i=0;i<4;i++) write8le(commandsPending[i].command,f);
-		for(u32 i=0;i<4;i++) write8le(commandsPending[i].countdown,f);
-		write8le(countdown,f);
+		write32le(2,f); //version
+		write32le(shiftCommand,f);
+		write32le(paramCounter,f);
 	}
 	
 	bool loadstate(EMUFILE *f)
 	{
 		u32 version;
 		if(read32le(&version,f) != 1) return false;
-		if(version > 1) return false;
+
+		u8 junk8;
+		u32 junk32;
 
 		if (version == 0)
 		{
-			read32le(&size,f);
-			commandCursor = 4-size;
-			for(u32 i=commandCursor;i<4;i++) read8le(&commandsPending[i-commandCursor].command,f);
-			read32le(&size,f);
-			size = 4-commandCursor;
-			for(u32 i=commandCursor;i<4;i++) read8le(&commandsPending[i-commandCursor].countdown,f);
+			//untested
+			read32le(&junk32,f);
+			int commandCursor = 4-junk32;
+			for(u32 i=commandCursor;i<4;i++) read8le(&junk8,f);
+			read32le(&junk32,f);
+			for(u32 i=commandCursor;i<4;i++) read8le(&junk8,f);
+			read8le(&junk8,f);
 		}
-		else
-		if (version == 1)
+		else if (version == 1)
 		{
-			read32le(&size,f);
-			read32le(&commandCursor,f);
-			for(u32 i=0;i<4;i++) read8le(&commandsPending[i].command,f);
-			for(u32 i=0;i<4;i++) read8le(&commandsPending[i].countdown,f);
+			//untested
+			read32le(&junk32,f);
+			read32le(&junk32,f);
+			for(u32 i=0;i<4;i++) read8le(&junk8,f);
+			for(u32 i=0;i<4;i++) read8le(&junk8,f);
+			read8le(&junk8,f);
+		}
+		else if(version == 2)
+		{
+			read32le(&shiftCommand,f);
+			read32le(&paramCounter,f);
 		}
-
-		read8le(&countdown,f);
 
 		return true;
 	}
@@ -304,7 +308,7 @@ static float normalTable[1024];
 #define fix2float(v)    (((float)((s32)(v))) / (float)(1<<12))
 #define fix10_2float(v) (((float)((s32)(v))) / (float)(1<<9))
 
-CACHE_ALIGN u8 gfx3d_convertedScreen[256*192*4];
+CACHE_ALIGN u8 gfx3d_convertedScreen[GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT*4];
 
 // Matrix stack handling
 CACHE_ALIGN MatrixStack	mtxStack[4] = {
@@ -459,6 +463,44 @@ static void makeTables() {
 			}
 }
 
+#define OSWRITE(x) os->fwrite((char*)&(x),sizeof((x)));
+#define OSREAD(x) is->fread((char*)&(x),sizeof((x)));
+
+void POLY::save(EMUFILE* os)
+{
+	OSWRITE(type);
+	OSWRITE(vertIndexes[0]); OSWRITE(vertIndexes[1]); OSWRITE(vertIndexes[2]); OSWRITE(vertIndexes[3]);
+	OSWRITE(polyAttr); OSWRITE(texParam); OSWRITE(texPalette);
+	OSWRITE(viewport);
+	OSWRITE(miny);
+	OSWRITE(maxy);
+}
+
+void POLY::load(EMUFILE* is)
+{
+	OSREAD(type);
+	OSREAD(vertIndexes[0]); OSREAD(vertIndexes[1]); OSREAD(vertIndexes[2]); OSREAD(vertIndexes[3]);
+	OSREAD(polyAttr); OSREAD(texParam); OSREAD(texPalette);
+	OSREAD(viewport);
+	OSREAD(miny);
+	OSREAD(maxy);
+}
+
+void VERT::save(EMUFILE* os)
+{
+	OSWRITE(x); OSWRITE(y); OSWRITE(z); OSWRITE(w);
+	OSWRITE(u); OSWRITE(v);
+	OSWRITE(color[0]); OSWRITE(color[1]); OSWRITE(color[2]);
+	OSWRITE(fcolor[0]); OSWRITE(fcolor[1]); OSWRITE(fcolor[2]);
+}
+void VERT::load(EMUFILE* is)
+{
+	OSREAD(x); OSREAD(y); OSREAD(z); OSREAD(w);
+	OSREAD(u); OSREAD(v);
+	OSREAD(color[0]); OSREAD(color[1]); OSREAD(color[2]);
+	OSREAD(fcolor[0]); OSREAD(fcolor[1]); OSREAD(fcolor[2]);
+}
+
 void gfx3d_init()
 {
 	gxf_hardware.reset();
@@ -479,8 +521,22 @@ void gfx3d_init()
 
 	//printf("SPEED TEST %d %d\n",diff,diff2);
 
-	if(polylists == NULL) { polylists = new POLYLIST[2]; polylist = &polylists[0]; }
-	if(vertlists == NULL) { vertlists = new VERTLIST[2]; vertlist = &vertlists[0]; }
+	// Use malloc() instead of new because, for some unknown reason, GCC 4.9 has a bug
+	// that causes a std::bad_alloc exception on certain memory allocations. Right now,
+	// POLYLIST and VERTLIST are POD-style structs, so malloc() can substitute for new
+	// in this case.
+	if(polylists == NULL)
+	{
+		polylists = (POLYLIST *)malloc(sizeof(POLYLIST)*2);
+		polylist = &polylists[0];
+	}
+	
+	if(vertlists == NULL)
+	{
+		vertlists = (VERTLIST *)malloc(sizeof(VERTLIST)*2);
+		vertlist = &vertlists[0];
+	}
+	
 	makeTables();
 	gfx3d_reset();
 }
@@ -2034,7 +2090,20 @@ void gfx3d_glFlush(u32 v)
 	GFX_DELAY(1);
 }
 
-static bool gfx3d_ysort_compare(int num1, int num2)
+static inline bool gfx3d_ysort_compare_orig(int num1, int num2)
+{
+	const POLY &poly1 = polylist->list[num1];
+	const POLY &poly2 = polylist->list[num2];
+
+	if(poly1.maxy != poly2.maxy) 
+		return poly1.maxy < poly2.maxy; 
+	if(poly1.miny != poly2.miny) 
+		return poly1.miny < poly2.miny; 
+
+	return num1 < num2;
+}
+
+static inline bool gfx3d_ysort_compare_kalven(int num1, int num2)
 {
 	const POLY &poly1 = polylist->list[num1];
 	const POLY &poly2 = polylist->list[num2];
@@ -2053,8 +2122,15 @@ static bool gfx3d_ysort_compare(int num1, int num2)
 	//make sure we respect the game's ordering in cases of complete ties
 	//this makes it a stable sort.
 	//this must be a stable sort or else advance wars DOR will flicker in the main map mode
-	if (num1 < num2) return true;
-	else return false;
+	return (num1 < num2);
+}
+
+static bool gfx3d_ysort_compare(int num1, int num2)
+{
+	bool original = gfx3d_ysort_compare_orig(num1,num2);
+	bool kalven = gfx3d_ysort_compare_kalven(num1,num2);
+	assert(original == kalven);
+	return original;
 }
 
 static void gfx3d_doFlush()
@@ -2109,16 +2185,20 @@ static void gfx3d_doFlush()
 	//also the buttons in the knights in the nightmare frontend depend on this
 	for(int i=0; i<polycount; i++)
 	{
+		// TODO: Possible divide by zero with the w-coordinate.
+		// Is the vertex being read correctly? Is 0 a valid value for w?
+		// If both of these questions answer to yes, then how does the NDS handle a NaN?
+		// For now, simply prevent w from being zero.
 		POLY &poly = polylist->list[i];
 		float verty = vertlist->list[poly.vertIndexes[0]].y;
-		float vertw = vertlist->list[poly.vertIndexes[0]].w;
+		float vertw = (vertlist->list[poly.vertIndexes[0]].w != 0.0f) ? vertlist->list[poly.vertIndexes[0]].w : 0.00000001f;
 		verty = 1.0f-(verty+vertw)/(2*vertw);
 		poly.miny = poly.maxy = verty;
 
 		for(int j=1; j<poly.type; j++)
 		{
 			verty = vertlist->list[poly.vertIndexes[j]].y;
-			vertw = vertlist->list[poly.vertIndexes[j]].w;
+			vertw = (vertlist->list[poly.vertIndexes[j]].w != 0.0f) ? vertlist->list[poly.vertIndexes[j]].w : 0.00000001f;
 			verty = 1.0f-(verty+vertw)/(2*vertw);
 			poly.miny = min(poly.miny, verty);
 			poly.maxy = max(poly.maxy, verty);
@@ -2141,17 +2221,22 @@ static void gfx3d_doFlush()
 		if(poly.isTranslucent())
 			gfx3d.indexlist.list[ctr++] = i;
 	}
+	
+	//NOTE: the use of the stable_sort below must be here as a workaround for some compilers on osx and linux.
+	//we're hazy on the exact behaviour of the resulting bug, all thats known is the list gets mangled somehow.
+	//it should not in principle be relevant since the predicate results in no ties.
+	//perhaps the compiler is buggy. perhaps the predicate is wrong.
 
 	//now we have to sort the opaque polys by y-value.
 	//(test case: harvest moon island of happiness character cretor UI)
 	//should this be done after clipping??
-	std::sort(gfx3d.indexlist.list, gfx3d.indexlist.list + opaqueCount, gfx3d_ysort_compare);
+	std::stable_sort(gfx3d.indexlist.list, gfx3d.indexlist.list + opaqueCount, gfx3d_ysort_compare);
 	
 	if(!gfx3d.state.sortmode)
 	{
 		//if we are autosorting translucent polys, we need to do this also
 		//TODO - this is unverified behavior. need a test case
-		std::sort(gfx3d.indexlist.list + opaqueCount, gfx3d.indexlist.list + polycount, gfx3d_ysort_compare);
+		std::stable_sort(gfx3d.indexlist.list + opaqueCount, gfx3d.indexlist.list + polycount, gfx3d_ysort_compare);
 	}
 
 	//switch to the new lists
@@ -2313,12 +2398,12 @@ void gfx3d_GetLineData(int line, u8** dst)
 void gfx3d_GetLineData15bpp(int line, u16** dst)
 {
 	//TODO - this is not very thread safe!!!
-	static u16 buf[256];
+	static u16 buf[GFX3D_FRAMEBUFFER_WIDTH];
 	*dst = buf;
 
 	u8* lineData;
 	gfx3d_GetLineData(line, &lineData);
-	for(int i=0;i<256;i++)
+	for(int i=0; i<GFX3D_FRAMEBUFFER_WIDTH; i++)
 	{
 		const u8 r = lineData[i*4+0];
 		const u8 g = lineData[i*4+1];
@@ -2415,7 +2500,7 @@ SFORMAT SF_GFX3D[]={
 	{ "GTVC", 4, 1, &tempVertInfo.count},
 	{ "GTVM", 4, 4, tempVertInfo.map},
 	{ "GTVF", 4, 1, &tempVertInfo.first},
-	{ "G3CX", 1, 4*256*192, gfx3d_convertedScreen},
+	{ "G3CX", 1, 4*GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT, gfx3d_convertedScreen},
 	{ 0 }
 };
 
