@@ -24,30 +24,45 @@
 #include "../OGLRender.h"
 #include "osmesa_3Demu.h"
 
-static bool osmesa_beginOpenGL(void) {
-  return 1;
+static bool osmesa_beginOpenGL(void) { return 1; }
+static void osmesa_endOpenGL(void) { }
+static bool osmesa_init(void) { return true; }
+
+static void * buffer = NULL;
+static OSMesaContext ctx;
+
+void deinit_osmesa_3Demu (void)
+{
+    free(buffer);
+    OSMesaDestroyContext(ctx);
 }
 
-static void osmesa_endOpenGL(void) {
-}
+int init_osmesa_3Demu(void) 
+{
+    if (!ctx)
+    {
+        printf("OSMesaCreateContext failed!\n");
+        return false;
+    }
 
-static bool osmesa_init(void) {
+    buffer = malloc(256 * 192 * 4);
+    if (!buffer)
+    {
+        printf("Could not allocate enough memory!\n");
+        return false;
+    }
 
-  return true;
-}
+    if (!OSMesaMakeCurrent(ctx, buffer, GL_UNSIGNED_BYTE, 256, 192))
+    {
+        printf("OSMesaMakeCurrent failed!\n");
+        free(buffer);
+        return false;
+    }
 
-int init_osmesa_3Demu(void) {
-  void * buffer;
-  OSMesaContext ctx;
+    oglrender_init = osmesa_init;
+    oglrender_beginOpenGL = osmesa_beginOpenGL;
+    oglrender_endOpenGL = osmesa_endOpenGL;
 
-  ctx = OSMesaCreateContext(OSMESA_RGBA, NULL);
-  buffer = malloc(256 * 192 * 4);
-  OSMesaMakeCurrent(ctx, buffer, GL_UNSIGNED_BYTE, 256, 192);
-
-  oglrender_init = osmesa_init;
-  oglrender_beginOpenGL = osmesa_beginOpenGL;
-  oglrender_endOpenGL = osmesa_endOpenGL;
-
-  return 1;
+    return true;
 }
 #endif
