@@ -1,7 +1,7 @@
 /*
 	Copyright (C) 2006 thoduv
 	Copyright (C) 2006-2007 Theo Berkau
-	Copyright (C) 2008-2013 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,13 +17,16 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "mc.h"
+
 #include <stdlib.h>
 #include <string.h>
 
-#include "types.h"
+#include "common.h"
+#include "armcpu.h"
 #include "debug.h"
-#include "mc.h"
 #include "movie.h"
+#include "MMU.h"
 #include "readwrite.h"
 #include "NDSSystem.h"
 #include "path.h"
@@ -64,13 +67,6 @@
 #define MCLOG(...)
 #endif
 
-//since r2203 this was 0x00.
-//but baby pals proves finally that it should be 0xFF:
-//the game reads its initial sound volumes from uninitialized data, and if it is 0, the game will be silent
-//if it is 0xFF then the game starts with its sound and music at max, as presumably it is supposed to.
-//so in r3303 I finally changed it (no$ appears definitely to initialize to 0xFF)
-static u8 kUninitializedSaveDataValue = 0xFF;
-
 static const char* DESMUME_BACKUP_FOOTER_TXT = "|<--Snip above here to create a raw sav by excluding this DeSmuME savedata footer:";
 static const char* kDesmumeSaveCookie = "|-DESMUME SAVE-|";
 
@@ -126,7 +122,8 @@ bool BackupDevice::save_state(EMUFILE* os)
 	u32 savePos = fpMC->ftell();
 	std::vector<u8> data(fsize);
 	fpMC->fseek(0, SEEK_SET);
-	fread((char*)&data[0], 1, fsize, fpMC->get_fp());
+	if(data.size()!=0)
+		fpMC->fread((char *)&data[0], fsize);
 
 	u32 version = 5;
 	//v0
@@ -197,7 +194,8 @@ bool BackupDevice::load_state(EMUFILE* is)
 	fsize = data.size();
 #ifndef _DONT_SAVE_BACKUP
 	fpMC->fseek(0, SEEK_SET);
-	fwrite((char*)&data[0], 1, fsize, fpMC->get_fp());
+	if(data.size()!=0)
+		fpMC->fwrite((char *)&data[0], fsize);
 	ensure(data.size(), fpMC);
 #endif
 
@@ -219,12 +217,20 @@ BackupDevice::BackupDevice()
 	addr_size = 0;
 	isMovieMode = false;
 
+	//default for most games; will be altered where appropriate
+	//usually 0xFF, but occasionally others. If these exceptions could be related to a particular backup memory type, that would be helpful.
+	//at first we assumed it would be 0x00, but baby pals proved that it should be 0xFF:
+	// the game reads its initial sound volumes from uninitialized data, and if it is 0, the game will be silent
+	// if it is 0xFF then the game starts with its sound and music at max, as presumably it is supposed to.
+	// so in r3303 we finally changed it (no$ appears definitely to initialize to 0xFF)
+	uninitializedValue = 0xFF;
+
 	if (gameInfo.romsize == 0) return;
 
 	char buf[MAX_PATH] = {0};
 	memset(buf, 0, MAX_PATH);
 	path.getpathnoext(path.BATTERY, buf);
-	filename = std::string(buf) + ".dsv";						// DeSmuME memory card
+	filename = std::string(buf) + ".dsv";
 
 	MCLOG("MC: %s\n", filename.c_str());
 
@@ -243,19 +249,29 @@ BackupDevice::BackupDevice()
 				if (!out->fail())
 				{
 					u8 *data = new u8[sz];
-					fread(data, 1, sz, in->get_fp());
-					fwrite(data, 1, sz, out->get_fp());
+					in->fread(data, sz);
+					out->fwrite(data, sz);
 					delete [] data;
 				}
+				else
+				{
+					printf("BackupDevice: Could not create the backup save file.\n");
+				}
+				
 				delete out;
 			}
 		}
+		else
+		{
+			printf("BackupDevice: Could not read the save file for creating a backup.\n");
+		}
+		
 		delete in;
 	}
 
 	if (!fexists)
 	{
-		printf("DeSmuME .dsv save file not found. Trying to load an old raw .sav file.\n");
+		printf("BackupDevice: DeSmuME .dsv save file not found. Trying to load a .sav file.\n");
 		std::string tmp_fsav = std::string(buf) + ".sav";
 
 		EMUFILE_FILE *fpTmp = new EMUFILE_FILE(tmp_fsav, "rb");
@@ -269,14 +285,19 @@ BackupDevice::BackupDevice()
 				if (!fpOut->fail())
 				{
 					u8 *buf = new u8[sz + 1];
-					if ((buf) && (fread(buf, 1, sz, fpTmp->get_fp()) == sz))
+					if ((buf) && (fpTmp->fread(buf, sz) == sz))
 					{
 						if (no_gba_unpack(buf, sz))
-							printf("Converted from no$gba save.\n");
+						{
+							printf("BackupDevice: Converting no$gba .sav file.\n");
+						}
 						else
+						{
+							printf("BackupDevice: Converting old raw .sav file.\n");
 							sz = trim(buf, sz);
+						}
 
-						if (fwrite(buf, 1, sz, fpOut->get_fp()) == sz)
+						if (fpOut->fwrite(buf, sz) == sz)
 						{
 							u8 res = searchFileSaveType(sz);
 							if (res != 0xFF)
@@ -291,6 +312,10 @@ BackupDevice::BackupDevice()
 								info.type = 0;
 							fexists = true;
 						}
+						else
+						{
+							printf("BackupDevice: Error converting .sav file.\n");
+						}
 					}
 					delete [] buf;
 				}
@@ -301,6 +326,14 @@ BackupDevice::BackupDevice()
 	}
 
 	fpMC = new EMUFILE_FILE(filename, fexists?"rb+":"wb+");
+	const bool fileCanReadWrite = (fpMC->get_fp() != NULL);
+	if (!fileCanReadWrite)
+	{
+		delete fpMC;
+		fpMC = new EMUFILE_MEMORY();
+		printf("BackupDevice: WARNING! Failed to get read/write access to the save file! Will operate in RAM instead.\n");
+	}
+	
 	if (!fpMC->fail())
 	{
 		fsize = fpMC->size();
@@ -366,7 +399,7 @@ BackupDevice::BackupDevice()
 		}
 
 		if (ss > 0)
-			printf("Backup size: %u %cbit\n", ss, _Mbit?'M':'K');
+			printf("BackupDevice: size = %u %cbit\n", ss, _Mbit?'M':'K');
 	}
 
 	state = (fsize > 0)?RUNNING:DETECTING;
@@ -381,6 +414,12 @@ BackupDevice::~BackupDevice()
 
 int BackupDevice::readFooter()
 {
+	// Check if the footer data exists.
+	if (fpMC->size() < (strlen(kDesmumeSaveCookie) + strlen(DESMUME_BACKUP_FOOTER_TXT) + 24))
+	{
+		return -1;
+	}
+	
 	//scan for desmume save footer
 	const s32 cookieLen = (s32)strlen(kDesmumeSaveCookie);
 	char *sigbuf = new char[cookieLen];
@@ -485,7 +524,7 @@ bool  BackupDevice::write(u8 val)
 	//never use save files if we are in movie mode
 	if (isMovieMode) return true;
 
-	return fwrite(&val, 1, 1, fpMC->get_fp())?true:false;
+	return (fpMC->fwrite(&val, 1) == 1);
 }
 
 void BackupDevice::writeByte(u32 addr, u8 val)
@@ -523,6 +562,16 @@ void BackupDevice::writeLong(u32 val)
 	fpMC->write32le(val);
 }
 
+void BackupDevice::seek(u32 pos)
+{
+	fpMC->fseek(pos, SEEK_SET);
+}
+
+void BackupDevice::flushBackup()
+{
+	fpMC->fflush();
+}
+
 bool BackupDevice::saveBuffer(u8 *data, u32 size, bool _rewind, bool _truncate)
 {
 	if (_rewind)
@@ -532,7 +581,7 @@ bool BackupDevice::saveBuffer(u8 *data, u32 size, bool _rewind, bool _truncate)
 			fpMC->truncate(0);
 	}
 	fsize = size;
-	fwrite(data, 1, size, fpMC->get_fp());
+	fpMC->fwrite(data, size);
 	ensure(size, fpMC);
 	return true;
 }
@@ -553,10 +602,6 @@ void BackupDevice::reset_hardware()
 
 	motionInitState = MOTION_INIT_STATE_IDLE;
 	motionFlag = MOTION_FLAG_NONE;
-
-	kUninitializedSaveDataValue = 0xFF;
-
-	if(!memcmp(gameInfo.header.gameCode,"AXBJ", 4)) kUninitializedSaveDataValue = 0x00; // Daigassou! Band Brothers DX (JP)
 }
 
 void BackupDevice::reset()
@@ -714,18 +759,25 @@ u8 BackupDevice::data_command(u8 val, u8 PROCNUM)
 					addr_counter++;
 					val = 0xFF;
 
-					if (addr_counter >= addr_size)
-					{
-						//why does tomb raider underworld access 0x180 and go clear through to 0x280?
-						//should this wrap around at 0 or at 0x100?
-						if(addr_size == 1) addr &= 0x1FF; 
-
+					//if we just finished building the address, seek the FP
+					if (addr_counter == addr_size)
 						fpMC->fseek(addr, SEEK_SET);
-					}
 				}
 				else
 				{
-					ensure(addr);
+					//why does tomb raider underworld access 0x180 and go clear through to 0x280?
+					//should this wrap around at 0 or at 0x100?
+					//TODO - dont other backup memory types have similar wraparound issues?
+					if(addr_size == 1)
+					{
+						addr &= 0x1FF;
+						//since we changed the address unexpectedly (non-sequentially), be sure to seek the FP
+						fpMC->fseek(addr, SEEK_SET);
+					}
+
+					//if we're writing a byte at address 0, we need to ensure that we have at least 1 byte in the file (IOW, ensure to size addr+1 == 0+1)
+					ensure(addr+1);
+
 					if(com == BM_CMD_READLOW)
 					{
 						val = read();
@@ -867,16 +919,16 @@ u8 BackupDevice::data_command(u8 val, u8 PROCNUM)
 }
 
 //guarantees that the data buffer has room enough for the specified number of bytes
-void BackupDevice::ensure(u32 addr, EMUFILE_FILE *fpOut)
+void BackupDevice::ensure(u32 addr, EMUFILE *fpOut)
 {
-	ensure(addr, kUninitializedSaveDataValue, fpOut);
+	ensure(addr, uninitializedValue, fpOut);
 }
 
-void BackupDevice::ensure(u32 addr, u8 val, EMUFILE_FILE *fpOut)
+void BackupDevice::ensure(u32 addr, u8 val, EMUFILE *fpOut)
 {
 	if (!fpOut && (addr < fsize)) return;
 
-	EMUFILE_FILE *fp = fpOut?fpOut:fpMC;
+	EMUFILE *fp = fpOut?fpOut:fpMC;
 
 #ifndef _DONT_SAVE_BACKUP
 	fp->fseek(fsize, SEEK_SET);
@@ -893,7 +945,7 @@ void BackupDevice::ensure(u32 addr, u8 val, EMUFILE_FILE *fpOut)
 	{
 		u8 *tmp = new u8[size];
 		memset(tmp, val, size);
-		fwrite(tmp, 1, size, fp->get_fp());
+		fp->fwrite(tmp, size);
 		delete [] tmp;
 	}
 
@@ -911,7 +963,13 @@ void BackupDevice::ensure(u32 addr, u8 val, EMUFILE_FILE *fpOut)
 
 	fp->fflush();
 
-	fp->fseek(addr, SEEK_SET);
+	//this is a HORRIBLE IDEA.
+	//leave the FP positioned to write the final byte
+	//this is a HACK to make the basic read/write byte operation work when it calls ensure().
+	//IDEALLY, no assumptions about the file pointer can be made.
+	//but someone (actually, not really) so very carefully profiled the save IO code and discovered that not fseeking for every byte read/write was a great optimization.
+	//so, now all this code is depending/assuming on the FP being kept in a precise position, and I dont think its smart to change the main user of this assumption to paper over this bug by making it fseek before read/write, while leaving other unknown assuming clients intact
+	fpMC->fseek(addr-1, SEEK_SET);
 #endif
 }
 
@@ -972,7 +1030,8 @@ u32 BackupDevice::importDataSize(const char *filename)
 	u32 res = 0;
 	if (strlen(filename) < 4) return 0;
 
-	if (memcmp(filename + strlen(filename) - 4, ".duc", 4) == 0)
+	if ((memcmp(filename + strlen(filename) - 4, ".duc", 4) == 0) ||
+		(memcmp(filename + strlen(filename) - 4, ".dss", 4) == 0))
 	{
 		res = get_save_duc_size(filename);
 		if (res == 0xFFFFFFFF) return 0;
@@ -993,7 +1052,8 @@ bool BackupDevice::importData(const char *filename, u32 force_size)
 	bool res = false;
 	if (strlen(filename) < 4) return res;
 
-	if (memcmp(filename + strlen(filename) - 4, ".duc", 4) == 0)
+	if ((memcmp(filename + strlen(filename) - 4, ".duc", 4) == 0) ||
+		(memcmp(filename + strlen(filename) - 4, ".dss", 4) == 0))
 		res = import_duc(filename, force_size);
 	else
 		if (import_no_gba(filename, force_size))
@@ -1270,7 +1330,7 @@ bool BackupDevice::export_no_gba(const char* fname)
 	std::vector<u8> data(fsize);
 	u32 pos = fpMC->ftell();
 	fpMC->fseek(0, SEEK_SET);
-	fread((char*)&data[0], 1, fsize, fpMC->get_fp());
+	fpMC->fread((char *)&data[0], fsize);
 	fpMC->fseek(pos, SEEK_SET);
 
 	FILE* outf = fopen(fname,"wb");
@@ -1299,7 +1359,7 @@ bool BackupDevice::export_raw(const char* filename)
 	std::vector<u8> data(fsize);
 	u32 pos = fpMC->ftell();
 	fpMC->fseek(0, SEEK_SET);
-	fread((char*)&data[0], 1, fsize, fpMC->get_fp());
+	fpMC->fread((char *)&data[0], fsize);
 	fpMC->fseek(pos, SEEK_SET);
 
 	FILE* outf = fopen(filename,"wb");
@@ -1309,7 +1369,7 @@ bool BackupDevice::export_raw(const char* filename)
 	if(data.size()>0)
 		fwrite(&data[0],1,size,outf);
 	for(u32 i=size;i<padSize;i++)
-		fputc(kUninitializedSaveDataValue,outf);
+		fputc(uninitializedValue,outf);
 	fclose(outf);
 
 	return true;
@@ -1498,7 +1558,7 @@ bool BackupDevice::load_movie(EMUFILE* is) {
 	is->fread((char*)&info.mem_size,4);
 
 	is->fseek(0, SEEK_SET);
-	fpMC = (EMUFILE_FILE*)&is;
+	fpMC = (EMUFILE*)&is;
 
 	state = RUNNING;
 	addr_size = info.addr_size;
