@@ -1,7 +1,7 @@
 /*
 	Copyright (C) 2006 thoduv
 	Copyright (C) 2006 Theo Berkau
-	Copyright (C) 2008-2013 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -23,10 +23,8 @@
 #include <stdio.h>
 #include <vector>
 #include <string>
+
 #include "types.h"
-#include "emufile.h"
-#include "common.h"
-#include "utils/tinyxml/tinyxml.h"
 
 #define MAX_SAVE_TYPES 13
 #define MC_TYPE_AUTODETECT      0x0
@@ -50,6 +48,8 @@
 #define MC_SIZE_256MBITS                0x2000000
 #define MC_SIZE_512MBITS                0x4000000
 
+class EMUFILE;
+
 //This "backup device" represents a typical retail NDS save memory accessible via AUXSPI.
 //It is managed as a core emulator service for historical reasons which are bad,
 //and possible infrastructural simplification reasons which are good.
@@ -84,9 +84,9 @@ public:
 	void writeWord(u16 val);
 	void writeLong(u32 val);
 
-	void seek(u32 pos) { fpMC->fseek(pos, SEEK_SET); }
+	void seek(u32 pos);
 
-	void flushBackup() { fpMC->fflush(); }
+	void flushBackup();
 	
 	u8 searchFileSaveType(u32 size);
 
@@ -103,8 +103,8 @@ public:
 		u32 addr_size;
 	} savedInfo;
 
-	void ensure(u32 addr, EMUFILE_FILE *fpOut = NULL);
-	void ensure(u32 addr, u8 val, EMUFILE_FILE *fpOut = NULL);
+	void ensure(u32 addr, EMUFILE *fpOut = NULL);
+	void ensure(u32 addr, u8 val, EMUFILE *fpOut = NULL);
 
 	//and these are used by old savestates
 	void load_old_state(u32 addr_size, u8* data, u32 datasize);
@@ -140,8 +140,11 @@ public:
 	bool importData(const char *filename, u32 force_size = 0);
 	bool exportData(const char *filename);
 
+	//the value contained in memory when shipped from factory (before user program ever writes to it). more details commented elsewhere.
+	u8 uninitializedValue;
+
 private:
-	EMUFILE_FILE *fpMC;
+	EMUFILE *fpMC;
 	std::string filename;
 	u32	fsize;
 	int readFooter();
@@ -155,7 +158,7 @@ private:
 	u32 addr_size, addr_counter;
 	u32 addr;
 	u8 write_protect;
-	
+
 	std::vector<u8> data_autodetect;
 	enum STATE {
 		DETECTING = 0, RUNNING = 1
