@@ -1,5 +1,5 @@
 /*
-	Copyright (C) 2009-2013 DeSmuME team
+	Copyright (C) 2009-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -33,6 +33,14 @@
 #include <math.h>
 #include <string.h>
 
+#if defined(_MSC_VER) && _MSC_VER == 1600
+#define SLEEP_HACK_2011
+#endif
+
+#ifdef SLEEP_HACK_2011
+#include <Windows.h>
+#endif
+
 #ifndef _MSC_VER 
 #include <stdint.h>
 #endif
@@ -43,6 +51,7 @@
 #include "render3D.h"
 #include "gfx3d.h"
 #include "texcache.h"
+#include "MMU.h"
 #include "NDSSystem.h"
 #include "utils/task.h"
 
@@ -115,8 +124,8 @@ static FORCEINLINE int fastFloor(float f)
 //	verts[vert_index] = &rawvert;
 //}
 
-static Fragment _screen[256*192];
-static FragmentColor _screenColor[256*192];
+static Fragment _screen[GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT];
+static FragmentColor _screenColor[GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT];
 
 static FORCEINLINE int iround(float f) {
 	return (int)f; //lol
@@ -407,7 +416,7 @@ public:
 
 		bool enabled;
 		int width, height;
-		int wmask, hmask;
+		s32 wmask, hmask;
 		int wrap;
 		int wshift;
 		int texFormat;
@@ -423,27 +432,27 @@ public:
 			enabled = gfx3d.renderState.enableTexturing && (texFormat!=0);
 		}
 
-		FORCEINLINE void clamp(int &val, const int size, const int sizemask){
+		FORCEINLINE void clamp(s32 &val, const int size, const s32 sizemask){
 			if(val<0) val = 0;
 			if(val>sizemask) val = sizemask;
 		}
-		FORCEINLINE void hclamp(int &val) { clamp(val,width,wmask); }
-		FORCEINLINE void vclamp(int &val) { clamp(val,height,hmask); }
+		FORCEINLINE void hclamp(s32 &val) { clamp(val,width,wmask); }
+		FORCEINLINE void vclamp(s32 &val) { clamp(val,height,hmask); }
 
-		FORCEINLINE void repeat(int &val, const int size, const int sizemask) {
+		FORCEINLINE void repeat(s32 &val, const int size, const s32 sizemask) {
 			val &= sizemask;
 		}
-		FORCEINLINE void hrepeat(int &val) { repeat(val,width,wmask); }
-		FORCEINLINE void vrepeat(int &val) { repeat(val,height,hmask); }
+		FORCEINLINE void hrepeat(s32 &val) { repeat(val,width,wmask); }
+		FORCEINLINE void vrepeat(s32 &val) { repeat(val,height,hmask); }
 
-		FORCEINLINE void flip(int &val, const int size, const int sizemask) {
+		FORCEINLINE void flip(s32 &val, const int size, const s32 sizemask) {
 			val &= ((size<<1)-1);
 			if(val>=size) val = (size<<1)-val-1;
 		}
-		FORCEINLINE void hflip(int &val) { flip(val,width,wmask); }
-		FORCEINLINE void vflip(int &val) { flip(val,height,hmask); }
+		FORCEINLINE void hflip(s32 &val) { flip(val,width,wmask); }
+		FORCEINLINE void vflip(s32 &val) { flip(val,height,hmask); }
 
-		FORCEINLINE void dowrap(int& iu, int& iv)
+		FORCEINLINE void dowrap(s32 &iu, s32 &iv)
 		{
 			switch(wrap) {
 				//flip none
@@ -477,15 +486,35 @@ public:
 
 		//finally, we can use floor here. but, it is slower than we want.
 		//the best solution is probably to wait until the pipeline is full of fixed point
-		s32 iu = s32floor(u);
-		s32 iv = s32floor(v);
-		sampler.dowrap(iu,iv);
-
+		s32 iu = 0;
+		s32 iv = 0;
+		
+		if (!CommonSettings.GFX3D_TXTHack)
+		{
+			iu = s32floor(u);
+			iv = s32floor(v);
+		}
+		else
+		{
+			iu = round_s(u);
+			iv = round_s(v);
+		}
+		
+		sampler.dowrap(iu, iv);
 		FragmentColor color;
 		color.color = ((u32*)lastTexKey->decoded)[(iv<<sampler.wshift)+iu];
 		return color;
 	}
 
+	//round function - tkd3
+	float round_s(double val){
+		if (val > 0.0 ){
+			return floorf(val*256.0f+0.5f)/256.0f; //this value(256.0) is good result.(I think)
+		} else {
+			return -1.0*floorf(fabs(val)*256.0f+0.5f)/256.0f;
+		}
+	}
+
 	struct Shader
 	{
 		u8 mode;
@@ -796,14 +825,14 @@ public:
 			width -= -x;
 			x = 0;
 		}
-		if(x+width > (RENDERER?256:engine->width))
+		if(x+width > (RENDERER?GFX3D_FRAMEBUFFER_WIDTH:engine->width))
 		{
 			if(RENDERER && !lineHack)
 			{
 				printf("rasterizer rendering at x=%d! oops!\n",x+width-1);
 				return;
 			}
-			width = (RENDERER?256:engine->width)-x;
+			width = (RENDERER?GFX3D_FRAMEBUFFER_WIDTH:engine->width)-x;
 		}
 
 		while(width-- > 0)
@@ -833,7 +862,7 @@ public:
 		bool first=true;
 
 		//HACK: special handling for horizontal line poly
-		if (lineHack && left->Height == 0 && right->Height == 0 && left->Y<192 && left->Y>=0)
+		if (lineHack && left->Height == 0 && right->Height == 0 && left->Y<GFX3D_FRAMEBUFFER_HEIGHT && left->Y>=0)
 		{
 			bool draw = (!SLI || (left->Y & SLI_MASK) == SLI_VALUE);
 			if(draw) drawscanline(left,right,lineHack);
@@ -940,7 +969,7 @@ public:
 			// hack for VC++ 2010 (bug in compiler optimization?)
 			// freeze on 3D
 			// TODO: study it
-			#if defined(_MSC_VER) && _MSC_VER == 1600
+			#ifdef SLEEP_HACK_2011
 				Sleep(0); // nop
 			#endif
 		}
@@ -1196,7 +1225,7 @@ static void SoftRastVramReconfigureSignal()
 
 static void SoftRastConvertFramebuffer()
 {
-	memcpy(gfx3d_convertedScreen,_screenColor,256*192*4);
+	memcpy(gfx3d_convertedScreen, _screenColor, GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT*4);
 }
 
 void SoftRasterizerEngine::initFramebuffer(const int width, const int height, const bool clearImage)
@@ -1225,7 +1254,7 @@ void SoftRasterizerEngine::initFramebuffer(const int width, const int height, co
 	if(clearImage)
 	{
 		//need to handle this somehow..
-		assert(width==256 && height==192);
+		assert(width==GFX3D_FRAMEBUFFER_WIDTH && height==GFX3D_FRAMEBUFFER_HEIGHT);
 
 		u16* clearImage = (u16*)MMU.texInfo.textureSlotAddr[2];
 		u16* clearDepth = (u16*)MMU.texInfo.textureSlotAddr[3];
@@ -1239,9 +1268,9 @@ void SoftRasterizerEngine::initFramebuffer(const int width, const int height, co
 		FragmentColor *dstColor = screenColor;
 		Fragment *dst = screen;
 
-		for(int iy=0;iy<192;iy++) {
+		for(int iy=0; iy<GFX3D_FRAMEBUFFER_HEIGHT; iy++) {
 			int y = ((iy + yscroll)&255)<<8;
-			for(int ix=0;ix<256;ix++) {
+			for(int ix=0; ix<GFX3D_FRAMEBUFFER_WIDTH; ix++) {
 				int x = (ix + xscroll)&255;
 				int adr = y + x;
 				
@@ -1376,9 +1405,9 @@ void SoftRasterizerEngine::framebufferProcess()
 			edgeMarkDisabled[i] = 0;
 		}
 
-		for(int i=0,y=0;y<192;y++)
+		for(int i=0,y=0; y<GFX3D_FRAMEBUFFER_HEIGHT; y++)
 		{
-			for(int x=0;x<256;x++,i++)
+			for(int x=0; x<GFX3D_FRAMEBUFFER_WIDTH; x++,i++)
 			{
 				Fragment destFragment = screen[i];
 				u8 self = destFragment.polyid.opaque;
@@ -1392,8 +1421,8 @@ void SoftRasterizerEngine::framebufferProcess()
 
 				FragmentColor edgeColor = edgeMarkColors[self>>3];
 
-#define PIXOFFSET(dx,dy) ((dx)+(256*(dy)))
-#define ISEDGE(dx,dy) ((x+(dx)!=256) && (x+(dx)!=-1) && (y+(dy)!=192) && (y+(dy)!=-1) && self > screen[i+PIXOFFSET(dx,dy)].polyid.opaque)
+#define PIXOFFSET(dx,dy) ((dx)+(GFX3D_FRAMEBUFFER_WIDTH*(dy)))
+#define ISEDGE(dx,dy) ((x+(dx)!=GFX3D_FRAMEBUFFER_WIDTH) && (x+(dx)!=-1) && (y+(dy)!=GFX3D_FRAMEBUFFER_HEIGHT) && (y+(dy)!=-1) && self > screen[i+PIXOFFSET(dx,dy)].polyid.opaque)
 #define DRAWEDGE(dx,dy) alphaBlend(screenColor[i+PIXOFFSET(dx,dy)], edgeColor)
 
 				bool upleft    = ISEDGE(-1,-1);
@@ -1436,7 +1465,7 @@ void SoftRasterizerEngine::framebufferProcess()
 		u32 g = GFX3D_5TO6((gfx3d.renderState.fogColor>>5)&0x1F);
 		u32 b = GFX3D_5TO6((gfx3d.renderState.fogColor>>10)&0x1F);
 		u32 a = (gfx3d.renderState.fogColor>>16)&0x1F;
-		for(int i=0;i<256*192;i++)
+		for(int i=0; i<GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT; i++)
 		{
 			Fragment &destFragment = screen[i];
 			if(!destFragment.fogged) continue;
@@ -1456,7 +1485,7 @@ void SoftRasterizerEngine::framebufferProcess()
 	}
 
 	////debug alpha channel framebuffer contents
-	//for(int i=0;i<256*192;i++)
+	//for(int i=0;i<GFX3D_FRAMEBUFFER_WIDTH*GFX3D_FRAMEBUFFER_HEIGHT;i++)
 	//{
 	//	FragmentColor &destFragmentColor = screenColor[i];
 	//	destFragmentColor.r = destFragmentColor.a;
@@ -1491,10 +1520,10 @@ void SoftRasterizerEngine::performClipping(bool hirez)
 
 template<bool CUSTOM> void SoftRasterizerEngine::performViewportTransforms(int width, int height)
 {
-	const float xfactor = width/256.0f;
-	const float yfactor = height/192.0f;
-	const float xmax = 256.0f*xfactor-(CUSTOM?0.001f:0); //fudge factor to keep from overrunning render buffers
-	const float ymax = 192.0f*yfactor-(CUSTOM?0.001f:0);
+	const float xfactor = (float)width/GFX3D_FRAMEBUFFER_WIDTH;
+	const float yfactor = (float)height/GFX3D_FRAMEBUFFER_HEIGHT;
+	const float xmax = GFX3D_FRAMEBUFFER_WIDTH*xfactor-(CUSTOM?0.001f:0); //fudge factor to keep from overrunning render buffers
+	const float ymax = GFX3D_FRAMEBUFFER_HEIGHT*yfactor-(CUSTOM?0.001f:0);
 
 
 	//viewport transforms
@@ -1647,18 +1676,18 @@ static void SoftRastRender()
 	mainSoftRasterizer.indexlist = &gfx3d.indexlist;
 	mainSoftRasterizer.screen = _screen;
 	mainSoftRasterizer.screenColor = _screenColor;
-	mainSoftRasterizer.width = 256;
-	mainSoftRasterizer.height = 192;
+	mainSoftRasterizer.width = GFX3D_FRAMEBUFFER_WIDTH;
+	mainSoftRasterizer.height = GFX3D_FRAMEBUFFER_HEIGHT;
 
 	//setup fog variables (but only if fog is enabled)
 	if(gfx3d.renderState.enableFog)
 		mainSoftRasterizer.updateFogTable();
 	
-	mainSoftRasterizer.initFramebuffer(256,192,gfx3d.renderState.enableClearImage?true:false);
+	mainSoftRasterizer.initFramebuffer(GFX3D_FRAMEBUFFER_WIDTH, GFX3D_FRAMEBUFFER_HEIGHT, gfx3d.renderState.enableClearImage?true:false);
 	mainSoftRasterizer.updateToonTable();
 	mainSoftRasterizer.updateFloatColors();
 	mainSoftRasterizer.performClipping(CommonSettings.GFX3D_HighResolutionInterpolateColor);
-	mainSoftRasterizer.performViewportTransforms<false>(256,192);
+	mainSoftRasterizer.performViewportTransforms<false>(GFX3D_FRAMEBUFFER_WIDTH, GFX3D_FRAMEBUFFER_HEIGHT);
 	mainSoftRasterizer.performBackfaceTests();
 	mainSoftRasterizer.performCoordAdjustment(true);
 	mainSoftRasterizer.setupTextures(true);
