@@ -146,7 +146,7 @@
 #define REG_ENCSEED0L                           0x040001B0
 #define REG_ENCSEED1L                           0x040001B4
 #define REG_ENCSEED0H                           0x040001B8
-#define REG_ENCSEED1H                           0x040001BC
+#define REG_ENCSEED1H                           0x040001BA
 #define REG_SPICNT                              0x040001C0
 #define REG_SPIDATA                             0x040001C2
 
@@ -428,4 +428,13 @@
 #define EXMEMCNT_MASK_SLOT2_ROM_2ND_TIME (1<<4)
 #define EXMEMCNT_MASK_SLOT2_CLOCKRATE (3<<5)
 
+#define SPI_DEVICE_POWERMAN 0
+#define SPI_DEVICE_FIRMWARE 1
+#define SPI_DEVICE_TOUCHSCREEN 2
+
+#define SPI_BAUDRATE_4MHZ 0
+#define SPI_BAUDRATE_2MHZ 1
+#define SPI_BAUDRATE_1MHZ 2
+#define SPI_BAUDRATE_512KHZ 3
+
 #endif
