@@ -1,21 +1,20 @@
-/*  Copyright (C) 2006 Normmatt
-    Copyright (C) 2007 Pascal Giard
+/*
+	Copyright (C) 2006 Normmatt
+	Copyright (C) 2007 Pascal Giard
+	Copyright (C) 2007-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _SRAM_H
@@ -23,9 +22,7 @@
 
 #include "types.h"
 
-#define SRAM_ADDRESS	0x0A000000
-#define SRAM_SIZE	0x10000
-#define NB_STATES       10
+#define NB_STATES 10
 
 extern int lastSaveState;
 
