@@ -1,7 +1,7 @@
 /*
 	Copyright (C) 2006 yopyop
 	Copyright (C) 2006-2007 shash
-	Copyright (C) 2008-2009 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@
 #ifndef _TEXCACHE_H_
 #define _TEXCACHE_H_
 
-#include "common.h"
 #include <map>
 
+#include "types.h"
+
 enum TexCache_TexFormat
 {
 	TexFormat_None, //used when nothing yet is cached
