@@ -1,7 +1,7 @@
 /*
 	Copyright (C) 2006 yopyop
 	Copyright (C) 2008 shash
-	Copyright (C) 2008-2013 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,12 +17,16 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "armcpu.h"
+#include "instructions.h"
+
+#include <assert.h>
+
 #include "bios.h"
 #include "debug.h"
 #include "MMU.h"
 #include "NDSSystem.h"
 #include "MMU_timing.h"
-#include <assert.h>
 
 #define cpu (&ARMPROC)
 #define TEMPLATE template<int PROCNUM> 
