@@ -1,6 +1,6 @@
 /*
 	Copyright (C) 2005 Guillaume Duhamel
-	Copyright (C) 2008-2012 DeSmuME team
+	Copyright (C) 2008-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -24,7 +24,9 @@
 	#define HOST_WINDOWS
 	
 	//todo - everyone will want to support this eventually, i suppose
-	#include "config.h"
+	#ifndef DESMUME_QT
+		#include "config.h"
+	#endif
 
 #endif //_MSC_VER
 
@@ -83,15 +85,20 @@
 #define WINAPI
 #endif
 
-#ifdef __GNUC__
-#include <limits.h>
-#ifndef PATH_MAX
-#define MAX_PATH 1024
-#else
-#define MAX_PATH PATH_MAX
+#if !defined(MAX_PATH)
+	#if defined(HOST_WINDOWS)
+		#define MAX_PATH 260
+	#elif defined(__GNUC__)
+		#include <limits.h>
+		#if !defined(PATH_MAX)
+			#define MAX_PATH 1024
+		#else
+			#define MAX_PATH PATH_MAX
+		#endif
+	#else
+		#define MAX_PATH 1024
+	#endif
 #endif
-#endif
-
 
 //------------alignment macros-------------
 //dont apply these to types without further testing. it only works portably here on declarations of variables
@@ -256,10 +263,10 @@ typedef int desmume_BOOL;
 #ifdef LOCAL_BE	/* local arch is big endian */
 # define LE_TO_LOCAL_16(x) ((((x)&0xff)<<8)|(((x)>>8)&0xff))
 # define LE_TO_LOCAL_32(x) ((((x)&0xff)<<24)|(((x)&0xff00)<<8)|(((x)>>8)&0xff00)|(((x)>>24)&0xff))
-# define LE_TO_LOCAL_64(x) ((((x)&0xff)<<56)|(((x)&0xff00)<<40)|(((x)&0xff0000)<<24)|(((x)&0xff000000)<<8)|(((x)>>8)&0xff000000)|(((x)>>24)&0xff00)|(((x)>>40)&0xff00)|(((x)>>56)&0xff))
+# define LE_TO_LOCAL_64(x) ((((x)&0xff)<<56)|(((x)&0xff00)<<40)|(((x)&0xff0000)<<24)|(((x)&0xff000000)<<8)|(((x)>>8)&0xff000000)|(((x)>>24)&0xff0000)|(((x)>>40)&0xff00)|(((x)>>56)&0xff))
 # define LOCAL_TO_LE_16(x) ((((x)&0xff)<<8)|(((x)>>8)&0xff))
 # define LOCAL_TO_LE_32(x) ((((x)&0xff)<<24)|(((x)&0xff00)<<8)|(((x)>>8)&0xff00)|(((x)>>24)&0xff))
-# define LOCAL_TO_LE_64(x) ((((x)&0xff)<<56)|(((x)&0xff00)<<40)|(((x)&0xff0000)<<24)|(((x)&0xff000000)<<8)|(((x)>>8)&0xff000000)|(((x)>>24)&0xff00)|(((x)>>40)&0xff00)|(((x)>>56)&0xff))
+# define LOCAL_TO_LE_64(x) ((((x)&0xff)<<56)|(((x)&0xff00)<<40)|(((x)&0xff0000)<<24)|(((x)&0xff000000)<<8)|(((x)>>8)&0xff000000)|(((x)>>24)&0xff0000)|(((x)>>40)&0xff00)|(((x)>>56)&0xff))
 #else		/* local arch is little endian */
 # define LE_TO_LOCAL_16(x) (x)
 # define LE_TO_LOCAL_32(x) (x)
