@@ -66,16 +66,16 @@
 /*
  * DetectRomType
  */
-int DetectRomType(const Header& header, char* secure)
+int DetectRomType(const Header &header, char *secure)
 {
-	unsigned int * data = (unsigned int*)(secure);
-
+	const u64 data = LE_TO_LOCAL_64(*(u64 *)secure);
+	
 	//this is attempting to check for an utterly invalid nds header
-	if(header.unitcode < 0 && header.unitcode > 3) return ROMTYPE_INVALID;
-
+	if(header.unitcode > 3) return ROMTYPE_INVALID;
+	
 	if (header.arm9_rom_offset < 0x4000) return ROMTYPE_HOMEBREW;
-	if (data[0] == 0x00000000 && data[1] == 0x00000000) return ROMTYPE_MULTIBOOT;
-	if (data[0] == 0xE7FFDEFF && data[1] == 0xE7FFDEFF) return ROMTYPE_NDSDUMPED;
+	else if (data == 0x0000000000000000ULL) return ROMTYPE_MULTIBOOT;
+	else if (data == 0xE7FFDEFFE7FFDEFFULL) return ROMTYPE_NDSDUMPED;
 	//TODO
 	//for (int i=0x200; i<0x4000; i++)
 	//	if (romdata[i]) return ROMTYPE_MASKROM;	// found something odd ;)
