@@ -1,5 +1,5 @@
 /*
-	Copyright 2009-2010 DeSmuME team
+	Copyright 2009-2015 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -32,9 +32,10 @@
 #define EMUFAT_H
 
 #include "emufat_types.h"
-#include "emufile.h"
 #include <stdio.h>
 
+class EMUFILE;
+
 #define BOOTCODE_SIZE		448
 #define BOOTCODE_FAT32_SIZE	420
 
