@@ -42,7 +42,10 @@
 #include "filetime.h"
 
 #ifdef __APPLE__
-//apple doesn't provide strnlen. how unkind
+#include <AvailabilityMacros.h>
+
+#ifndef MAC_OS_X_VERSION_10_7
+// In Mac OS X, strnlen() is unsupported prior to v10.7, so define it here.
 static size_t strnlen(const char *s, size_t n)
 {
 	const char *p = (const char *)memchr(s, 0, n);
@@ -50,6 +53,8 @@ static size_t strnlen(const char *s, size_t n)
 }
 #endif
 
+#endif // __APPLE__
+
 // Directory entry codes
 #define DIR_ENTRY_LAST 0x00
 #define DIR_ENTRY_FREE 0xE5
