@@ -1,6 +1,23 @@
 //taken from fceux on 27-oct-2008
 //subsequently modified for desmume
 
+/*
+	Copyright (C) 2008-2009 DeSmuME team
+
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
+
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 #ifndef _STRINGUTIL_H_
 #define _STRINGUTIL_H_
 
@@ -12,7 +29,7 @@
 #include <cstdio>
 
 #include "../types.h"
-#include "emufile.h"
+#include "../emufile.h"
 
 
 //definitions for str_strip() flags
